<?php

class Jackmail_Authentification {

	public function __construct( Jackmail_Core $core ) {

		$this->core = $core;

		if ( $this->core->is_admin() ) {

			add_action( 'wp_ajax_jackmail_account_creation', array( $this, 'account_creation_callback' ) );

			add_action( 'wp_ajax_jackmail_account_connection', array( $this, 'account_connection_callback' ) );

			add_action( 'wp_ajax_jackmail_account_reset', array( $this, 'account_reset_callback' ) );

			add_action( 'wp_ajax_jackmail_account_resend_activation_email', array( $this, 'account_resend_activation_email_callback' ) );

		}

	}

	public function account_creation_callback() {
		$this->core->check_auth();
		if ( isset( $_POST['email'] ) && isset( $_POST['password'] ) && isset( $_POST['firstname'] ) && isset( $_POST['lastname'] ) ) {
			$email     = $this->core->request_email_data( $_POST['email'] );
			$password  = $this->core->request_text_data( $_POST['password'] );
			$firstname = $this->core->request_text_data( $_POST['firstname'] );
			$lastname  = $this->core->request_text_data( $_POST['lastname'] );
			$result    = $this->account_creation( $email, $password, $firstname, $lastname );
			if ( $result ) {
				wp_send_json_success();
			}
		}
		wp_send_json_error();
		die;
	}

	public function account_connection_callback() {
		$this->core->check_auth();
		$json = array(
			'success' => false,
			'message' => ''
		);
		if ( isset( $_POST['email'] ) && isset( $_POST['password'] ) ) {
			$email    = $this->core->request_email_data( $_POST['email'] );
			$password = $this->core->request_text_data( $_POST['password'] );
			$json     = $this->account_connection( $email, $password );
		}
		wp_send_json( $json );
		die;
	}

	public function account_reset_callback() {
		$this->core->check_auth();
		if ( isset( $_POST['email'] ) ) {
			$email  = $this->core->request_email_data( $_POST['email'] );
			$result = $this->account_reset( $email );
			if ( $result ) {
				wp_send_json_success();
			}
		}
		wp_send_json_error();
		die;
	}

	public function account_resend_activation_email_callback() {
		$this->core->check_auth();
		if ( isset( $_POST['email'] ) && isset( $_POST['password'] ) ) {
			$email    = $this->core->request_email_data( $_POST['email'] );
			$password = $this->core->request_text_data( $_POST['password'] );
			$result   = $this->account_resend_activation_email( $email, $password );
			if ( $result ) {
				wp_send_json_success();
			}
		}
		wp_send_json_error();
		die;
	}

	private function account_creation( $email, $password, $firstname, $lastname ) {
		if ( is_email( $email ) ) {
			$url      = $this->core->get_jackmail_url_identity() . 'users';
			$headers  = array(
				'content-type' => 'application/json'
			);
			$body     = array(
				'email'       => $email,
				'password'    => $password,
				'firstName'   => $firstname,
				'lastName'    => $lastname,
				'locale'      => $this->core->get_current_language(),
				'url'         => get_home_url(),
				'accountType' => 'JACKMAIL'
			);
			$timeout  = 30;
			$response = $this->core->remote_post( $url, $headers, $body, $timeout );
			if ( is_array( $response ) ) {
				if ( isset( $response['body'] ) ) {
					$data = json_decode( $response['body'], true );
					if ( isset( $data['email'] ) ) {
						$url  = $this->core->get_jackmail_url_ws() . 'create-user.php';
						$body = array(
							'email' => $email
						);
						$this->core->remote_post( $url, $headers, $body, $timeout );
						return true;
					}
				}
			}
		}
		return false;
	}

	private function account_connection( $email, $password ) {
		$json = array(
			'success' => false,
			'message' => ''
		);
		if ( is_email( $email ) && $password !== '' ) {
			$url      = $this->core->get_jackmail_url_identity() . 'authenticate';
			$headers  = array(
				'content-type' => 'application/json'
			);
			$body     = array(
				'email'    => $email,
				'password' => $password
			);
			$timeout  = 30;
			$response = $this->core->remote_post( $url, $headers, $body, $timeout );
			if ( is_array( $response ) ) {
				if ( isset( $response['response'] ) ) {
					if ( is_array( $response['response'] ) ) {
						if ( isset( $response['response']['code'] ) ) {
							if ( $response['response']['code'] === 403 ) {
								$json['message'] = 'NOT_ACTIVED';
							} else {
								if ( isset( $response['body'] ) ) {
									$data = json_decode( $response['body'], true );
									if ( isset( $data['token'] ) && isset( $data['accounts'] ) && isset( $data['lastName'] ) && isset( $data['firstName'] ) ) {
										$token     = $data['token'];
										$accounts  = $data['accounts'];
										$user_id   = $data['email'];
										$lastname  = $data['lastName'];
										$firstname = $data['firstName'];
										if ( is_array( $accounts ) ) {
											if ( count( $accounts ) > 0 ) {
												if ( isset( $accounts[0]['accountId'] ) ) {
													$account_id          = $accounts[0]['accountId'];
													$current_acccount_id = $this->core->get_account_id();
													$current_user_id     = $this->core->get_user_id();
													if ( ( $current_acccount_id === '' && $current_user_id === '' ) || ( $current_acccount_id === $account_id && $current_user_id === $user_id ) ) {
														$this->core->set_account_token( $token );
														if ( $current_acccount_id === '' ) {
															$this->core->set_account_id( $account_id );
														}
														if ( $current_user_id === '' ) {
															$this->core->set_user_id( $user_id );
														}
														$this->core->set_lastname( $lastname );
														$this->core->set_firstname( $firstname );
														$json = array(
															'success' => true,
															'message' => 'OK'
														);
														update_option( 'jackmail_authentification_failed', '0' );
														$this->core->cron_domain();
													}
												}
											}
										}
									}
								}
							}
						}
					}
				}
			}
		}
		return $json;
	}

	private function account_reset( $email ) {
		if ( is_email( $email ) ) {
			$url     = $this->core->get_jackmail_url_identity() . 'users/' . urlencode( $email ) . '/reset';
			$headers = array(
				'content-type' => 'application/json'
			);
			$body    = array();
			$timeout = 30;
			$this->core->remote_post( $url, $headers, $body, $timeout );
			return true;
		}
		return false;
	}

	public function account_resend_activation_email( $email, $password ) {
		if ( is_email( $email ) ) {
			$url     = $this->core->get_jackmail_url_identity() . 'users/reactivate';
			$headers = array(
				'content-type' => 'application/json'
			);
			$body    = array(
				'accountType' => 'jackmail',
				'email'       => $email,
				'password'    => $password
			);
			$timeout = 30;
			$this->core->remote_post( $url, $headers, $body, $timeout );
			return true;
		}
		return false;
	}

}