<?php


class Jackmail_Emailcontent extends Jackmail_Emailcontent_Common {

	public function __construct( Jackmail_Core $core ) {

		$this->core = $core;

		if ( $this->core->is_admin() ) {

			add_action( 'wp_ajax_jackmail_get_images', array( $this, 'get_images_callback' ) );

			add_action( 'wp_ajax_jackmail_get_post_categories', array( $this, 'get_post_categories_callback' ) );

			add_action( 'wp_ajax_jackmail_get_posts', array( $this, 'get_posts_callback' ) );

			add_action( 'wp_ajax_jackmail_get_post_or_page_full_content', array( $this, 'get_post_or_page_full_content_callback' ) );

			add_action( 'wp_ajax_jackmail_get_pages', array( $this, 'get_pages_callback' ) );

			add_action( 'wp_ajax_jackmail_get_woocommerce_product_categories', array( $this, 'get_woocommerce_product_categories_callback' ) );

			add_action( 'wp_ajax_jackmail_get_woocommerce_products', array( $this, 'get_products_woocommerce_callback' ) );

			add_action( 'wp_ajax_jackmail_get_woocommerce_product_full_content', array( $this, 'get_woocommerce_product_full_content_callback' ) );

		}

	}

	public function get_images_callback() {
		$this->core->check_auth();
		$wp_paths          = wp_upload_dir();
		$baseurl           = $wp_paths['baseurl'] . '/';
		$query_images_args = array(
			'post_type'      => 'attachment',
			'post_mime_type' => 'image',
			'post_status'    => 'inherit',
			'posts_per_page' => - 1,
		);
		$query_images      = new WP_Query( $query_images_args );
		$json              = array(
			'libraries' => array(),
			'images'    => array()
		);
		$years             = array();
		$months            = array();
		foreach ( $query_images->posts as $image ) {
			$dir      = str_replace( $baseurl, '', wp_get_attachment_thumb_url( $image->ID ) );
			$year     = substr( $dir, 0, 4 );
			$month    = substr( $dir, 5, 2 );
			$filename = substr( $dir, 8 );
			if ( ! in_array( $year, $years ) ) {
				$years[]             = $year;
				$json['libraries'][] = array(
					'libraryLabel' => $year,
					'filename'     => $year,
					'libraries'    => array(),
					'images'       => array()
				);
			}
			$year_key  = - 1;
			$month_key = - 1;
			foreach ( $json['libraries'] as $key => $library ) {
				if ( $library['libraryLabel'] === $year ) {
					$year_key = $key;
					break;
				}
			}
			if ( ! in_array( $month, $months ) ) {
				$months[]                                      = $month;
				$json['libraries'][ $year_key ]['libraries'][] = array(
					'libraryLabel' => $month,
					'filename'     => $month,
					'libraries'    => array(),
					'images'       => array()
				);
			}
			foreach ( $json['libraries'][ $year_key ]['libraries'] as $key => $library ) {
				if ( $library['libraryLabel'] === $month ) {
					$month_key = $key;
					break;
				}
			}
			if ( $year_key !== - 1 && $month_key !== - 1 ) {
				$json['libraries'][ $year_key ]['libraries'][ $month_key ]['images'][] = array(
					'name'     => $filename,
					'filename' => $filename
				);
			}
		}
		if ( count( $json['libraries'] ) === 0 ) {
			$year                = date( "Y" );
			$month               = date( "m" );
			$json['libraries'][] = array(
				'libraryLabel' => $year,
				'filename'     => $year,
				'libraries'    => array(
					array(
						'libraryLabel' => $month,
						'filename'     => $month,
						'libraries'    => array(),
						'images'       => array()
					)
				),
				'images'       => array()
			);
		}
		wp_send_json( $json );
		die;
	}

	public function get_post_categories_callback() {
		$this->core->check_auth();
		$json       = array();
		$args       = array(
			'hide_empty' => false
		);
		$categories = get_categories( $args );
		foreach ( $categories as $category ) {
			if ( isset( $category->term_id ) && isset( $category->name ) ) {
				$json[] = array(
					'id'    => strval( $category->term_id ),
					'label' => $category->name
				);
			}
		}
		wp_send_json( $json );
		die;
	}

	public function get_posts_callback() {
		$this->core->check_auth();
		if ( isset( $_POST['title'] ) && isset( $_POST['categories'] ) ) {
			$title            = $this->core->request_text_data( $_POST['title'] );
			$categories       = $this->core->request_text_data( $_POST['categories'] );
			$categories_array = $this->core->explode_data( $categories );
			$args             = array(
				'post_status'   => 'publish',
				'post_password' => '',
				's'             => $title,
				'category'      => implode( ',', $categories_array ),
				'numberposts'   => 50
			);
			$json             = $this->get_posts_or_pages( $args );
			wp_send_json( $json );
		}
		die;
	}

	public function get_post_or_page_full_content_callback() {
		$this->core->check_auth();
		if ( isset( $_POST['post_id'] ) ) {
			$post_id = $this->core->request_text_data( $_POST['post_id'] );
			$json    = array(
				'id'          => '',
				'title'       => '',
				'description' => '',
				'link'        => '',
				'imageUrl'    => ''
			);
			$post    = get_post( $post_id );
			if ( isset( $post->ID ) && isset( $post->post_title ) && isset( $post->post_excerpt ) && isset( $post->post_content ) ) {
				$json = $this->get_post_data( $post->ID, $post->post_title, $post->post_excerpt, $post->post_content, true );
			}
			wp_send_json( $json );
		}
		die;
	}

	public function get_pages_callback() {
		$this->core->check_auth();
		if ( isset( $_POST['title'] ) ) {
			$title = $this->core->request_text_data( $_POST['title'] );
			$args  = array(
				'post_status'   => 'publish',
				'post_password' => '',
				's'             => $title,
				'post_type'     => 'page',
				'numberposts'   => 50
			);
			$json  = $this->get_posts_or_pages( $args );
			wp_send_json( $json );
		}
		die;
	}

	private function get_posts_or_pages( $args ) {
		$json  = array();
		$posts = get_posts( $args );
		foreach ( $posts as $post ) {
			if ( isset( $post->ID ) && isset( $post->post_title ) && isset( $post->post_excerpt ) && isset( $post->post_content ) ) {
				$json[] = $this->get_post_data( $post->ID, $post->post_title, $post->post_excerpt, $post->post_content, false );
			}
		}
		return $json;
	}

	public function get_woocommerce_product_categories_callback() {
		$this->core->check_auth();
		$json = array();
		if ( $this->core->get_woocommerce_actived() ) {
			$args       = array(
				'hide_empty' => false,
				'taxonomy'   => 'product_cat'
			);
			$categories = get_categories( $args );
			foreach ( $categories as $category ) {
				if ( isset( $category->term_id ) && isset( $category->name ) ) {
					$json[] = array(
						'id'    => strval( $category->term_id ),
						'label' => $category->name
					);
				}
			}
		}
		wp_send_json( $json );
		die;
	}

	public function get_products_woocommerce_callback() {
		$this->core->check_auth();
		$json = array();
		if ( isset( $_POST['title'] ) && isset( $_POST['categories'] ) ) {
			if ( $this->core->get_woocommerce_actived() ) {
				$title      = $this->core->request_text_data( $_POST['title'] );
				$categories = $this->core->request_text_data( $_POST['categories'] );
				$products   = $this->get_woocommerce_products_selection( $categories, 50, $title, '', '' );
				foreach ( $products as $product ) {
					if ( isset( $product['id'] ) && isset( $product['post_title'] ) && isset( $product['post_content'] ) && isset( $product['post_price'] ) ) {
						$json[] = $this->get_woocommerce_product_data( $product['id'], $product['post_title'], $product['post_content'], $product['post_price'], false );
					}
				}
			}
			wp_send_json( $json );
		}
		die;
	}

	public function get_woocommerce_product_full_content_callback() {
		$this->core->check_auth();
		$json = array();
		if ( isset( $_POST['post_id'] ) ) {
			if ( $this->core->get_woocommerce_actived() ) {
				$post_id  = $this->core->request_text_data( $_POST['post_id'] );
				$args     = array(
					'status'  => array( 'publish' ),
					'type'    => array( 'simple' ),
					'orderby' => 'name',
					'order'   => 'ASC',
					'limit'   => 1,
					'include' => array( $post_id )
				);
				$products = $this->get_woocommerce_products( $args );
				foreach ( $products as $product ) {
					if ( isset( $product['id'] ) && isset( $product['post_title'] ) && isset( $product['post_content'] ) && isset( $product['post_price'] ) ) {
						$json = $this->get_woocommerce_product_data( $product['id'], $product['post_title'], $product['post_content'], $product['post_price'], true );
					}
				}
			}
			wp_send_json( $json );
		}
		die;
	}

}