<?php


class Jackmail_List extends Jackmail_List_And_Campaign_Common {

	public function __construct( Jackmail_Core $core ) {

		$this->core = $core;

		if ( $this->core->is_admin() ) {

			add_action( 'wp_ajax_jackmail_list_page', array( $this, 'list_page_callback' ) );

			add_action( 'wp_ajax_jackmail_get_list', array( $this, 'get_list_callback' ) );

			add_action( 'wp_ajax_jackmail_create_list', array( $this, 'create_list_callback' ) );

			add_action( 'wp_ajax_jackmail_export_list', array( $this, 'export_list_callback' ) );

			add_action( 'wp_ajax_jackmail_save_name', array( $this, 'save_name_callback' ) );

		}

	}

	public function list_page_callback() {
		$this->core->check_auth();
		$params = array(
			'page_type'     => 'list',
			'campaign_type' => ''
		);
		$this->core->include_html_file( 'list', $params );
		die;
	}

	public function get_list_callback() {
		$this->core->check_auth();
		$json = $this->get_or_export_list( (string) $this->core->grid_limit() );
		wp_send_json( $json );
		die;
	}

	public function create_list_callback() {
		$this->core->check_auth();
		$json = array(
			'success' => false,
			'id'      => '0'
		);
		if ( isset( $_POST['name'] ) ) {
			$name = $this->core->request_text_data( $_POST['name'] );
			$json = $this->create_list( $name );
		}
		wp_send_json( $json );
		die;
	}

	private function create_list( $name ) {
		$json = array(
			'success' => false,
			'id'      => '0'
		);
		if ( $this->check_list_name_unique( $name ) ) {
			$id = $this->core->create_list( $name );
			if ( $id !== 0 ) {
				$json = array(
					'success' => true,
					'id'      => $id
				);
			}
		}
		return $json;
	}

	public function export_list_callback() {
		$this->core->check_auth();
		$json = $this->get_or_export_list( (string) $this->core->export_send_limit() );
		wp_send_json( $json );
		die;
	}

	private function get_or_export_list( $part ) {
		$data = $this->get_list_or_get_campaign_contacts_post_data();
		if ( isset( $data ['id'] ) && isset( $data ['begin'] ) && isset( $data ['sort_by'] ) && isset( $data ['sort_order'] ) && isset( $data ['search'] ) && isset( $data ['targeting_rules'] ) ) {
			if ( $data ['id'] === '0' ) {
				return array(
					'list'               => array(
						'name'   => __( 'List', 'jackmail-newsletters' ) . ' ' . $this->core->get_current_time_sql(),
						'id'     => '',
						'fields' => '[]'
					),
					'contacts'           => array(),
					'nb_contacts'        => '0',
					'nb_contacts_search' => '0'
				);
			} else {
				return $this->get_list_data( $data ['id'], $data ['begin'], $part, $data ['sort_by'], $data ['sort_order'], $data ['search'], $data ['targeting_rules'] );
			}
		}
		return array();
	}

	public function save_name_callback() {
		$this->core->check_auth();
		if ( isset( $_POST['id'] ) && isset( $_POST['name'] ) ) {
			$id_list = $this->core->request_text_data( $_POST['id'] );
			$name    = $this->core->request_text_data( $_POST['name'] );
			$result  = $this->save_name( $id_list, $name );
			if ( $result ) {
				wp_send_json_success();
			}
		}
		wp_send_json_error();
		die;
	}

	public function save_name( $id_list, $name ) {
		if ( $this->check_list_name_unique( $name ) ) {
			$data = array(
				'name' => $name
			);
			$this->update_list( $id_list, $data );
			return true;
		}
		return false;
	}

	private function check_list_name_unique( $name ) {
		global $wpdb;
		$sql = "SELECT COUNT(*) AS `nb` FROM `{$wpdb->prefix}jackmail_lists` WHERE `name` = %s";
		$nb  = $wpdb->get_row( $wpdb->prepare( $sql, $name ) );
		if ( isset( $nb->nb ) ) {
			if ( $nb->nb === '0' ) {
				return true;
			}
		}
		return false;
	}


}