<?php


class Jackmail_List_Connectors {

	public function __construct( Jackmail_Core $core ) {

		$this->core = $core;

		add_action( 'pre_get_posts', array( $this, 'init_connectors' ) );

		if ( $this->core->is_admin() ) {

			add_action( 'wp_ajax_jackmail_display_connectors', array( $this, 'display_connectors_callback' ) );

			add_action( 'wp_ajax_jackmail_connectors_configuration', array( $this, 'connectors_configuration_callback' ) );

			add_action( 'wp_ajax_jackmail_connectors_configure', array( $this, 'connectors_configure_callback' ) );

			add_action( 'wp_ajax_jackmail_connectors_configure_ip_restriction', array( $this, 'connectors_configure_ip_restriction_callback' ) );

			add_action( 'wp_ajax_jackmail_connectors_configure_allowed_ips', array( $this, 'connectors_configure_allowed_ips_callback' ) );

		}

	}

	public function init_connectors() {
		if ( isset( $_REQUEST['jackmail'] ) && isset( $_REQUEST['list'] ) && isset( $_REQUEST['action'] ) ) {
			$action  = '';
			$id_list = 0;
			$message = 'Wrong parameters';
			if ( get_option( 'jackmail_connectors' ) === '1' && $this->ip_autorized() ) {
				$action        = $this->core->request_text_data( $_REQUEST['action'] );
				$connector_key = $this->core->request_text_data( $_REQUEST['list'] );
				if ( $action === 'add' || $action === 'update' || $action === 'delete' ) {
					$id_list = $this->get_id_list( $connector_key );
					if ( $id_list !== false ) {
						if ( $action === 'add' || $action === 'delete' ) {
							if ( isset( $_REQUEST['email'] ) ) {
								$email = $this->core->request_email_data( $_REQUEST['email'] );
								if ( is_email( $email ) ) {
									if ( $action === 'add' ) {
										$message = $this->add_contact( $id_list, $email );
									} else if ( $action === 'delete' ) {
										$message = $this->delete_contact( $id_list, $email );
									}
								} else {
									$message = 'Email isn\'t valid';
								}
							}
						} else if ( $action === 'update' ) {
							if ( isset( $_REQUEST['email'] ) || ( isset( $_REQUEST['email'] ) && isset( $_REQUEST['new_email'] ) ) ) {
								if ( isset( $_REQUEST['new_email'] ) ) {
									$email     = $this->core->request_email_data( $_REQUEST['email'] );
									$new_email = $this->core->request_email_data( $_REQUEST['new_email'] );
								} else {
									$email     = $this->core->request_email_data( $_REQUEST['email'] );
									$new_email = $this->core->request_email_data( $_REQUEST['email'] );
								}
								if ( is_email( $email ) && is_email( $new_email ) ) {
									if ( $email !== $new_email || isset( $_REQUEST['email'] ) ) {
										$message = $this->update_contact( $id_list, $email, $new_email );
									} else {
										$message = 'Current email and new email are equals';
									}
								} else {
									$message = 'Current email or new email isn\'t valid';
								}
							}
						}
					}
				}
			}
			$this->get_status_message( $action, $id_list, $message );
			die;
		}
	}

	private function get_status_message( $action, $id_list, $message ) {
		$data = array();
		if ( $message !== 'ok' ) {
			$data['success'] = false;
			$data['message'] = $message;
			status_header( 404 );
		} else {
			$data['success'] = true;
			if ( $action === 'add' || $action === 'update' || $action === 'delete' ) {
				$this->update_list( $id_list, $action );
			}
		}
		wp_send_json( $data );
	}

	private function ip_autorized() {
		if ( get_option( 'jackmail_connectors_ip_restriction' ) === '1' ) {
			$ip = '';
			if ( getenv( 'HTTP_CLIENT_IP' ) ) {
				$ip = getenv( 'HTTP_CLIENT_IP' );
			} else if ( getenv( 'HTTP_X_FORWARDED_FOR' ) ) {
				$ip = getenv( 'HTTP_X_FORWARDED_FOR' );
			} else if ( getenv( 'HTTP_X_FORWARDED' ) ) {
				$ip = getenv( 'HTTP_X_FORWARDED' );
			} else if ( getenv( 'HTTP_FORWARDED_FOR' ) ) {
				$ip = getenv( 'HTTP_FORWARDED_FOR' );
			} else if ( getenv( 'HTTP_FORWARDED' ) ) {
				$ip = getenv( 'HTTP_FORWARDED' );
			} else if ( getenv( 'REMOTE_ADDR' ) ) {
				$ip = getenv( 'REMOTE_ADDR' );
			}
			if ( filter_var( $ip, FILTER_VALIDATE_IP ) ) {
				$allowed_ips = $this->core->explode_data( get_option( 'jackmail_connectors_allowed_ips' ) );
				$ip_found    = array_search( $ip, $allowed_ips );
				if ( $ip_found !== false && filter_var( $allowed_ips [ $ip_found ], FILTER_VALIDATE_IP ) ) {
					return true;
				}
			}
			return false;
		}
		return true;
	}

	private function get_id_list( $connector_key ) {
		global $wpdb;
		if ( $this->core->str_len( $connector_key ) >= 100 ) {
			$sql    = "
			SELECT `id`, `connector_activation_date_gmt`, `connector_used_date_gmt`
			FROM `{$wpdb->prefix}jackmail_lists`
			WHERE `connector_key` = %s
			AND `type` = ''
			AND `connector_activation_date_gmt` != '0000-00-00 00:00:00'";
			$result = $wpdb->get_row( $wpdb->prepare( $sql, $connector_key ) );
			if ( isset( $result->id ) ) {
				$id_list                       = $result->id;
				$connector_activation_date_gmt = $result->connector_activation_date_gmt;
				$connector_used_date_gmt       = $result->connector_used_date_gmt;
				if ( $connector_used_date_gmt === '0000-00-00 00:00:00' ) {
					$connector_activation_timestamp = strtotime( $connector_activation_date_gmt );
					$current_time_gmt               = $this->core->get_current_time_gmt_sql();
					$current_timestamp              = strtotime( $current_time_gmt );
					if ( $current_timestamp - $connector_activation_timestamp > 2500000 ) {
						return '0';
					} else {
						$sql = "
						UPDATE `{$wpdb->prefix}jackmail_lists`
						SET `connector_used_date_gmt` = %s
						WHERE `id` = %s
						AND `type` = ''
						AND `connector_activation_date_gmt` != '0'";
						$wpdb->query( $wpdb->prepare( $sql, $current_time_gmt, $id_list ) );
						update_option( 'jackmail_connectors_errors', '0' );
					}
				}
				if ( $this->core->check_table_exists( 'jackmail_lists_contacts_' . $id_list ) ) {
					return $id_list;
				} else {
					return false;
				}
			}
		}
		$errors = ( int ) get_option( 'jackmail_connectors_errors' ) + 1;
		update_option( 'jackmail_connectors_errors', $errors );
		if ( $errors >= 500000 ) {
			update_option( 'jackmail_connectors', '0' );
		}
		return false;
	}

	private function add_contact( $id_list, $email ) {
		global $wpdb;
		$sql    = "
		SELECT `fields`
		FROM `{$wpdb->prefix}jackmail_lists`
		WHERE `id` = %s";
		$result = $wpdb->get_row( $wpdb->prepare( $sql, $id_list ) );
		if ( isset( $result->fields ) ) {
			$fields   = $this->core->explode_fields( $result->fields );
			$requests = array();
			foreach ( $_REQUEST as $key => $request ) {
				if ( $key !== 'jackmail' && $key !== 'list' && $key !== 'action' && $key !== 'email' ) {
					$requests[ $this->core->str_to_upper( str_replace( '_', ' ', $key ) ) ] = $this->core->request_text_data( $request );
					
				}
			}
			$sql_header_fields    = array();
			$sql_header_values    = array();
			$sql_values           = array();
			$sql_header_fields[]  = '`email`';
			$sql_header_values[]  = '%s';
			$sql_values[]         = $email;
			$do_action            = array();
			$do_action['id_list'] = $id_list;
			$do_action['email']   = $email;
			foreach ( $requests as $key => $value ) {
				$field_id = array_search( $key, $fields );
				if ( $field_id !== false ) {
					$sql_header_fields[] = '`field' . ( $field_id + 1 ) . '`';
					$sql_header_values[] = '%s';
					$sql_values[]        = $value;
					$do_action[ $key ]   = $value;
				}
			}
			if ( count( $sql_header_fields ) > 0 && count( $sql_header_values ) > 0 && count( $sql_values ) > 0 ) {
				$sql_header_fields = implode( ', ', $sql_header_fields );
				$sql_header_values = implode( ', ', $sql_header_values );
				$sql               = "
				INSERT IGNORE INTO `{$wpdb->prefix}jackmail_lists_contacts_{$id_list}` ( {$sql_header_fields} ) VALUES ( {$sql_header_values} )";
				$result            = $wpdb->query( $wpdb->prepare( $sql, $sql_values ) );
				if ( $result > 0 ) {
					do_action( 'jackmail_connector_contact_added', $do_action );
					return 'ok';
				}
			}
		}
		return 'Email already exists';
	}

	private function update_contact( $id_list, $email, $new_email ) {
		global $wpdb;
		$sql    = "
		SELECT `fields`
		FROM `{$wpdb->prefix}jackmail_lists`
		WHERE `id` = %s";
		$result = $wpdb->get_row( $wpdb->prepare( $sql, $id_list ) );
		if ( isset( $result->fields ) ) {
			$fields   = $this->core->explode_fields( $result->fields );
			$requests = array();
			foreach ( $_REQUEST as $key => $request ) {
				if ( $key !== 'jackmail' && $key !== 'list' && $key !== 'action' && $key !== 'email' && $key !== 'new_email' ) {
					$requests[ $this->core->str_to_upper( str_replace( '_', ' ', $key ) ) ] = $this->core->request_text_data( $request );
				}
			}
			$sql_header_fields      = array();
			$sql_values             = array();
			$sql_header_fields[]    = '`email` = %s';
			$sql_values[]           = $new_email;
			$do_action              = array();
			$do_action['id_list']   = $id_list;
			$do_action['email']     = $email;
			$do_action['new_email'] = $new_email;
			foreach ( $requests as $key => $value ) {
				$field_id = array_search( $key, $fields );
				if ( $field_id !== false ) {
					$sql_header_fields[] = '`field' . ( $field_id + 1 ) . '` = %s';
					$sql_values[]        = $value;
					$do_action[ $key ]   = $value;
				}
			}
			$sql_values[] = $email;
			if ( count( $sql_header_fields ) > 0 && count( $sql_values ) > 0 ) {
				$sql_header_fields = implode( ', ', $sql_header_fields );
				$sql               = "
				UPDATE IGNORE `{$wpdb->prefix}jackmail_lists_contacts_{$id_list}`
				SET {$sql_header_fields}
				WHERE `email` = %s";
				$result            = $wpdb->query( $wpdb->prepare( $sql, $sql_values ) );
				if ( $result > 0 ) {
					do_action( 'jackmail_connector_contact_updated', $do_action );
					return 'ok';
				} else {
					$sql         = "
					SELECT COUNT( * )
					FROM `{$wpdb->prefix}jackmail_lists_contacts_{$id_list}`
					WHERE `email` = %s";
					$nb_contacts = $wpdb->get_var( $wpdb->prepare( $sql, $email ) );
					if ( $nb_contacts === '0' ) {
						return 'Current email doesn\'t exist';
					}
					if ( $email !== $new_email ) {
						$nb_contacts = $wpdb->get_var( $wpdb->prepare( $sql, $new_email ) );
						if ( $nb_contacts > 0 ) {
							return 'New email already exists';
						}
					}
					return 'No contact changes';
				}
			}
		}
		return 'Current email doesn\'t exists';
	}

	private function delete_contact( $id_list, $email ) {
		global $wpdb;
		$delete_return = $wpdb->delete( "{$wpdb->prefix}jackmail_lists_contacts_{$id_list}", array(
			'email' => $email
		) );
		if ( $delete_return !== false && $wpdb->rows_affected > 0 ) {
			$do_action            = array();
			$do_action['id_list'] = $id_list;
			$do_action['email']   = $email;
			do_action( 'jackmail_connector_contact_deleted', $do_action );
			return 'ok';
		}
		return 'Email doesn\'t found';
	}

	private function update_list( $id_list, $action ) {
		global $wpdb;
		$update = array(
			'updated_date_gmt' => $this->core->get_current_time_gmt_sql()
		);
		if ( $action === 'add' || $action === 'delete' ) {
			$table_list_contacts = "{$wpdb->prefix}jackmail_lists_contacts_{$id_list}";
			if ( $this->core->check_table_exists( $table_list_contacts, false ) ) {
				$sql                          = "
				SELECT COUNT( * )
				FROM `{$table_list_contacts}`";
				$nb_contacts                  = $wpdb->get_var( $sql );
				$sql                          = "
				SELECT COUNT( * )
				FROM `{$table_list_contacts}`
				WHERE `blacklist` = '0'";
				$nb_contacts_valids           = $wpdb->get_var( $sql );
				$update['nb_contacts']        = $nb_contacts;
				$update['nb_contacts_valids'] = $nb_contacts_valids;
			}
		}
		$wpdb->update( "{$wpdb->prefix}jackmail_lists", $update, array(
			'id' => $id_list
		) );
	}

	public function display_connectors_callback() {
		$this->core->check_auth();
		global $wpdb;
		if ( isset( $_POST['id'] ) ) {
			$id_list = $this->core->request_text_data( $_POST['id'] );
			$wpdb->update( "{$wpdb->prefix}jackmail_lists", array(
				'connector_activation_date_gmt' => $this->core->get_current_time_gmt_sql()
			), array(
				'id'   => $id_list,
				'type' => ''
			) );

		}
		$data = array(
			'active' => ( bool ) get_option( 'jackmail_connectors' )
		);
		wp_send_json( $data );
		die;
	}

	public function connectors_configuration_callback() {
		$this->core->check_auth();
		$data = array(
			'active'         => ( bool ) get_option( 'jackmail_connectors' ),
			'ip_restriction' => ( bool ) get_option( 'jackmail_connectors_ip_restriction' ),
			'allowed_ips'    => get_option( 'jackmail_connectors_allowed_ips', '' )
		);
		wp_send_json( $data );
		die;
	}

	public function connectors_configure_callback() {
		$this->core->check_auth();
		if ( isset( $_POST['active'] ) ) {
			$active = $this->core->request_text_data( $_POST['active'] );
			update_option( 'jackmail_connectors', $active );
			update_option( 'jackmail_connectors_errors', '0' );
			wp_send_json_success();
		}
		wp_send_json_error();
		die;
	}

	public function connectors_configure_ip_restriction_callback() {
		$this->core->check_auth();
		if ( isset( $_POST['ip_restriction'] ) ) {
			$ip_restriction = $this->core->request_text_data( $_POST['ip_restriction'] );
			update_option( 'jackmail_connectors_ip_restriction', $ip_restriction );
			wp_send_json_success();
		}
		wp_send_json_error();
		die;
	}

	public function connectors_configure_allowed_ips_callback() {
		$this->core->check_auth();
		$data = array(
			'success'     => false,
			'allowed_ips' => '[]'
		);
		if ( isset( $_POST['allowed_ips'] ) ) {
			$allowed_ips     = $this->core->explode_data( $this->core->request_text_data( $_POST['allowed_ips'] ) );
			$new_allowed_ips = array();
			foreach ( $allowed_ips as $allowed_ip ) {
				if ( filter_var( $allowed_ip, FILTER_VALIDATE_IP ) ) {
					if ( ! in_array( $allowed_ip, $new_allowed_ips ) ) {
						$new_allowed_ips[] = $allowed_ip;
					}
				}
			}
			$new_allowed_ips     = $this->core->implode_data( $new_allowed_ips );
			$current_allowed_ips = get_option( 'jackmail_connectors_allowed_ips', '' );
			$success             = false;
			if ( $new_allowed_ips !== $current_allowed_ips ) {
				update_option( 'jackmail_connectors_allowed_ips', $new_allowed_ips );
				$success = true;
			}
			$data = array(
				'success'     => $success,
				'allowed_ips' => $new_allowed_ips
			);
		}
		wp_send_json( $data );
		die;
	}

}