<?php


class Jackmail_Search {

	public function __construct( Jackmail_Core $core ) {

		$this->core = $core;

		if ( $this->core->is_admin() ) {

			add_action( 'wp_ajax_jackmail_search_faq', array( $this, 'search_faq_callback' ) );

			add_action( 'wp_ajax_jackmail_search_campaigns', array( $this, 'search_campaigns_callback' ) );

			add_action( 'wp_ajax_jackmail_search_all', array( $this, 'search_all_callback' ) );

			add_action( 'wp_ajax_jackmail_suggestion_faq', array( $this, 'suggestion_faq_callback' ) );

			add_action( 'wp_ajax_jackmail_suggestion_forum', array( $this, 'suggestion_forum_callback' ) );

		}

	}

	public function search_faq_callback() {
		$this->core->check_auth();
		if ( isset( $_POST['search'] ) ) {
			$search   = $this->core->request_text_data( $_POST['search'] );
			$lang     = $this->core->get_current_language();
			$url      = $this->core->get_jackmail_url_ws() . 'search.php?search=' . urlencode( $search ) . '&type=faq&lang=' . $lang;
			$headers  = array();
			$timeout  = 30;
			$response = $this->core->remote_get( $url, $headers, $timeout );
			$json     = array();
			if ( is_array( $response ) ) {
				if ( isset( $response['body'] ) ) {
					$json = json_decode( $response['body'], true );
					if ( $json === null ) {
						$json = array();
					}
				}
			}
			wp_send_json( $json );
		}
		die;
	}

	public function search_campaigns_callback() {
		$this->core->check_auth();
		global $wpdb;
		if ( isset( $_POST['search'] ) ) {
			$search  = $this->core->request_text_data( $_POST['search'] );
			$sql     = "
			SELECT `id`, `name`, `preview`, `updated_date_gmt`, `status`
			FROM `{$wpdb->prefix}jackmail_campaigns` WHERE `name` LIKE %s
			ORDER BY `updated_date_gmt` DESC";
			$results = $wpdb->get_results( $wpdb->prepare( $sql, '%' . $wpdb->esc_like( $search ) . '%' ) );
			foreach ( $results as $key => $result ) {
				$results[ $key ]->preview = $this->core->content_email_preview_url( $result->preview, 'campaign' );
			}
			wp_send_json( $results );
		}
		die;
	}

	public function search_all_callback() {
		$this->core->check_auth();
		global $wpdb;
		if ( isset( $_POST['search'] ) ) {
			$search     = $this->core->request_text_data( $_POST['search'] );
			$sql        = "
			SELECT `id`, `name`, 'campaign' AS `type`, `updated_date_gmt`
			FROM `{$wpdb->prefix}jackmail_campaigns`
			WHERE `name` LIKE %s
			ORDER BY `updated_date_gmt` DESC";
			$campaigns  = $wpdb->get_results( $wpdb->prepare( $sql, '%' . $wpdb->esc_like( $search ) . '%' ) );
			$sql        = "
			SELECT `id`, `name`, 'scenario' AS `type`, `updated_date_gmt`
			FROM `{$wpdb->prefix}jackmail_scenarios`
			WHERE `name` LIKE %s
			ORDER BY `updated_date_gmt` DESC";
			$scenarios  = $wpdb->get_results( $wpdb->prepare( $sql, '%' . $wpdb->esc_like( $search ) . '%' ) );
			$sql        = "
			SELECT `id`, `name`, 'statistics' AS `type`, `updated_date_gmt`
			FROM `{$wpdb->prefix}jackmail_campaigns`
			WHERE `name` LIKE %s AND `status` != 'DRAFT'
			ORDER BY `updated_date_gmt` DESC";
			$statistics = $wpdb->get_results( $wpdb->prepare( $sql, '%' . $wpdb->esc_like( $search ) . '%' ) );
			$sql        = "
			SELECT `id`, `name`, 'list' AS `type`, `updated_date_gmt`
			FROM `{$wpdb->prefix}jackmail_lists`
			WHERE `name` LIKE %s
			ORDER BY `updated_date_gmt` DESC";
			$lists      = $wpdb->get_results( $wpdb->prepare( $sql, '%' . $wpdb->esc_like( $search ) . '%' ) );
			$sql        = "
			SELECT `id`, `name`, 'template' AS `type`, `updated_date_gmt`
			FROM `{$wpdb->prefix}jackmail_templates`
			WHERE `name` LIKE %s
			ORDER BY `updated_date_gmt` DESC";
			$templates  = $wpdb->get_results( $wpdb->prepare( $sql, '%' . $wpdb->esc_like( $search ) . '%' ) );
			$results    = array_merge( $campaigns, $scenarios, $statistics, $lists, $templates );
			usort( $results, function ( $a, $b ) {
				return strcmp( $b->updated_date_gmt, $a->updated_date_gmt );
			} );
			wp_send_json( $results );
		}
		die;
	}

	public function suggestion_faq_callback() {
		$this->core->check_auth();
		$lang     = $this->core->get_current_language();
		$url      = $this->core->get_jackmail_url_ws() . 'suggestion.php?type=faq&lang=' . $lang;
		$headers  = array();
		$timeout  = 30;
		$response = $this->core->remote_get( $url, $headers, $timeout );
		$json     = array();
		if ( is_array( $response ) ) {
			if ( isset( $response['body'] ) ) {
				$json = json_decode( $response['body'], true );
				if ( $json === null ) {
					$json = array();
				}
			}
		}
		wp_send_json( $json );
		die;
	}

	public function suggestion_forum_callback() {
		$this->core->check_auth();
		if ( isset( $_POST['search'] ) ) {
			$search   = $this->core->request_text_data( $_POST['search'] );
			$lang     = $this->core->get_current_language();
			$url      = $this->core->get_jackmail_url_ws() . 'search.php?search=' . urlencode( $search ) . '&type=forum&lang=' . $lang;
			$headers  = array();
			$timeout  = 30;
			$response = $this->core->remote_get( $url, $headers, $timeout );
			$json     = array();
			if ( is_array( $response ) ) {
				if ( isset( $response['body'] ) ) {
					$json = json_decode( $response['body'], true );
					if ( $json === null ) {
						$json = array();
					}
				}
			}
			wp_send_json( $json );
		}
		die;
	}

}