<?php


class Jackmail_Settings {

	public function __construct( Jackmail_Core $core ) {

		$this->core = $core;

		add_action( 'jackmail_cron_domain', array( $this, 'cron_domain' ) );

		add_action( 'jackmail_cron_clean_files', array( $this, 'cron_clean_files' ) );

		if ( $this->core->is_admin() ) {

			add_action( 'wp_ajax_jackmail_settings_page', array( $this, 'settings_page_callback' ) );

			add_action( 'wp_ajax_jackmail_domain_configuration', array( $this, 'domain_configuration_callback' ) );

			add_action( 'wp_ajax_jackmail_domain_get_txt_ns', array( $this, 'domain_get_txt_ns_callback' ) );

			add_action( 'wp_ajax_jackmail_domain_check_txt', array( $this, 'domain_check_txt_callback' ) );

			add_action( 'wp_ajax_jackmail_domain_create_delegation', array( $this, 'domain_create_delegation_callback' ) );

			add_action( 'wp_ajax_jackmail_domain_delete', array( $this, 'domain_delete_callback' ) );

			add_action( 'wp_ajax_jackmail_get_link_tracking', array( $this, 'get_link_tracking_callback' ) );

			add_action( 'wp_ajax_jackmail_set_link_tracking', array( $this, 'set_link_tracking_callback' ) );

			add_action( 'wp_ajax_jackmail_uninstall_emailbuilder', array( $this, 'uninstall_emailbuilder_callback' ) );

			add_action( 'wp_ajax_jackmail_credits_available', array( $this, 'credits_available_callback' ) );

			add_action( 'wp_ajax_jackmail_user_disconnect', array( $this, 'user_disconnect_callback' ) );

			add_action( 'wp_ajax_jackmail_get_debug', array( $this, 'get_degug_callback' ) );

			add_action( 'wp_ajax_jackmail_get_improvement', array( $this, 'get_improvement_callback' ) );

			add_action( 'wp_ajax_jackmail_set_improvement', array( $this, 'set_improvement_callback' ) );

			add_action( 'wp_ajax_jackmail_get_support_chat', array( $this, 'get_support_chat_callback' ) );

			add_action( 'wp_ajax_jackmail_set_support_chat', array( $this, 'set_support_chat_callback' ) );

			add_action( 'wp_ajax_jackmail_get_premium_notification', array( $this, 'get_premium_notification_callback' ) );

			add_action( 'wp_ajax_jackmail_set_premium_notification', array( $this, 'set_premium_notification_callback' ) );

			add_action( 'wp_ajax_jackmail_update_cookies', array( $this, 'update_cookies_callback' ) );

		}

	}

	public function settings_page_callback() {
		$this->core->check_auth();
		$this->core->include_html_file( 'settings' );
		die;
	}

	public function domain_configuration_callback() {
		$this->core->check_auth();
		$subdomain = get_option( 'jackmail_domain_sub' );
		$txt       = $this->get_txt( $subdomain );
		$data      = array(
			'subdomain' => '',
			'txt'       => '',
			'ns1'       => '',
			'ns2'       => '',
			'is_valid'  => ''
		);
		if ( $subdomain !== '' && $txt !== '' ) {
			$this->core->cron_domain();
			$ns1  = $this->get_ns1( $subdomain );
			$ns2  = $this->get_ns2( $subdomain );
			$data = array(
				'subdomain' => $subdomain,
				'txt'       => $txt,
				'ns1'       => $ns1,
				'ns2'       => $ns2,
				'is_valid'  => $this->core->custom_domain_is_valid()
			);
		}
		wp_send_json( $data );
		die;
	}

	private function get_txt( $subdomain, $full = true ) {
		$subdomain_array = explode( '.', $subdomain );
		$subdomain_txt   = '';
		if ( count( $subdomain_array ) === 1 ) {
			$subdomain_txt = $subdomain_array[0];
		} else {
			foreach ( $subdomain_array as $key => $data ) {
				if ( $key !== 0 ) {
					if ( $subdomain_txt !== '' ) {
						$subdomain_txt .= '.';
					}
					$subdomain_txt .= $data;
				}
			}
		}
		$txt = base64_encode( sha1( 'txt' . $this->core->get_account_id() . 'jackmail' . $subdomain . 'subdomain', false ) );
		if ( $full ) {
			return $subdomain_txt . ' IN TXT "' . $txt . '"';
		}
		return $txt;
	}

	private function get_ns1( $subdomain ) {
		$ns = $this->get_ns( $subdomain );
		return $ns . ' IN NS dns.customizedurl.com.';
	}

	private function get_ns2( $subdomain ) {
		$ns = $this->get_ns( $subdomain );
		return $ns . ' IN NS dns2.customizedurl.com.';
	}

	private function get_ns( $subdomain ) {
		$subdomain_array = explode( '.', $subdomain );
		$subdomain_txt   = '';
		if ( count( $subdomain_array ) !== 0 ) {
			$subdomain_txt = $subdomain_array[0];
		}
		return $subdomain_txt;
	}

	public function domain_get_txt_ns_callback() {
		$this->core->check_auth();
		$data = array(
			'txt' => '',
			'ns1' => '',
			'ns2' => ''
		);
		if ( isset( $_POST['subdomain'] ) ) {
			$subdomain = $this->core->request_text_data( $_POST['subdomain'] );
			$data      = array(
				'txt' => $this->get_txt( $subdomain ),
				'ns1' => $this->get_ns1( $subdomain ),
				'ns2' => $this->get_ns2( $subdomain )
			);
		}
		wp_send_json( $data );
		die;
	}

	public function domain_check_txt_callback() {
		$this->core->check_auth();
		$json = array(
			'success' => false
		);
		if ( isset( $_POST['subdomain'] ) ) {
			$subdomain       = $this->core->request_text_data( $_POST['subdomain'] );
			$json['success'] = $this->domain_check_txt( $subdomain );
		}
		wp_send_json( $json );
		die;
	}

	private function domain_check_txt( $subdomain ) {
		$txt           = $this->get_txt( $subdomain, false );
		$pos           = strpos( $subdomain, '.' );
		$parent_domain = substr( $subdomain, $pos + 1 );
		$url           = $this->core->get_jackmail_url_api() . 'domains/check/txt';
		$headers       = array(
			'content-type' => 'application/json',
			'token'        => $this->core->get_account_token(),
			'accountId'    => $this->core->get_account_id(),
			'userId'       => $this->core->get_user_id(),
			'licenseKey'   => 'licenseKey'
		);
		$body          = array(
			'domainName' => $parent_domain,
			'txt'        => $txt
		);
		$timeout       = 30;
		$response      = $this->core->remote_post_retry( $url, $headers, $body, $timeout );
		if ( is_array( $response ) ) {
			if ( isset( $response['body'] ) ) {
				$results = json_decode( $response['body'], true );
				if ( is_array( $results ) ) {
					if ( isset( $results['status'] ) ) {
						if ( $results['status'] === 'ACCEPTED' ) {
							return true;
						}
					}
				}
			}
		}
		return false;
	}

	public function domain_create_delegation_callback() {
		$this->core->check_auth();
		$json = array(
			'success' => false
		);
		if ( isset( $_POST['subdomain'] ) ) {
			$subdomain = $this->core->request_text_data( $_POST['subdomain'] );
			if ( $this->domain_check_txt( $subdomain ) ) {
				$url      = $this->core->get_jackmail_url_api() . 'domains/' . urlencode( $subdomain ) . '/create/delegation';
				$headers  = array(
					'content-type' => 'application/json',
					'token'        => $this->core->get_account_token(),
					'accountId'    => $this->core->get_account_id(),
					'userId'       => $this->core->get_user_id(),
					'licenseKey'   => 'licenseKey'
				);
				$body     = array();
				$timeout  = 30;
				$response = $this->core->remote_post_retry( $url, $headers, $body, $timeout );
				if ( is_array( $response ) ) {
					if ( isset( $response['body'] ) ) {
						$results = json_decode( $response['body'], true );
						if ( is_array( $results ) ) {
							if ( isset( $results['status'] ) ) {
								update_option( 'jackmail_domain_sub', $subdomain );
								if ( $results['status'] === 'ACCEPTED' ) {
									update_option( 'jackmail_domain_check', '2' );
								} else {
									update_option( 'jackmail_domain_check', '6' );
								}
								$url     = $this->core->get_jackmail_url_domain() . 'v2/domains/' . urlencode( $this->core->get_account_id() );
								$headers = array(
									'content-type' => 'application/json',
									'token'        => $this->core->get_account_token(),
									'accountId'    => $this->core->get_account_id(),
									'userId'       => $this->core->get_user_id(),
									'licenseKey'   => 'licenseKey'
								);
								$body    = array(
									'AccountID'          => $this->core->get_account_id(),
									'DomainName'         => $subdomain,
									'TrackingDomainName' => 'eye.' . $subdomain,
									'ImgDomainName'      => 'img.' . $subdomain,
									'Validity'           => intval( get_option( 'jackmail_domain_check' ) )
								);
								$timeout = 30;
								$this->core->remote_post_retry( $url, $headers, $body, $timeout );
								$json['success'] = true;
							}
						}
					}
				}
			}
		}
		$this->core->cron_domain();
		wp_send_json( $json );
		die;
	}

	public function domain_delete_callback() {
		$this->core->check_auth();
		update_option( 'jackmail_domain_sub', '' );
		update_option( 'jackmail_domain_check', '' );
		$this->core->cron_domain();
		wp_send_json_success();
		die;
	}

	public function get_link_tracking_callback() {
		$this->core->check_auth();
		$json = array(
			'active' => get_option( 'jackmail_link_tracking' )
		);
		wp_send_json( $json );
		die;
	}

	public function set_link_tracking_callback() {
		$this->core->check_auth();
		if ( isset( $_POST['active'] ) ) {
			$tracking = $this->core->request_text_data( $_POST['active'] );
			update_option( 'jackmail_link_tracking', $tracking );
			wp_send_json_success();
		}
		die;
	}

	public function uninstall_emailbuilder_callback() {
		$this->core->check_auth();
		update_option( 'jackmail_emailbuilder', '0' );
		wp_send_json_success();
		die;
	}

	public function credits_available_callback() {
		$this->core->check_auth();
		$this->core->cron_notifications();
		$json              = array(
			'nb_credits'        => '',
			'subscription_type' => '',
			'product_key'       => ''
		);
		$credits_available = $this->core->get_credits_available( true );
		if ( $credits_available !== false ) {
			$json = $credits_available;
		}
		wp_send_json( $json );
		die;
	}

	public function user_disconnect_callback() {
		$this->core->check_auth();
		Jackmail_Plugin::uninstall();
		Jackmail_Plugin::install();
		wp_send_json_success();
		die;
	}


	public function get_degug_callback() {
		$this->core->check_auth();
		global $wpdb;
		$server = 'unknown';
		$php    = 'unknown';
		if ( function_exists( 'apache_get_version' ) ) {
			$server = apache_get_version();
		}
		if ( function_exists( 'phpversion' ) ) {
			$php = phpversion();
		}
		$wordfence = false;
		$plugins   = get_option( 'active_plugins' );
		foreach ( $plugins as $plugin_url ) {
			if ( $plugin_url === 'wordfence/wordfence.php' ) {
				$wordfence = true;
			}
		}
		$json       = array(
			'server'    => $server,
			'database'  => $wpdb->db_version(),
			'php'       => $php,
			'jackmail'  => $this->core->get_jackmail_version(),
			'wordpress' => get_bloginfo( 'version' ),
			'wordfence' => $wordfence,
			'logs'      => ''
		);
		$error_path = @ini_get( 'error_log' );
		if ( $error_path !== false && $error_path !== '' ) {
			if ( file_exists( $error_path ) ) {
				if ( is_readable( $error_path ) ) {
					$logs  = '';
					$size  = filesize( $error_path );
					$begin = $size - 100000;
					if ( $begin < 0 ) {
						$begin = 0;
					}
					$content      = @file_get_contents( $error_path, false, null, $begin );
					$content      = str_replace( "\r\n", "\n", $content );
					$lines        = explode( "\n", $content );
					$plugin_path  = substr( get_home_path(), 0, - 1 );
					$plugin_path2 = str_replace( '/', '\\', $plugin_path );
					foreach ( $lines as $line ) {
						if ( strpos( $line, 'jackmail' ) !== false ) {
							$line = str_replace( $plugin_path, '', $line );
							$line = str_replace( $plugin_path2, '', $line );
							$logs .= $line;
						}
					}
					$json['logs'] = $logs;
				}
			}
		}
		wp_send_json( $json );
		die;
	}

	public function get_improvement_callback() {
		$this->core->check_auth();
		$json = $this->get_improvement();
		wp_send_json( $json );
		die;
	}

	public function set_improvement_callback() {
		$this->core->check_auth();
		if ( isset( $_POST['active'] ) ) {
			$improvement = $this->core->request_text_data( $_POST['active'] );
			$this->set_improvement( $improvement );
			wp_send_json_success();
		}
		wp_send_json_error();
		die;
	}

	public function get_support_chat_callback() {
		$this->core->check_auth();
		$json = $this->get_support_chat();
		wp_send_json( $json );
		die;
	}

	public function set_support_chat_callback() {
		$this->core->check_auth();
		if ( isset( $_POST['active'] ) ) {
			$support_chat = $this->core->request_text_data( $_POST['active'] );
			$this->set_support_chat( $support_chat );
			wp_send_json_success();
		}
		wp_send_json_error();
		die;
	}

	public function get_premium_notification_callback() {
		$this->core->check_auth();
		$json = $this->get_premium_notification();
		wp_send_json( $json );
		die;
	}

	public function set_premium_notification_callback() {
		$this->core->check_auth();
		if ( isset( $_POST['active'] ) ) {
			$premium_notification = $this->core->request_text_data( $_POST['active'] );
			$this->set_premium_notification( $premium_notification );
			wp_send_json_success();
		}
		wp_send_json_error();
		die;
	}

	public function update_cookies_callback() {
		$this->core->check_auth();
		$expiration = time() + 3600;
		if ( isset( $_POST['selected_date1'] ) ) {
			$value = $this->core->request_text_data( $_POST['selected_date1'] );
			setcookie( 'jackmail_selected_date1', $value, $expiration, COOKIEPATH, COOKIE_DOMAIN );
		}
		if ( isset( $_POST['selected_date2'] ) ) {
			$value = $this->core->request_text_data( $_POST['selected_date2'] );
			setcookie( 'jackmail_selected_date2', $value, $expiration, COOKIEPATH, COOKIE_DOMAIN );
		}
		if ( isset( $_POST['campaign_emailing'] ) ) {
			$value = $this->core->request_text_data( $_POST['campaign_emailing'] );
			if ( $value === 'false' || $value === '0' || $value === '' ) {
				$value = 'false';
			} else {
				$value = 'true';
			}
			setcookie( 'jackmail_campaign_emailing', $value, $expiration, COOKIEPATH, COOKIE_DOMAIN );
		}
		if ( isset( $_POST['campaign_scenario'] ) ) {
			$value = $this->core->request_text_data( $_POST['campaign_scenario'] );
			if ( $value === 'false' || $value === '0' || $value === '' ) {
				$value = 'false';
			} else {
				$value = 'true';
			}
			setcookie( 'jackmail_campaign_scenario', $value, $expiration, COOKIEPATH, COOKIE_DOMAIN );
		}
		wp_send_json_success();
		die;
	}

	private function get_improvement() {
		return array(
			'active' => $this->core->boolval( get_option( 'jackmail_improvement' ) )
		);
	}

	private function set_improvement( $improvement ) {
		update_option( 'jackmail_improvement', (int) ( $improvement === 'true' ) );
	}

	private function get_support_chat() {
		return array(
			'active' => $this->core->boolval( get_option( 'jackmail_support_chat' ) )
		);
	}

	private function set_support_chat( $support_chat ) {
		update_option( 'jackmail_support_chat', (int) ( $support_chat === 'true' ) );
	}

	private function get_premium_notification() {
		return array(
			'active' => $this->core->boolval( get_option( 'jackmail_premium_notification' ) )
		);
	}

	private function set_premium_notification( $premium_notification ) {
		if ( $premium_notification === 'true' ) {
			update_option( 'jackmail_premium_notification', '1' );
		} else {
			update_option( 'jackmail_premium_notification', '0' );
			update_option( 'jackmail_display_premium_notification', '0' );
			update_option( 'jackmail_display_premium_notification_last_hide', '' );
		}
	}

	public function cron_domain() {
		$this->core->get_jackmail_update_available();
		$this->core->cron_domain();
	}

	public function cron_clean_files() {
		$this->core->get_jackmail_update_available();
		global $wpdb;
		$sql           = "
		( SELECT `preview`, `content_email_images` FROM `{$wpdb->prefix}jackmail_campaigns` )
		UNION
		( SELECT `preview`, `content_email_images` FROM `{$wpdb->prefix}jackmail_scenarios` )
		UNION
		( SELECT `preview`, `content_email_images` FROM `{$wpdb->prefix}jackmail_templates` )";
		$results       = $wpdb->get_results( $sql );
		$all_images_db = array();
		$images_name   = array();
		foreach ( $results as $result ) {
			$all_images_db[] = $result->preview . '.png';
			$images_name[]   = $result->preview . '.png';
			$images          = json_decode( $result->content_email_images, true );
			foreach ( $images as $image ) {
				if ( isset( $image['image_id'] ) && isset( $image['image_type'] ) ) {
					$image_info = $image['image_id'] . '.' . $image['image_type'];
					if ( ! in_array( $image_info, $images_name ) ) {
						$all_images_db[] = $image_info;
						$images_name[]   = $image_info;
					}
				}
			}
		}
		$images = json_decode( get_option( 'jackmail_default_template_images' ), true );
		foreach ( $images as $image ) {
			if ( isset( $image['image_id'] ) && isset( $image['image_type'] ) ) {
				$image_info = $image['image_id'] . '.' . $image['image_type'];
				if ( ! in_array( $image_info, $images_name ) ) {
					$all_images_db[] = $image_info;
					$images_name[]   = $image_info;
				}
			}
		}
		$nb_images_db              = count( $all_images_db );
		$jackmail_file_path        = $this->core->get_jackmail_file_path();
		$all_images_files          = glob( $jackmail_file_path . "*.{jpeg,png,gif,bmp,tiff}", GLOB_BRACE );
		$jackmail_file_path_length = $this->core->str_len( $jackmail_file_path );
		if ( $all_images_files !== false ) {
			$nb_images_files = count( $all_images_files );
			if ( $nb_images_files > $nb_images_db ) {
				$current_time_gmt  = $this->core->get_current_time_gmt_sql();
				$current_timestamp = strtotime( $current_time_gmt );
				foreach ( $all_images_files as $image_path ) {
					$image_name = substr( $image_path, $jackmail_file_path_length );
					if ( ! in_array( $image_name, $all_images_db ) ) {
						$file_updated_date = @filemtime( $image_path );
						if ( $file_updated_date !== false ) {
							if ( $current_timestamp - $file_updated_date > 172800 ) {
								@unlink( $image_path );
							}
						}
					}
				}
			}
		}
	}

}