<?php


class Jackmail_Template {

	public function __construct( Jackmail_Core $core ) {

		$this->core = $core;

		if ( $this->core->is_admin() ) {

			add_action( 'wp_ajax_jackmail_template_page', array( $this, 'template_page_callback' ) );

			add_action( 'wp_ajax_jackmail_get_template', array( $this, 'get_template_callback' ) );

			add_action( 'wp_ajax_jackmail_get_template_json', array( $this, 'get_template_json_callback' ) );

			add_action( 'wp_ajax_jackmail_get_gallery_template_json', array( $this, 'get_gallery_template_json_callback' ) );

			add_action( 'wp_ajax_jackmail_create_template', array( $this, 'create_template_callback' ) );

			add_action( 'wp_ajax_jackmail_update_template', array( $this, 'update_template_callback' ) );

		}

	}

	public function template_page_callback() {
		$this->core->check_auth();
		$this->core->include_html_file( 'template' );
		die;
	}

	public function get_template_callback() {
		$this->core->check_auth();
		global $wpdb;
		$template = null;
		if ( isset( $_POST['id'] ) ) {
			$id_template = $this->core->request_text_data( $_POST['id'] );
			if ( $id_template === '0' ) {
				$template = array(
					'id'                 => '0',
					'name'               => __( 'Template with no name', 'jackmail-newsletters' ),
					'content_email_json' => '',
					'content_email_html' => '',
					'content_email_txt'  => ''
				);
			} else {
				$sql      = "SELECT * FROM `{$wpdb->prefix}jackmail_templates` WHERE `id` = %s";
				$template = $wpdb->get_row( $wpdb->prepare( $sql, $id_template ) );
			}
			wp_send_json( $template );
		}
		die;
	}

	public function get_template_json_callback() {
		$this->core->check_auth();
		global $wpdb;
		$json = array(
			'content_email_json' => '',
		);
		if ( isset( $_POST['id'] ) && isset( $_POST['link_tracking'] ) ) {
			$id_template   = $this->core->request_text_data( $_POST['id'] );
			$link_tracking = $this->core->request_text_data( $_POST['link_tracking'] );
			$sql           = "
			SELECT `content_email_json`, `content_email_html`, `content_email_txt`, `content_email_images`
			FROM `{$wpdb->prefix}jackmail_templates`
			WHERE `id` = %s";
			$template      = $wpdb->get_row( $wpdb->prepare( $sql, $id_template ) );
			if ( isset( $template->content_email_json ) ) {
				$json['content_email_json'] = $this->core->content_email_json_link_tracking( $template->content_email_json, $link_tracking );
			}
		}
		wp_send_json( $json );
		die;
	}

	public function get_gallery_template_json_callback() {
		$this->core->check_auth();
		ini_set( 'max_execution_time', 200 );
		$json = array(
			'content_email_json' => ''
		);
		if ( isset( $_POST['gallery_id'] ) && isset( $_POST['link_tracking'] ) ) {
			$gallery_id                 = $this->core->request_text_data( $_POST['gallery_id'] );
			$link_tracking              = $this->core->request_text_data( $_POST['link_tracking'] );
			$json['content_email_json'] = $this->core->get_gallery_template_json( $gallery_id, $link_tracking );
		}
		wp_send_json( $json );
		die;
	}

	public function create_template_callback() {
		$this->core->check_auth();
		global $wpdb;
		$result = array(
			'success'            => false,
			'id'                 => '0',
			'content_email_json' => ''
		);
		if ( isset( $_POST['name'] ) && isset( $_POST['content_email_json'] ) && isset( $_POST['content_email_html'] ) && isset( $_POST['content_email_txt'] ) ) {
			$name               = $this->core->request_text_data( $_POST['name'] );
			$content_email_json = $this->core->request_stripslashes( $_POST['content_email_json'] );
			$content_email_html = '';
			$content_email_txt  = '';
			$preview            = $this->core->generate_jackmail_preview_filename();
			$content_email      = $this->core->set_content_email( 'template', '0', $preview, $content_email_json, $content_email_html, $content_email_txt );
			if ( $content_email !== false ) {
				if ( isset( $content_email['content_email_json'] ) && isset( $content_email['content_email_images'] ) ) {
					$content_email_json   = $content_email['content_email_json'];
					$content_email_images = $content_email['content_email_images'];
					$current_date_gmt     = $this->core->get_current_time_gmt_sql();
					$insert_return        = $wpdb->insert( "{$wpdb->prefix}jackmail_templates", array(
						'name'                 => $name,
						'content_email_json'   => $content_email_json,
						'content_email_images' => $content_email_images,
						'preview'              => $preview,
						'created_date_gmt'     => $current_date_gmt,
						'updated_date_gmt'     => $current_date_gmt
					) );
					$id_template          = strval( $wpdb->insert_id );
					if ( $insert_return !== false && $id_template !== false ) {
						$result = array(
							'success'            => true,
							'id'                 => $id_template,
							'content_email_json' => $content_email_json
						);
					}
				}
			}
		}
		wp_send_json( $result );
		die;
	}

	public function update_template_callback() {
		$this->core->check_auth();
		global $wpdb;
		$result = array(
			'success'            => false,
			'content_email_json' => ''
		);
		if ( isset( $_POST['id'] ) && isset( $_POST['name'] ) && isset( $_POST['content_email_json'] ) && isset( $_POST['content_email_html'] ) && isset( $_POST['content_email_txt'] ) ) {
			$id_template        = $this->core->request_text_data( $_POST['id'] );
			$name               = $this->core->request_text_data( $_POST['name'] );
			$content_email_json = $this->core->request_stripslashes( $_POST['content_email_json'] );
			$content_email_html = '';
			$content_email_txt  = '';
			$current_date_gmt   = $this->core->get_current_time_gmt_sql();
			$sql                = "
			SELECT `preview`
			FROM `{$wpdb->prefix}jackmail_templates`
			WHERE `id` = %s";
			$template           = $wpdb->get_row( $wpdb->prepare( $sql, $id_template ) );
			if ( isset( $template->preview ) ) {
				$content_email = $this->core->set_content_email( 'template', $id_template, $template->preview, $content_email_json, $content_email_html, $content_email_txt );
				if ( $content_email !== false ) {
					if ( isset( $content_email['content_email_json'] ) && isset( $content_email['content_email_images'] ) ) {
						$content_email_json   = $content_email['content_email_json'];
						$content_email_images = $content_email['content_email_images'];
						$wpdb->update( "{$wpdb->prefix}jackmail_templates", array(
							'name'                 => $name,
							'content_email_json'   => $content_email_json,
							'content_email_images' => $content_email_images,
							'updated_date_gmt'     => $current_date_gmt
						), array(
							'id' => $id_template
						) );
						if ( $wpdb->rows_affected > 0 ) {
							$result = array(
								'success'            => true,
								'content_email_json' => $content_email_json
							);
						}
					}
				}
			}
		}
		wp_send_json( $result );
		die;
	}

}