<?php


class Jackmail_Templates {

	public function __construct( Jackmail_Core $core ) {

		$this->core = $core;

		if ( $this->core->is_admin() ) {

			add_action( 'wp_ajax_jackmail_templates_page', array( $this, 'templates_page_callback' ) );

			add_action( 'wp_ajax_jackmail_get_templates', array( $this, 'get_templates_callback' ) );

			add_action( 'wp_ajax_jackmail_get_templates_gallery', array( $this, 'get_templates_gallery_callback' ) );

			add_action( 'wp_ajax_jackmail_delete_template', array( $this, 'delete_template_callback' ) );

			add_action( 'wp_ajax_jackmail_duplicate_template', array( $this, 'duplicate_template_callback' ) );

		}

	}

	public function templates_page_callback() {
		$this->core->check_auth();
		$params = array(
			'current_page' => 'templates'
		);
		$this->core->include_html_file( 'templates', $params );
		die;
	}

	public function get_templates_callback() {
		$this->core->check_auth();
		global $wpdb;
		$sql       = "
		SELECT `id`, `name`, `preview`, `updated_date_gmt`
		FROM `{$wpdb->prefix}jackmail_templates`
		ORDER BY `updated_date_gmt` DESC";
		$templates = $wpdb->get_results( $sql );
		foreach ( $templates as $key => $template ) {
			$templates[ $key ]->preview = $this->core->content_email_preview_url( $template->preview, 'template' );
		}
		wp_send_json( $templates );
		die;
	}

	public function get_templates_gallery_callback() {
		$this->core->check_auth();
		$json     = array();
		$lang     = $this->core->get_current_language();
		$url      = $this->core->get_jackmail_url_ws() . 'gallery.php?lang=' . $lang;
		$headers  = array();
		$timeout  = 30;
		$response = $this->core->remote_get( $url, $headers, $timeout );
		if ( is_array( $response ) ) {
			if ( isset( $response['body'] ) ) {
				$json = json_decode( $response['body'], true );
				if ( $json === null ) {
					$json = array();
				}
				foreach ( $json as $key => $data ) {
					$json[ $key ]['preview'] = $this->core->get_jackmail_url_img() . $data['preview'];
				}
			}
		}
		wp_send_json( $json );
		die;
	}

	public function delete_template_callback() {
		$this->core->check_auth();
		global $wpdb;
		if ( isset( $_POST['id'] ) ) {
			$id_template = $this->core->request_text_data( $_POST['id'] );
			$sql         = "
			DELETE FROM `{$wpdb->prefix}jackmail_templates`
			WHERE `id` = %s";
			$wpdb->query( $wpdb->prepare( $sql, $id_template ) );
			if ( $wpdb->rows_affected === 0 ) {
				wp_send_json_error();
			}
			wp_send_json_success();
		}
		wp_send_json_error();
		die;
	}

	public function duplicate_template_callback() {
		$this->core->check_auth();
		global $wpdb;
		if ( isset( $_POST['id'] ) ) {
			$id_template = $this->core->request_text_data( $_POST['id'] );
			$sql         = "
			SELECT *
			FROM `{$wpdb->prefix}jackmail_templates`
			WHERE `id` = %s";
			$template    = $wpdb->get_row( $wpdb->prepare( $sql, $id_template ) );
			if ( isset( $template->id ) ) {
				$current_date_gmt = $this->core->get_current_time_gmt_sql();
				$preview          = $this->core->generate_jackmail_preview_filename();
				if ( $this->core->duplicate_preview( $template->preview, $preview ) ) {
					$insert_return = $wpdb->insert( "{$wpdb->prefix}jackmail_templates", array(
						'name'                 => $template->name,
						'content_email_json'   => $template->content_email_json,
						'content_email_html'   => $template->content_email_html,
						'content_email_txt'    => $template->content_email_txt,
						'content_email_images' => $template->content_email_images,
						'preview'              => $preview,
						'created_date_gmt'     => $current_date_gmt,
						'updated_date_gmt'     => $current_date_gmt
					) );
					$id_template   = strval( $wpdb->insert_id );
					if ( $insert_return !== false && $id_template !== false ) {
						wp_send_json_success();
					}
				}
			}
		}
		wp_send_json_error();
		die;
	}

}