'use strict';

angular.module( 'jackmail.controllers' ).controller( 'AccountConnectionController', [
	'$rootScope', 'UrlService', 'VerificationService',
	function ( $rootScope, UrlService, VerificationService ) {

		var vm = this;

		vm.current_page_type = 'popup';

		if ( window.location.href.indexOf( 'jackmail_installation' ) !== - 1 ) {
			vm.current_page_type = 'installation';
		}

		vm.email = $rootScope.settings.email;

		vm.create_login = {
			'password': '',
			'password_confirmation': '',
			'lastname': $rootScope.settings.lastname,
			'firstname': $rootScope.settings.firstname
		};

		vm.login_error = {
			'terms': false,
			'email': false,
			'password': false,
			'password_confirmation': false,
			'lastname': false,
			'firstname': false
		};

		vm.login = {
			'password': ''
		};

		vm.terms = false;

		function init_account_messages() {
			vm.account_created = false;
			vm.account_not_actived = false;
			vm.account_ids_not_valid = false;
			vm.account_resend_activation_email = false;
		}

		init_account_messages();

		vm.recheck_account_creation = function () {
			if ( vm.terms && vm.login_error.terms ) {
				vm.terms = false;
			}
			if ( vm.create_login.lastname !== '' && vm.login_error.lastname ) {
				vm.login_error.lastname = false;
			}
			if ( vm.create_login.firstname !== '' && vm.login_error.firstname ) {
				vm.login_error.firstname = false;
			}
			if ( VerificationService.email( vm.email ) && vm.login_error.email ) {
				vm.login_error.email = false;
			}
			if ( vm.create_login.password.length >= 8 && vm.login_error.password ) {
				vm.login_error.password = false;
			}
			if ( vm.create_login.password_confirmation === vm.create_login.password && vm.login_error.password_confirmation ) {
				vm.login_error.password_confirmation = false;
			}
		};

		vm.check_account_creation = function () {
			var login_error = {
				'terms': false,
				'email': false,
				'password': false,
				'password_confirmation': false,
				'lastname': false,
				'firstname': false
			};
			if ( ! vm.terms ) {
				login_error.terms = true;
				$rootScope.display_error( $rootScope.translations.you_must_accept_the_general_terms_of_jackmail );
			}
			if ( vm.create_login.lastname === '' ) {
				login_error.lastname = true;
			}
			if ( vm.create_login.firstname === '' ) {
				login_error.firstname = true;
			}
			if ( ! VerificationService.email( vm.email ) ) {
				login_error.email = true;
			}
			if ( vm.create_login.password.length < 8 ) {
				login_error.password = true;
			}
			if ( vm.create_login.password !== vm.create_login.password_confirmation ) {
				login_error.password_confirmation = true;
				$rootScope.display_error( $rootScope.translations.password_confirmation_doesn_t_match_password );
			}
			vm.login_error = login_error;
			if ( ! vm.login_error.terms && ! vm.login_error.lastname && ! vm.login_error.firstname && ! vm.login_error.email && ! vm.login_error.password && ! vm.login_error.password_confirmation ) {
				return true;
			}
			return false;
		};

		vm.account_creation = function () {
			init_account_messages();
			if ( vm.check_account_creation() ) {
				var data_parameters = {
					'email': vm.email,
					'password': vm.create_login.password,
					'lastname': vm.create_login.lastname,
					'firstname': vm.create_login.firstname
				};
				UrlService.post_data( 'jackmail_account_creation', data_parameters, function ( data ) {
					if ( data.success ) {
						vm.account_created = true;
						vm.show_login_form( 'connection' );
					}
					else {
						$rootScope.display_error( $rootScope.translations.an_error_occurred );
					}
				}, function () {

				} );
			}
		};

		vm.account_connection = function () {
			init_account_messages();
			var data_parameters = {
				'email': vm.email,
				'password': vm.login.password
			};
			UrlService.post_data( 'jackmail_account_connection', data_parameters, function ( data ) {
				if ( data.success ) {
					if ( angular.element( '.jackmail_notice' ).length !== 0 ) {
						window.location.reload();
					}
					else {
						$rootScope.settings.email = vm.email;
						$rootScope.settings.is_authenticated = true;
						$rootScope.hide_account_connection_popup();
					}
				}
				else {
					var message = '';
					if ( data.message === 'NOT_ACTIVED' ) {
						vm.account_not_actived = true;
					}
					else {
						vm.account_ids_not_valid = true;
					}
				}
			}, function () {

			} );
		};

		vm.resend_activation_email = function () {
			init_account_messages();
			var data_parameters = {
				'email': vm.email,
				'password': vm.login.password
			};
			UrlService.post_data( 'jackmail_account_resend_activation_email', data_parameters, function ( data ) {
				if ( data.success ) {
					vm.account_resend_activation_email = true;
				}
			}, function () {

			} );
		};

		vm.account_reset = function () {
			init_account_messages();
			var data_parameters = {
				'email': vm.email
			};
			if ( VerificationService.email( vm.email ) ) {
				UrlService.post_data( 'jackmail_account_reset', data_parameters, function ( data ) {
					vm.show_login_form( 'new_password_confirm' );
				}, function () {

				} );
			}
			else {
				vm.show_login_form( 'new_password_confirm' );
			}
		};

		vm.show_login_form = function ( page ) {
			$rootScope.show_account_connection_popup_form = page;
		};

		vm.check_uncheck_terms = function () {
			vm.terms = ! vm.terms;
		};

	} ] );

angular.module( 'jackmail.controllers' ).controller( 'CampaignsController', [
	'UrlService', '$timeout', '$window', '$rootScope', 'GridService', '$interval', '$filter',
	function ( UrlService, $timeout, $window, $rootScope, GridService, $interval, $filter ) {

		var vm = this;

		$rootScope.grid_service = new GridService();

		var all_campaigns = [];
		var campaigns = [];
		vm.nb_campaigns = - 1;

		vm.campaigns_grid = [];
		vm.nb_campaigns_grid = - 1;

		vm.campaigns_status = [
			$rootScope.translations.draft,
			$rootScope.translations.sent,
			$rootScope.translations.scheduled,
			$rootScope.translations.sending,
			$rootScope.translations.refused,
			$rootScope.translations.error
		];

		vm.filter = {
			'emailing': $rootScope.settings.campaign_emailing,
			'scenario': $rootScope.settings.campaign_scenario,
			'status': {
				'draft': true,
				'sent': true,
				'scheduled': true,
				'sending': true,
				'refused': true,
				'error': true
			},
			'selected_date1': $rootScope.settings.selected_date1,
			'selected_date2': $rootScope.settings.selected_date2
		};

		$interval( function () {
			var i;
			var nb_campaigns = all_campaigns.length;
			var refresh = false;
			for ( i = 0; i < nb_campaigns; i ++ ) {
				if ( all_campaigns[ i ].status === 'PROCESS_SENDING' || all_campaigns[ i ].status === 'PROCESS_SCHEDULED' ) {
					refresh = true;
					break;
				}
				if ( all_campaigns[ i ].status === 'SENDING' || all_campaigns[ i ].status === 'SCHEDULED' ) {
					if ( $filter( 'date1DiffDate2' )( all_campaigns[ i ].send_option_date_begin_gmt, $rootScope.settings.current_time ) < 0 ) {
						refresh = true;
						break;
					}
				}
			}
			if ( refresh ) {
				get_campaigns( true );
			}
			$rootScope.settings.current_time = $rootScope.add_date_interval( $rootScope.settings.current_time, 120 );
		}, 60000 );

		vm.create_campaign = function () {
			$rootScope.change_page_with_parameters( 'campaign', '0/contacts' );
		};

		vm.create_scenario = function () {
			$rootScope.change_page( 'scenario_choice' );
		};

		vm.go_edit_page = function ( type, url, json ) {
			if ( $rootScope.settings.emailbuilder_installed ) {
				$rootScope.change_page_with_parameters( type, url );
			} else {
				if ( json === '1' ) {
					$rootScope.display_emailbuilder_popup();
				} else {
					$rootScope.change_page_with_parameters( type, url );
				}
			}
		};

		function get_campaigns( refresh_status ) {
			var data_parameters = {
				'refresh_status': refresh_status
			};
			UrlService.post_data( 'jackmail_get_campaigns', data_parameters, function ( data ) {
				vm.nb_campaigns = data.length;
				all_campaigns = data;
				$timeout( function () {
					change_filter( false );
				} );
			}, function () {

			} );
		}

		get_campaigns( false );

		vm.change_filter_date = function ( date1, date2 ) {
			if ( vm.filter.selected_date1 !== date1 || vm.filter.selected_date2 !== date2 ) {
				vm.filter.selected_date1 = date1;
				vm.filter.selected_date2 = date2;
				change_filter( true );
			}
		};

		function change_filter( update_cookie ) {
			if ( vm.nb_campaigns !== - 1 ) {
				var selected_date1 = new Date( vm.filter.selected_date1.substring( 0, 10 ) );
				var selected_date2 = new Date( vm.filter.selected_date2.substring( 0, 10 ) );
				var i;
				var nb_campaigns = all_campaigns.length;
				var new_campaigns = [];
				for ( i = 0; i < nb_campaigns; i ++ ) {
					var campaign_date_gmt = new Date( all_campaigns[ i ].updated_date_gmt.substring( 0, 10 ) );
					if ( campaign_date_gmt >= selected_date1 && campaign_date_gmt <= selected_date2 ) {
						if ( vm.filter.emailing ) {
							if ( all_campaigns[ i ].type === 'campaign' ) {
								if ( ( vm.filter.status.draft && all_campaigns[ i ].status === 'DRAFT' )
									|| ( vm.filter.status.sent && all_campaigns[ i ].status === 'SENT' )
									|| ( vm.filter.status.scheduled && ( all_campaigns[ i ].status === 'PROCESS_SCHEDULED' || all_campaigns[ i ].status === 'SCHEDULED' ) )
									|| ( vm.filter.status.sending && ( all_campaigns[ i ].status === 'PROCESS_SENDING' || all_campaigns[ i ].status === 'SENDING' ) )
									|| ( vm.filter.status.refused && all_campaigns[ i ].status === 'REFUSED' )
									|| ( vm.filter.status.error && all_campaigns[ i ].status === 'ERROR' ) ) {
									new_campaigns.push( all_campaigns[ i ] );
								}
							}
						}
						if ( vm.filter.scenario ) {
							if ( all_campaigns[ i ].type.indexOf( 'scenario' ) !== - 1 ) {
								new_campaigns.push( all_campaigns[ i ] );
							}
						}
					}
				}
				campaigns = $rootScope.grid_service.grid_order_by( new_campaigns, 'updated_date_gmt', 'DESC' );
				generate_grid();
				if ( update_cookie ) {
					var data_parameters = {
						'selected_date1': vm.filter.selected_date1,
						'selected_date2': vm.filter.selected_date2,
						'campaign_emailing': vm.filter.emailing,
						'campaign_scenario': vm.filter.scenario
					};
					UrlService.post_data( 'jackmail_update_cookies', data_parameters, function ( data ) {

					}, function () {

					} );
				}
			}
		}

		vm.change_option = function ( option ) {
			vm.filter[ option ] = ! vm.filter[ option ];
			change_filter( true );
		};

		vm.filter_status = function ( key ) {
			if ( key === 0 ) {
				vm.filter.status.draft = ! vm.filter.status.draft;
			}
			else if ( key === 1 ) {
				vm.filter.status.sent = ! vm.filter.status.sent;
			}
			else if ( key === 2 ) {
				vm.filter.status.scheduled = ! vm.filter.status.scheduled;
			}
			else if ( key === 3 ) {
				vm.filter.status.sending = ! vm.filter.status.sending;
			}
			else if ( key === 4 ) {
				vm.filter.status.refused = ! vm.filter.status.refused;
			}
			else if ( key === 5 ) {
				vm.filter.status.error = ! vm.filter.status.error;
			}
			change_filter( false );
		};

		angular.element( $window ).on( 'resize', function () {
			generate_grid();
		} );

		function generate_grid() {
			vm.campaigns_grid = $rootScope.generate_grid( campaigns );
			vm.nb_campaigns_grid = vm.campaigns_grid.length;
		}

		vm.display_actions = function ( key, subkey ) {
			vm.campaigns_grid[ key ][ subkey ].show_actions = true;
		};

		vm.hide_actions = function ( key, subkey ) {
			vm.campaigns_grid[ key ][ subkey ].show_actions = false;
		};

		vm.duplicate_campaign = function ( id ) {
			var data_parameters = {
				'id': id
			};
			UrlService.post_data( 'jackmail_duplicate_campaign', data_parameters, function ( data ) {
				get_campaigns( false );
				$rootScope.display_success_error_writable( data.success, $rootScope.translations.the_campaign_has_been_duplicated );
			}, function () {

			} );
		};

		vm.cancel_scheduled_campaign = function ( id ) {
			var data_parameters = {
				'id': id
			};
			UrlService.post_data( 'jackmail_cancel_scheduled_campaign', data_parameters, function ( data ) {
				$rootScope.display_success_error( data.success, $rootScope.translations.the_campaign_has_been_canceled, $rootScope.translations.an_error_occurred_while_the_campaign_was_canceled );
				get_campaigns( false );
			}, function () {

			} );
		};

		vm.delete_campaign = function ( id ) {
			var data_parameters = {
				'id': id
			};
			UrlService.post_data( 'jackmail_delete_campaign', data_parameters, function ( data ) {
				$rootScope.display_success_error_writable( data.success, $rootScope.translations.the_campaign_has_been_deleted );
				get_campaigns( false );
			}, function () {

			} );
		};

		vm.delete_confirm = function ( key, subkey ) {
			vm.campaigns_grid[ key ][ subkey ].show_delete_confirmation = true;
		};

		vm.delete_cancel = function ( key, subkey ) {
			vm.campaigns_grid[ key ][ subkey ].show_delete_confirmation = false;
		};

		vm.delete_scenario = function ( id ) {
			var data_parameters = {
				'id': id
			};
			UrlService.post_data( 'jackmail_delete_scenario', data_parameters, function ( data ) {
				get_campaigns( false );
				$rootScope.display_success_error_writable( data.success, $rootScope.translations.the_scenario_was_deleted );
			}, function () {

			} );
		};

		vm.deactivate_scenario = function ( id ) {
			var data_parameters = {
				'id': id
			};
			UrlService.post_data( 'jackmail_deactivate_scenario', data_parameters, function ( data ) {
				get_campaigns( false );
				$rootScope.display_success_error( data.success, $rootScope.translations.the_scenario_was_disabled, '' );
			}, function () {

			} );
		};

	} ] );

angular.module( 'jackmail.controllers' ).controller( 'InstallationController', [
	'$rootScope', 'UrlService', 'PluginService', '$sce', '$window', '$location',
	function ( $rootScope, UrlService, PluginService, $sce, $window, $location ) {

		var vm = this;

		vm.current_step = 1;

		vm.plugins = PluginService.get_compatible_plugins();
		vm.nb_plugins_actived = - 1;

		$rootScope.emailbuilder_popup_licence = '';

		if ( $rootScope.settings.openssl_random_pseudo_bytes_extension_function_exists && $rootScope.settings.gzdecode_gzencode_function_exists && $rootScope.settings.base64_decode_base64_encode_function_exists && $rootScope.settings.json_encode_json_decode_function_exists && $rootScope.settings.jackmail_file_path_exists && $rootScope.settings.jackmail_file_path_writable ) {
			vm.configuration_ok = true;
		}
		else {
			vm.configuration_ok = false;
		}

		UrlService.post_data( 'jackmail_get_plugins_init', {}, function ( data ) {
			vm.plugins = PluginService.get_plugins_to_display( vm.plugins, data );
			vm.nb_plugins_actived = data.length;
		}, function () {

		} );

		UrlService.post_data( 'jackmail_get_emailbuilder_licence', {}, function ( data ) {
			if ( data.licence ) {
				if ( data.licence.length > 2000 && data.licence.indexOf( 'EmailBuilder' ) !== - 1 ) {
					$rootScope.emailbuilder_popup_licence = $sce.trustAsHtml( data.licence );
				}
			}
		}, function () {

		} );

		vm.reload_page = function () {
			window.location.reload();
		};

		vm.skip_account_creation = function () {
			if ( $rootScope.settings.analytics ) {
				$window.ga( 'send', {
					'hitType': 'event',
					'appName': 'Jackmail',
					'appVersion': $rootScope.settings.version,
					'screenName': $location.url(),
					'eventCategory': 'Ship account creation',
					'eventLabel': 'Ship account creation',
					'eventAction': 'click',
					'eventValue': 0
				} );
			}
			vm.go_step( 5 );
		};

		vm.go_step = function ( step ) {
			if ( vm.configuration_ok ) {
				if ( vm.current_step > step ) {
					vm.current_step = step;
				}
				else if ( step - vm.current_step === 1 ) {
					vm.current_step = step;
				}
				if ( vm.current_step === 5 ) {
					is_configured();
				}
			}
		};

		vm.install_emailbuilder = function () {
			UrlService.post_data( 'jackmail_install_emailbuilder', {}, function ( data ) {
				if ( data.success ) {
					vm.go_step( 3 );
				}
			}, function () {

			} );
		};

		vm.select_or_unselect_plugin = function ( plugin ) {
			vm.plugins[ plugin ].selected = ! vm.plugins[ plugin ].selected;
		};

		vm.import_plugins = function () {
			var plugins = [];
			if ( vm.plugins.bloom.active && vm.plugins.bloom.selected ) {
				plugins.push( 'bloom' );
			}
			if ( vm.plugins.contactform7.active && vm.plugins.contactform7.selected ) {
				plugins.push( 'contactform7' );
			}
			if ( vm.plugins.formidableforms.active && vm.plugins.formidableforms.selected ) {
				plugins.push( 'formidableforms' );
			}
			if ( vm.plugins.gravityforms.active && vm.plugins.gravityforms.selected ) {
				plugins.push( 'gravityforms' );
			}
			if ( vm.plugins.mailpoet2.active && vm.plugins.mailpoet2.selected ) {
				plugins.push( 'mailpoet2' );
			}
			if ( vm.plugins.mailpoet3.active && vm.plugins.mailpoet3.selected ) {
				plugins.push( 'mailpoet3' );
			}
			if ( vm.plugins.ninjaforms.active && vm.plugins.ninjaforms.selected ) {
				plugins.push( 'ninjaforms' );
			}
			if ( vm.plugins.popupbysupsystic.active && vm.plugins.popupbysupsystic.selected ) {
				plugins.push( 'popupbysupsystic' );
			}
			if ( vm.plugins.woocommerce.active && vm.plugins.woocommerce.selected ) {
				plugins.push( 'woocommerce' );
			}
			var data_parameters = {
				'plugins': $rootScope.join( plugins )
			};
			UrlService.post_data( 'jackmail_import_plugins', data_parameters, function () {

			}, function () {

			} );
			vm.go_step( 4 );
		};

		function is_configured() {
			UrlService.post_data( 'jackmail_is_configured', {}, function () {

			}, function () {

			} );
			
			angular.element( '#toplevel_page_jackmail_installation > a, #toplevel_page_jackmail_installation > ul.wp-submenu > li.current > a.current' ).attr( 'href', 'admin.php?page=jackmail_campaigns' );
		}

		vm.show_login_form = function ( page ) {
			vm.login_form = page;
		};

	} ] );

angular.module( 'jackmail.controllers' ).controller( 'ListAndCampaignController', [
	'$rootScope', 'UrlService', '$location', '$routeParams', '$timeout', 'VerificationService', '$interval', '$sce', '$filter', 'ExportService', 'GridService', '$window', 'EmailContentService',
	function ( $rootScope, UrlService, $location, $routeParams, $timeout, VerificationService, $interval, $sce, $filter, ExportService, GridService, $window, EmailContentService ) {

		var vm = this;

		$rootScope.grid_service = new GridService();

		var url_id = $routeParams.id;

		if ( $location.absUrl().indexOf( 'page=jackmail_list' ) !== - 1 ) {
			$rootScope.active_item_menu( 6 );
			vm.page_type = 'list';
			var url_create = 'jackmail_create_campaign';
			var url_get_list = 'jackmail_get_list';
			var url_export_list = 'jackmail_export_list';
			vm.campaign_type = 'list';
			vm.show_import_lists = false;
			vm.param_option = $routeParams.option;
			var show_name_popup = url_id === '0';
			vm.import_choices = [ $rootScope.translations.by_copy_pasting ];
		}
		else {
			$rootScope.active_item_menu( 2 );
			vm.page_type = 'campaign';
			if ( $location.absUrl().indexOf( 'page=jackmail_campaign' ) !== - 1 ) {
				vm.show_import_lists = false;
				var steps = [ 'contacts', 'create', 'checklist' ];
				var url_get_data = 'jackmail_get_campaign';
				var url_create = 'jackmail_create_campaign';
				var url_update = 'jackmail_update_campaign';
				var url_get_list = 'jackmail_get_campaign_contacts';
				var url_export_list = 'jackmail_export_campaign_contacts';
				var url_activate_link_tracking = 'jackmail_activate_campaign_link_tracking';
				var url_deactivate_link_tracking = 'jackmail_deactivate_campaign_link_tracking';
				var url_lists_available = 'jackmail_get_campaign_lists_available';
				var url_last_step_checker = 'jackmail_campaign_last_step_checker';
				var url_send_test = 'jackmail_send_campaign_test';
				vm.campaign_type = 'campaign';
				vm.import_choices = [ $rootScope.translations.from_a_list, $rootScope.translations.by_copy_pasting ];
			}
			else {
				if ( ! $rootScope.settings.emailbuilder_installed ) {
					$rootScope.go_page( 'campaigns' );
				}
				vm.show_import_lists = true;
				var steps = [ 'settings', 'contacts', 'create', 'checklist' ];
				var url_get_data = 'jackmail_get_scenario';
				var url_create = 'jackmail_create_scenario';
				var url_update = 'jackmail_update_scenario';
				var url_activate_link_tracking = 'jackmail_activate_scenario_link_tracking';
				var url_deactivate_link_tracking = 'jackmail_deactivate_scenario_link_tracking';
				var url_lists_available = 'jackmail_get_scenario_lists_available';
				var url_categories = 'jackmail_get_post_categories';
				if ( $routeParams.choice === 'woocommerce_automated_newsletter' ) {
					var url_categories = 'jackmail_get_woocommerce_product_categories';
				}
				var url_last_step_checker = 'jackmail_scenario_last_step_checker';
				var url_send_test = 'jackmail_send_scenario_test';
				vm.campaign_type = 'scenario';
			}
			var previous_page_is_list = ( document.referrer ).indexOf( 'jackmail_list' ) !== - 1;
			var show_name_popup = ( url_id === '0' && $routeParams.step === steps[ 0 ] && ! previous_page_is_list ) ? true : false;

			$timeout( function () {
				var border_width = 60;
				var step1_position = angular.element( '.jackmail_footer_middle > div > span:nth-child(1)' ).position().left;
				var step1_width = angular.element( '.jackmail_footer_middle > div > span:nth-child(1)' ).width();
				var step2_position = angular.element( '.jackmail_footer_middle > div > span:nth-child(3)' ).position().left;
				var step2_width = angular.element( '.jackmail_footer_middle > div > span:nth-child(3)' ).width();
				var step3_position = angular.element( '.jackmail_footer_middle > div > span:nth-child(5)' ).position().left;
				var step3_width = angular.element( '.jackmail_footer_middle > div > span:nth-child(5)' ).width();
				var step4_position = angular.element( '.jackmail_footer_middle > div > span:nth-child(7)' ).position().left;
				var step4_width = angular.element( '.jackmail_footer_middle > div > span:nth-child(7)' ).width();
				if ( border_width > step1_width ) {
					var border1_position = step1_position - (border_width - step1_width) / 2;
				} else {
					var border1_position = step1_position + (step1_width - border_width) / 2;
				}
				if ( border_width > step2_width ) {
					var border2_position = step2_position - (border_width - step2_width) / 2;
				} else {
					var border2_position = step2_position + (step2_width - border_width) / 2;
				}
				if ( border_width > step3_width ) {
					var border3_position = step3_position - (border_width - step3_width) / 2;
				} else {
					var border3_position = step3_position + (step3_width - border_width) / 2;
				}
				if ( border_width > step4_width ) {
					var border4_position = step4_position - (border_width - step4_width) / 2;
				} else {
					var border4_position = step4_position + (step4_width - border_width) / 2;
				}
				var css_begin = '.jackmail .jackmail_footer_middle .jackmail_footer_item';
				angular.element( 'body' ).append(
					'<style>' +
					css_begin + ':nth-child(1):hover ~ span.jackmail_footer_active_border {left: ' + border1_position + 'px !important;}' +
					css_begin + '.jackmail_footer_active:nth-child(1) ~ span.jackmail_footer_active_border {left: ' + border1_position + 'px;}' +
					css_begin + ':nth-child(3):hover ~ span.jackmail_footer_active_border {left: ' + border2_position + 'px !important;}' +
					css_begin + '.jackmail_footer_active:nth-child(3) ~ span.jackmail_footer_active_border {left: ' + border2_position + 'px;}' +
					css_begin + ':nth-child(5):hover ~ span.jackmail_footer_active_border {left: ' + border3_position + 'px !important;}' +
					css_begin + '.jackmail_footer_active:nth-child(5) ~ span.jackmail_footer_active_border {left: ' + border3_position + 'px;}' +
					css_begin + ':nth-child(7):hover ~ span.jackmail_footer_active_border {left: ' + border4_position + 'px !important;}' +
					css_begin + '.jackmail_footer_active:nth-child(7) ~ span.jackmail_footer_active_border {left: ' + border4_position + 'px;}' +
					'</style>'
				);
			} );

		}

		vm.common = {

			show_grid: - 1,

			list: {},

			current_contact: [],

			list_first_load: true,
			list_top_load: true,

			name_editing: false,

			manual_select_all: false,

			list_search: '',

			list_editable: - 1,
			list_full_editable: - 1,
			list_targeting: - 1,
			columns_editable: - 1,

			list_fields: [],
			list_fields_plus: [],

			export_select: [ $rootScope.translations.all, $rootScope.translations.selection ],

			show_copy_paste: false,

			copy_paste_content: '',

			show_name_popup: show_name_popup,

			hide_name_popup: function () {
				vm.common.show_name_popup = false;
			},

			display_copy_paste: function () {
				vm.common.show_copy_paste = true;
			},

			hide_copy_paste: function () {
				vm.common.show_copy_paste = false;
				vm.common.copy_paste_content = '';
			},

			confirm_copy_paste: function () {
				if ( vm.page_type === 'campaign' && url_id === '0' ) {
					UrlService.post_data( url_create, vm.c_common.campaign, function ( data ) {
						if ( data.success ) {
							vm.c_common.set_created_campaign_info( data );
							vm.common.add_contacts( vm.common.copy_paste_content );
						}
					}, function () {

					} );
				}
				else {
					vm.common.add_contacts( vm.common.copy_paste_content );
				}
			},

			go_back: function () {
				if ( vm.param_option ) {
					var params = vm.param_option.split( '-' );
					var index = vm.param_option.indexOf( 'campaign-' );
					if ( index !== - 1 ) {
						if ( params.length === 2 ) {
							$rootScope.change_page_with_parameters( 'campaign', params[ 1 ] + '/contacts' );
							return;
						}
					}
					index = vm.param_option.indexOf( 'scenario-' );
					if ( index !== - 1 ) {
						if ( params.length === 3 ) {
							$rootScope.change_page_with_parameters( 'scenario', params[ 1 ] + '/' + params[ 2 ] + '/contacts' );
							return;
						}
					}
				}
				$rootScope.change_page( 'lists' );
			},

			update_list_fields_array: function () {
				var list_fields = [];
				var list_fields_plus = [];
				list_fields.push( 'EMAIL' );
				var i = 1;
				while ( vm.common.list.list[ 'field' + i ] !== undefined ) {
					list_fields.push( vm.common.list.list[ 'field' + i ] );
					i ++;
				}
				if ( vm.page_type === 'campaign' ) {
					vm.c_common.refresh_content_email();
					var object = vm.c_common.campaign.object;
					var content_email_json = vm.c_common.campaign.content_email_json;
					var content_email_html = vm.c_common.campaign.content_email_html;
					var content_email_txt = vm.c_common.campaign.content_email_txt;
					var i;
					var nb_fields = vm.common.list_fields.length;
					for ( i = 0; i < nb_fields; i ++ ) {
						var old_column = vm.common.list_fields[ i ];
						var new_column = list_fields[ i ];
						if ( old_column !== undefined && new_column !== undefined && old_column !== new_column ) {
							
							var old_column_html = $rootScope.text_to_html( old_column );
							var new_column_html = $rootScope.text_to_html( new_column );
							object = $rootScope.replace_all( object, '((' + old_column + '))', '((' + new_column + '))' );
							content_email_json = $rootScope.replace_all( content_email_json, '((' + old_column + '))', '((' + new_column + '))' );
							content_email_html = $rootScope.replace_all( content_email_html, '((' + old_column + '))', '((' + new_column + '))' );
							content_email_txt = $rootScope.replace_all( content_email_txt, '((' + old_column + '))', '((' + new_column + '))' );
							content_email_json = $rootScope.replace_all( content_email_json, '((' + old_column_html + '))', '((' + new_column_html + '))' );
							content_email_html = $rootScope.replace_all( content_email_html, '((' + old_column_html + '))', '((' + new_column_html + '))' );
						}
					}
					var fields = angular.copy( list_fields );
					fields.splice( 0, 1 );
					vm.common.list.list.fields = $rootScope.join_fields( fields );
					list_fields_plus = angular.copy( list_fields );
					list_fields_plus.push( 'WEBCOPY_LINK' );
					list_fields_plus.push( 'UNSUBSCRIBE_LINK' );
					var object_changed = false;
					if ( vm.c_common.campaign.object !== object ) {
						vm.c_common.campaign.object = object;
						object_changed = true;
					}
					var content_email_changed = false;
					if ( vm.c_common.campaign.content_email_json !== content_email_json ) {
						vm.c_common.campaign.content_email_json = content_email_json;
						content_email_changed = true;
					}
					if ( vm.c_common.campaign.content_email_html !== content_email_html ) {
						vm.c_common.campaign.content_email_html = content_email_html;
						content_email_changed = true;
					}
					if ( vm.c_common.campaign.content_email_txt !== content_email_txt ) {
						vm.c_common.campaign.content_email_txt = content_email_txt;
						content_email_changed = true;
					}
					if ( content_email_changed || object_changed ) {
						if ( content_email_changed ) {
							EmailContentService.set( vm.c_common.campaign.content_email_json, vm.c_common.campaign.content_email_html, vm.c_common.campaign.content_email_txt );
						}
						vm.c_common.save_campaign( true );
					}
					EmailContentService.update_list_fields( list_fields_plus );
					vm.common.list_fields_plus = list_fields_plus;
				}
				vm.common.list_fields = angular.copy( list_fields );
			},

			get_list_data_reset: function () {
				$rootScope.grid_service.reset_begin();
				$rootScope.grid_service.reset_nb_lines_grid();
				vm.common.get_list_data();
				$rootScope.grid_service.set_nb_selected( 0 );
				vm.common.list_top_load = true;
				$rootScope.scroll_top();
				vm.common.manual_select_all = false;
				angular.element( '.jackmail_grid_content' ).scrollTop( 0 );
			},

			get_list_data_search_reset: function ( list_search ) {
				vm.common.list_search = list_search;
				vm.common.get_list_data_reset();
			},

			split_list_fields: function ( data ) {
				if ( data.list.fields !== '' ) {
					var fields = $rootScope.split_fields( data.list.fields );
					var i;
					var nb_fields = fields.length;
					var main_nb_fields = [];
					for ( i = 0; i < nb_fields; i ++ ) {
						data.list[ 'field' + ( i + 1 ) ] = fields[ i ];
						main_nb_fields.push( i + 1 );
					}
				}
				return data;
			},

			check_list_editable: function () {
				if ( vm.page_type === 'list' ) {
					if ( vm.common.list.list ) {
						if ( vm.common.list.list.type !== '' ) {
							vm.common.columns_editable = 0;
							vm.common.list_editable = 0;
							vm.common.list_full_editable = 0;
							vm.common.list_targeting = 0;
							
							
							return;
						}
					}
				}
				vm.common.list_editable = 1;
				vm.common.list_full_editable = 1;
				vm.common.list_targeting = 1;
				vm.common.columns_editable = 1;
			},

			display_grid: function () {
				vm.common.show_grid = 1;
				vm.common_list_detail.show_list_contact_detail = false;
				if ( vm.common.list.nb_contacts === '0' && vm.common.list_full_editable ) {
					var add_contact_manual = true;
					if ( vm.common.list.contacts[ 0 ] ) {
						if ( vm.common.list.contacts[ 0 ].email === '' ) {
							add_contact_manual = false;
						}
					}
					if ( add_contact_manual ) {
						vm.common.add_contact_manual();
					}
				}
				$timeout( function () {
					$rootScope.grid_service.refresh_grid_max_height( 'refresh' );
				} );
			},

			grid_select_or_unselect_all: function () {
				vm.common.list.contacts = $rootScope.grid_service.grid_select_or_unselect_all( vm.common.list.contacts );
				if ( $rootScope.grid_service.nb_selected > 0 ) {
					vm.common.manual_select_all = true;
				}
				else {
					vm.common.manual_select_all = false;
				}
			},

			grid_select_or_unselect_row: function ( key ) {
				vm.common.list.contacts = $rootScope.grid_service.grid_select_or_unselect_row( vm.common.list.contacts, key );
			},

			add_contacts_file: function ( event ) {
				if ( vm.page_type === 'campaign' && url_id === '0' ) {
					UrlService.post_data( url_create, vm.c_common.campaign, function ( data ) {
						if ( data.success ) {
							vm.c_common.set_created_campaign_info( data );
							add_contacts_file();
						}
						else {
							$rootScope.display_error_writable();
						}
					}, function () {

					} );
				}
				else {
					add_contacts_file();
				}
				function add_contacts_file() {
					var file_name = event.target.files[ 0 ].name;
					var file_extension = file_name.substring( file_name.lastIndexOf( '.' ), file_name.length );
					if ( file_extension === '.csv' || file_extension === '.CSV' || file_extension === '.txt' || file_extension === '.TXT' || file_extension === '.json' || file_extension === '.JSON' ) {
						var tmpPath = URL.createObjectURL( event.target.files[ 0 ] );
						UrlService.get_file_data( tmpPath, function ( data ) {
							if ( file_extension === '.json' || file_extension === '.JSON' ) {
								try {
									data = JSON.parse( data );
									var i;
									var nb_rows = data.length;
									var emails_import = '';
									var nb_columns = 0;
									var new_data = {
										'contacts': [],
										'list': {
											'email': '',
											'fields': []
										}
									};
									var nb_columns = 0;
									var fields = [];
									if ( nb_rows > 0 ) {
										var j = 0;
										for ( var key in data[ 0 ] ) {
											if ( j === 0 ) {
												new_data.list[ 'email' ] = key;
											}
											else {
												new_data.list[ 'fields' ].push( key );
												fields.push( key );
												nb_columns ++;
											}
											j ++;
										}
										for ( i = 0; i < nb_rows; i ++ ) {
											j = 0;
											var contact = {};
											for ( j = 0; j <= nb_columns; j ++ ) {
												if ( j === 0 ) {
													if ( data[ i ][ new_data.list[ 'email' ] ] ) {
														contact[ 'email' ] = data[ i ][ new_data.list[ 'email' ] ];
													}
													else {
														contact[ 'email' ] = '';
													}
												}
												else {
													if ( data[ i ][ fields[ j - 1 ] ] ) {
														contact[ 'field' + j ] = data[ i ][ fields[ j - 1 ] ];
													}
													else {
														contact[ 'field' + j ] = '';
													}
												}
											}
											new_data.contacts.push( contact );
										}
										new_data.list[ 'fields' ] = $rootScope.join_fields( new_data.list[ 'fields' ] );
										emails_import = ExportService.get_contact_file( new_data, 'all' );
									}
								}
								catch ( e ) {
									$rootScope.display_error( $rootScope.translations.the_json_file_is_not_valid );
									return;
								}
							}
							else {
								var emails_import = data;
							}
							vm.common.add_contacts( emails_import );
						}, function () {

						} );
					}
					else {
						$rootScope.display_error( $rootScope.translations.the_file_format_is_not_valid );
						$rootScope.$apply();
					}
				}
			},

			split_csv: function ( row, field_separator ) {
				for ( var foo = row.split( field_separator ), x = foo.length - 1, tl; x >= 0; x -- ) {
					if ( foo[ x ].replace( /"\s+$/, '"' ).charAt( foo[ x ].length - 1 ) == '"' ) {
						if ( (tl = foo[ x ].replace( /^\s+"/, '"' )).length > 1 && tl.charAt( 0 ) == '"' ) {
							foo[ x ] = foo[ x ].replace( /^\s*"|"\s*$/g, '' ).replace( /""/g, '"' );
						} else if ( x ) {
							foo.splice( x - 1, 2, [ foo[ x - 1 ], foo[ x ] ].join( field_separator ) );
						} else foo = foo.shift().split( field_separator ).concat( foo );
					} else foo[ x ].replace( /""/g, '"' );
				}
				return foo;
			},

			add_contacts: function ( emails_import ) {
				emails_import = emails_import.replace( /(\r\n|\r|\n)/g, "\n" );
				emails_import = emails_import.replace( /;\n/g, "\n" );
				
				var field_separator = ';';
				var emails_header = '';
				var rows = emails_import.split( "\n" );
				var nb_rows = rows.length;
				if ( nb_rows > 0 ) {
					var has_header = false;
					emails_header = rows[ 0 ];
					if ( emails_header.indexOf( '@' ) === - 1 || emails_header.indexOf( '.' ) === - 1 ) {
						has_header = true;
						rows.splice( 0, 1 );
					}
					var i;
					var nb_rows = rows.length;
					var has_at = false;
					var has_email = false;
					var email_position = - 1;
					var nb_test_rows = 50;
					if ( nb_test_rows > nb_rows ) {
						nb_test_rows = nb_rows;
					}
					var nb_semicolons = 0;
					var nb_commas = 0;
					var nb_tabs = 0;
					var nb_seps = 0;
					for ( i = 0; i < nb_test_rows; i ++ ) {
						nb_semicolons += rows[ i ].split( ';' ).length - 1;
						nb_commas += rows[ i ].split( ',' ).length - 1;
						nb_tabs += rows[ i ].split( '	' ).length - 1;
						nb_seps += rows[ i ].split( '|' ).length - 1;
					}
					if ( nb_commas > nb_semicolons ) {
						field_separator = ',';
					}
					if ( nb_tabs > nb_commas ) {
						field_separator = 'TAB';
					}
					if ( nb_seps > nb_tabs ) {
						field_separator = '|';
					}
					for ( i = nb_rows - 1; i >= 0; i -- ) {
						if ( rows[ i ].indexOf( '@' ) === - 1 || rows[ i ].indexOf( '.' ) === - 1 ) {
							rows.splice( i, 1 );
						}
						else {
							has_at = true;
							var row_splited = vm.common.split_csv( rows[ i ], field_separator );
							var nb_columns_row = row_splited.length;
							if ( email_position === - 1 ) {
								email_position = 0;
								var j;
								var email_position_found = false;
								for ( j = 0; j < nb_columns_row; j ++ ) {
									if ( VerificationService.email( $rootScope.replace_all( row_splited[ j ], '"', '' ) ) ) {
										email_position = j;
										email_position_found = true;
										break;
									}
								}
								if ( ! email_position_found ) {
									email_position = - 1;
								}
							}
							if ( email_position !== - 1 ) {
								if ( VerificationService.email( $rootScope.replace_all( row_splited[ email_position ], '"', '' ) ) ) {
									has_email = true;
								}
								else {
									rows.splice( i, 1 );
								}
							}
							else {
								rows.splice( i, 1 );
							}
						}
					}
					nb_rows = rows.length;
					if ( has_email && has_at ) {
						var part = 3000;
						var nb_parts = Math.ceil( nb_rows / part );
						var begin = 0;
						var i;
						var j;
						var data_parameters = [];
						for ( i = 0; i < nb_parts; i ++ ) {
							var emails_import = '';
							if ( has_header ) {
								emails_import += emails_header + "\n";
							}
							for ( j = begin; j < begin + part; j ++ ) {
								if ( rows[ j ] !== undefined ) {
									emails_import += rows[ j ] + "\n";
								} else {
									break;
								}
							}
							data_parameters.push( {
								'field_separator': field_separator,
								'email_position': email_position,
								'contacts': emails_import
							} );
							if ( vm.page_type === 'campaign' ) {
								data_parameters[ i ][ 'id_campaign' ] = url_id;
							}
							else {
								data_parameters[ i ][ 'id_list' ] = url_id;
							}
							begin = begin + part;
						}
						UrlService.post_multiple_data( 'jackmail_import_contacts', data_parameters, $rootScope.translations.downloading, function ( data ) {
							vm.common.hide_copy_paste();
							if ( vm.page_type === 'campaign' ) {
								vm.c_common.get_campaign_data( false );
							}
							vm.common.get_list_data_reset();
						}, function () {

						} );
					}
					else {
						var message = $rootScope.translations.no_valid_contacts_found;
						if ( has_at && ! has_email ) {
							message = message + ' ' + $rootScope.translations.fields_separator_must_be_semicolon_comma_vertical_bar_or_tabulation;
						}
						$rootScope.display_error( message );
					}
				}
				else {
					$rootScope.display_error( $rootScope.translations.content_is_empty );
				}
			},

			add_contact_manual: function () {
				var new_contact = {
					'email': '',
					'id_list': '0',
					'blacklist': '0'
				};
				var i;
				var nb_fields = vm.common.list_fields.length - 1;
				for ( i = 1; i < nb_fields; i ++ ) {
					new_contact[ 'field' + i ] = '';
				}
				vm.common.list.contacts.unshift( new_contact );
				angular.element( '.jackmail_grid_content' ).scrollTop( 0 );
				$timeout( function () {
					angular.element( '.jackmail_grid_content .jackmail_column_0:first input' ).focus();
				} );
			},

			focus_contact: function ( key ) {
				vm.common.current_contact = angular.copy( vm.common.list.contacts[ key ] );
			},

			blur_contact: function ( $event ) {
				if ( $event.keyCode === 13 ) {
					angular.element( $event.target ).blur();
				}
			},

			update_contact: function ( key, field_id ) {
				var current_contact = vm.common.current_contact;
				vm.common.list.contacts[ key ].email = vm.common.list.contacts[ key ].email.toLowerCase();
				var new_contact = vm.common.list.contacts[ key ];
				if ( VerificationService.differents_arrays( current_contact, new_contact ) ) {
					if ( field_id === - 1 ) {
						if ( vm.common.list.contacts[ key ].error ) {
							current_contact.email = '';
						}
						var data_parameters = {
							'email': current_contact.email,
							'new_email': new_contact.email
						};
						var url = 'jackmail_update_contact_email';
					}
					else {
						if ( vm.common.list.contacts[ key ].error ) {
							new_contact.email = '';
						}
						var data_parameters = {
							'email': new_contact.email,
							'field_id': field_id,
							'field': new_contact[ 'field' + field_id ]
						};
						var url = 'jackmail_update_contact_field';
					}
					if ( vm.common.list.contacts[ key ].error ) {
						vm.common.list.contacts[ key ].error = false;
					}
					if ( vm.page_type === 'campaign' ) {
						data_parameters[ 'id_campaign' ] = url_id;
					}
					else {
						data_parameters[ 'id_list' ] = url_id;
					}
					if ( ( url === 'jackmail_update_contact_field' && data_parameters.email !== '' && VerificationService.email( data_parameters.email ) ) || ( url === 'jackmail_update_contact_email' && data_parameters.new_email !== '' && VerificationService.email( data_parameters.new_email ) ) ) {
						UrlService.post_data( url, data_parameters, function ( data ) {
							if ( data.success ) {
								if ( ! VerificationService.email( current_contact.email ) ) {
									vm.common.list.nb_contacts ++;
									vm.common.list.nb_contacts_search ++;
								}
								if ( url === 'jackmail_update_contact_email' && field_id === - 1 ) {
									vm.common.list.contacts[ key ].id_list = '0';
									vm.common.list.contacts[ key ].blacklist = '0';
								}
								$rootScope.display_success( $rootScope.translations.contact_saved );
							}
							else {
								if ( current_contact.email === '' ) {
									vm.common.list.contacts[ key ].error = true;
								}
								else if ( field_id === - 1 ) {
									vm.common.list.contacts[ key ].email = current_contact.email;
								}
								$rootScope.display_error( data.message );
							}
						}, function () {

						} );
					}
					else {
						$rootScope.display_error( $rootScope.translations.a_valid_email_address_is_required );
						if ( current_contact.email === '' ) {
							vm.common.list.contacts[ key ].error = true;
						} else {
							vm.common.list.contacts[ key ].email = current_contact.email;
						}
					}
					vm.common.focus_contact( key );
				}
			},

			add_header_column: function () {
				if ( vm.page_type === 'list' ) {
					var data_parameters = {
						'id_list': url_id
					};
				}
				else {
					var data_parameters = {
						'id_campaign': url_id
					};
				}
				UrlService.post_data( 'jackmail_add_header_column', data_parameters, function ( data ) {
					if ( data.success ) {
						var id_columns = vm.common.list_fields.length;
						vm.common.list.list[ 'field' + id_columns ] = '';
						vm.common.update_list_fields_array();
						var i;
						var nb_contacts = vm.common.list.contacts.length;
						for ( i = 0; i < nb_contacts; i ++ ) {
							vm.common.list.contacts[ i ][ 'field' + id_columns ] = '';
						}
						$rootScope.grid_service.display_column( id_columns );
						$timeout( function () {
							angular.element( 'th.jackmail_column_' + id_columns + ' span.jackmail_input_edit_container > span.dashicons-edit' ).click();
						} );
					}
					else {
						vm.common.get_list_data_reset();
					}
					$rootScope.display_success_error( data.success, $rootScope.translations.the_column_has_been_saved, '' );
				} );
			},

			edit_header_column: function ( field_id ) {
				if ( vm.common.list.list[ 'field' + field_id ] === '' ) {
					var i;
					var nb_fields = vm.common.list_fields.length - 1;
					var fields = [];
					for ( i = 0; i < nb_fields; i ++ ) {
						fields.push( vm.common.list.list[ 'field' + ( i + 1 ) ] );
					}
					var fields_name_try = '( ' + ( nb_fields ) + ' )';
					var i = 1;
					while ( fields.indexOf( fields_name_try ) !== - 1 ) {
						fields_name_try = '( ' + ( nb_fields ) + '-' + i + ' )';
						i ++;
					}
					vm.common.list.list[ 'field' + field_id ] = fields_name_try;
					vm.common.update_list_fields_array();
				}
				else {
					vm.common.list.list[ 'field' + field_id ] = vm.common.list.list[ 'field' + field_id ].toUpperCase();
					vm.common.update_list_fields_array();
					var data_parameters = {
						'field_id': field_id,
						'field': vm.common.list.list[ 'field' + field_id ]
					};
					if ( vm.page_type === 'list' ) {
						data_parameters[ 'id_list' ] = url_id;
					}
					else {
						data_parameters[ 'id_campaign' ] = url_id;
					}
					UrlService.post_data( 'jackmail_edit_header_column', data_parameters, function ( data ) {
						if ( ! data.success ) {
							vm.common.get_list_data_reset();
						}
						$rootScope.display_success_error( data.success, $rootScope.translations.the_column_has_been_saved, data.message );
					} );
				}
			},

			delete_header_column: function ( field_id ) {
				var data_parameters = {
					'field_id': field_id
				};
				if ( vm.page_type === 'list' ) {
					data_parameters[ 'id_list' ] = url_id;
				}
				else {
					data_parameters[ 'id_campaign' ] = url_id;
				}
				UrlService.post_data( 'jackmail_delete_header_column', data_parameters, function ( data ) {
					vm.common.get_list_data_reset();
					$rootScope.display_success_error( data.success, $rootScope.translations.the_column_has_been_deleted, '' );
				} );
			},

			delete_contacts_selection_confirmation: function () {
				var filtering = false;
				if ( vm.common.list_search !== '' ) {
					filtering = true;
				}
				if ( vm.page_type === 'list' ) {
					if ( vm.only_list.targeting_rules.length > 0 ) {
						filtering = true;
					}
				}
				if ( ! filtering && $rootScope.grid_service.nb_selected === vm.common.list.contacts.length ) {
					if ( vm.page_type === 'campaign' ) {
						var data_parameters = {
							'id_campaign': url_id
						};
					}
					else {
						var data_parameters = {
							'id_list': url_id
						};
					}
					UrlService.post_data( 'jackmail_delete_all_contacts', data_parameters, function ( data ) {
						vm.common.get_list_data_reset();
						var message = $rootScope.translations.deleted_contacts;
						if ( $rootScope.grid_service.nb_selected === 1 ) {
							message = $rootScope.translations.deleted_contact;
						}
						$rootScope.display_success_error( data.success, message, '' );
					}, function () {

					} );
				}
				else {
					var i;
					var nb_contacts = vm.common.list.contacts.length;
					var contacts_selection_and_contacts_selection_type = vm.common.get_contacts_selection_and_contacts_selection_type();
					var data_parameters = {
						'search': vm.common.list_search,
						'targeting_rules': vm.common.get_targeting_rules(),
						'contacts_selection': JSON.stringify( contacts_selection_and_contacts_selection_type[ 'contacts_selection' ] ),
						'contacts_selection_type': contacts_selection_and_contacts_selection_type[ 'contacts_selection_type' ]
					};
					if ( vm.page_type === 'list' ) {
						data_parameters[ 'id_list' ] = url_id;
					}
					else if ( vm.page_type === 'campaign' ) {
						data_parameters[ 'id_campaign' ] = url_id;
					}
					UrlService.post_data( 'jackmail_delete_contacts_selection', data_parameters, function ( data ) {
						vm.common.get_list_data_reset();
						var message = $rootScope.translations.deleted_contacts;
						if ( contacts_selection_and_contacts_selection_type[ 'contacts_selection' ].length === 1 ) {
							message = $rootScope.translations.deleted_contact;
						}
						$rootScope.display_success_error( data.success, message, '' );
					}, function () {

					} );
				}

			},

			range_by: function ( i ) {
				if ( vm.common.list.nb_contacts_search < 100 ) {
					vm.common.list.contacts = $rootScope.grid_service.grid_range( vm.common.list.contacts, i );
				}
				else {
					$rootScope.grid_service.grid_range_from_server( i );
					vm.common.get_list_data_reset();
				}
			},

			export_all_or_export_selection: function ( key ) {
				if ( key === 0 ) {
					vm.common.export_all();
				}
				else {
					vm.common.export_selection();
				}
			},

			export_all: function () {
				if ( vm.common.list.nb_contacts_search > 0 ) {
					if ( vm.common.list.nb_contacts_search <= 100 ) {
						ExportService.export_contact_file( vm.common.list, 'all' );
					} else {
						var part = 3000;
						var nb_parts = Math.ceil( vm.common.list.nb_contacts_search / part );
						var begin = 0;
						var i;
						var data_parameters = [];
						for ( i = 0; i < nb_parts; i ++ ) {
							data_parameters.push( {
								'id': url_id,
								'begin': begin.toString(),
								'sort_by': '',
								'sort_order': '',
								'search': vm.common.list_search,
								'targeting_rules': vm.common.get_targeting_rules()
							} );
							begin = begin + part;
						}
						UrlService.post_multiple_data( url_export_list, data_parameters, $rootScope.translations.downloading, function ( data ) {
							ExportService.export_contact_file_multiple_data( data );
						}, function () {

						} );
					}
				}
				else {
					$rootScope.display_error( $rootScope.translations.list_is_empty );
				}
			},

			export_selection: function () {
				if ( $rootScope.grid_service.nb_selected > 0 ) {
					if ( vm.common.list.nb_contacts_search <= 100 ) {
						var contacts = vm.common.list.contacts;
						var i;
						var nb_contacts = contacts.length;
						var export_contacts = [];
						for ( i = 0; i < nb_contacts; i ++ ) {
							if ( contacts[ i ].selected ) {
								export_contacts.push( contacts[ i ] );
							}
						}
						var data = {
							'list': vm.common.list.list,
							'contacts': export_contacts
						};
						ExportService.export_contact_file( data, 'all' );
					}
					else {
						if ( $rootScope.grid_service.nb_selected === vm.common.list.contacts.length ) {
							vm.common.export_all();
						}
						else if ( vm.common.manual_select_all && $rootScope.grid_service.nb_selected !== vm.common.list.contacts.length ) {
							var contacts_selection_and_contacts_selection_type = vm.common.get_contacts_selection_and_contacts_selection_type();
							var contacts_selection = contacts_selection_and_contacts_selection_type[ 'contacts_selection' ];
							if ( contacts_selection.length > 0 ) {
								var nb_rows_file = vm.common.list.nb_contacts_search - contacts_selection.length;
								var part = 3000;
								var nb_parts = Math.ceil( nb_rows_file / part );
								var begin = 0;
								var i;
								var data_parameters = [];
								for ( i = 0; i < nb_parts; i ++ ) {
									data_parameters.push( {
										'begin': begin.toString(),
										'contacts_selection': JSON.stringify( contacts_selection ),
										'search': vm.common.list_search,
										'targeting_rules': vm.common.get_targeting_rules()
									} );
									if ( vm.page_type === 'list' ) {
										data_parameters[ i ][ 'id_list' ] = url_id;
									}
									else if ( vm.page_type === 'campaign' ) {
										data_parameters[ i ][ 'id_campaign' ] = url_id;
									}
									begin = begin + part;
								}
								UrlService.post_multiple_data( 'jackmail_export_contacts_selection', data_parameters, $rootScope.translations.downloading, function ( data ) {
									ExportService.export_contact_file_multiple_data( data );
								} );
							}
						}
						else {
							ExportService.export_contact_file( vm.common.list, 'selection' );
						}
					}
				}
				else {
					$rootScope.display_error( $rootScope.translations.selection_is_empty );
				}
			},


			get_contacts_selection_and_contacts_selection_type: function () {
				var i;
				var nb_contacts = vm.common.list.contacts.length;
				var contacts_selection = [];
				var manual_select_all = vm.common.manual_select_all;
				for ( i = 0; i < nb_contacts; i ++ ) {
					if ( ( manual_select_all && ! vm.common.list.contacts[ i ].selected ) || ( ! manual_select_all && vm.common.list.contacts[ i ].selected ) ) {
						contacts_selection.push( vm.common.list.contacts[ i ].email );
					}
				}
				if ( contacts_selection.length === nb_contacts && manual_select_all === 'SELECTED' ) {
					return {
						'contacts_selection': [],
						'contacts_selection_type': 'NOT_SELECTED'
					};
				}
				return {
					'contacts_selection': contacts_selection,
					'contacts_selection_type': manual_select_all ? 'NOT_SELECTED' : 'SELECTED'
				};
			},

			get_list_data: function ( callback ) {
				var data_parameters = {
					'id': url_id,
					'begin': $rootScope.grid_service.begin,
					'sort_by': $rootScope.grid_service.grid_range_by_item,
					'sort_order': $rootScope.grid_service.grid_range_by_order ? 'DESC' : 'ASC',
					'search': vm.common.list_search,
					'targeting_rules': vm.common.get_targeting_rules()
				};
				UrlService.post_data( url_get_list, data_parameters, function ( data ) {
					if ( callback !== undefined ) {
						callback();
					}
					if ( vm.page_type === 'list' && data.list === null ) {
						
						$rootScope.go_page( 'lists' );
					}
					else {
						data = vm.common.split_list_fields( data );
						if ( vm.common.list_first_load || vm.common.list_top_load ) {
							vm.common.list = angular.copy( data );
							$rootScope.select_name_popup( show_name_popup );
							vm.common.check_list_editable();
							var show_grid = 0;
							if ( vm.common.list.nb_contacts > 0 || ! vm.common.list_full_editable || vm.common.list_fields.length > 1 || vm.common.show_grid === 1 || ( vm.page_type === 'campaign' && ( vm.c_common.campaign.id_lists !== '0' || vm.show_import_lists ) ) ) {
								if ( url_id === '0' ) {
									show_grid = 0;
								} else {
									show_grid = 1;
								}
							}
							vm.common.show_grid = show_grid;
							vm.common_list_detail.show_list_contact_detail = false;
							vm.common.update_list_fields_array();
							vm.common.list_top_load = false;
							if ( vm.common.show_grid === 1 ) {
								vm.common.display_grid();
							}
							if ( vm.common.list_first_load ) {
								vm.common.list_first_load = false;
								$rootScope.grid_service.init_columns_list( $rootScope.split_fields( vm.common.list.list.fields ) );
							}
						}
						else {
							var select_all = false;
							if ( $rootScope.grid_service.nb_selected === vm.common.list.contacts.length || vm.common.manual_select_all ) {
								select_all = true;
							}
							data.contacts = $rootScope.grid_service.merge_arrays( vm.common.list.contacts, data.contacts );
							vm.common.list = angular.copy( data );
							vm.common.update_list_fields_array();
							if ( select_all ) {
								vm.list.contacts = $rootScope.grid_service.grid_select_all_new( vm.common.list.contacts );
							}
						}
					}
				}, function () {

				} );
				if ( vm.page_type === 'campaign' ) {
					vm.c_common.get_lists_available();
				}
			},

			get_targeting_rules: function () {
				if ( vm.page_type === 'list' ) {
					var i;
					var nb_targeting_rules = vm.only_list.targeting_rules.length;
					var targeting_rules = [];
					for ( i = 0; i < nb_targeting_rules; i ++ ) {
						if ( vm.only_list.targeting_rules[ i ].rule_content !== '' || vm.only_list.targeting_rules[ i ].rule_option === 'EMPTY' || vm.only_list.targeting_rules[ i ].rule_option === 'UNSUBSCRIBED' ) {
							targeting_rules.push( vm.only_list.targeting_rules[ i ] );
						}
					}
					return JSON.stringify( targeting_rules );
				}
				return '[]';
			}

		};

		vm.common_list_detail = {

			show_list_contact_detail: false,

			list_contact_detail_email: '',
			list_contact_detail_email_key: 0,

			email_lists_detail: [],
			email_lists_detail_saved: [],

			email_lists_detail_current_list_name: '',
			email_lists_detail_current_list_id: 0,
			email_lists_detail_current_list_type: '',

			show_list_contact_detail_columns: false,

			nb_sends: '',
			nb_opens: '',
			nb_clicks: '',

			synthesis_timeline: [],
			nb_synthesis_timeline: 0,

			display_list_contact_detail: function ( key ) {
				vm.common_list_detail.list_contact_detail_email_key = key;
				vm.common_list_detail.list_contact_detail_email = vm.common.list.contacts[ key ].email;
				if ( VerificationService.email( vm.common_list_detail.list_contact_detail_email ) ) {
					var data_parameters = {
						'email': vm.common_list_detail.list_contact_detail_email
					};
					if ( vm.page_type === 'campaign' ) {
						data_parameters[ 'id_campaign' ] = url_id;
					}
					UrlService.post_data( 'jackmail_get_email_lists_detail', data_parameters, function ( data ) {
						var i;
						var nb_lists = data.length;
						if ( nb_lists > 0 ) {
							vm.common.show_grid = - 1;
							vm.common_list_detail.show_list_contact_detail = true;
							for ( i = 0; i < nb_lists; i ++ ) {
								data[ i ].fields = $rootScope.split_fields( data[ i ].fields );
								if ( data[ i ].id === url_id && data[ i ].type === vm.page_type ) {
									data[ i ].checked = true;
									vm.common_list_detail.email_lists_detail_current_list_name = data[ i ].name;
									vm.common_list_detail.email_lists_detail_current_list_id = data[ i ].id;
									vm.common_list_detail.email_lists_detail_current_list_type = data[ i ].type;
								}
							}
							vm.common_list_detail.get_email_detail_data();
							vm.common_list_detail.email_lists_detail = angular.copy( data );
							vm.common_list_detail.email_lists_detail_saved = angular.copy( data );
						}
					}, function () {

					} );
				}
			},

			hide_list_contact_detail: function () {
				vm.common.show_grid = 1;
				vm.common_list_detail.show_list_contact_detail = false;
				vm.common_list_detail.show_list_contact_detail_columns = false;
			},

			select_email_list_details: function ( key ) {
				var i;
				var nb_lists = vm.common_list_detail.email_lists_detail.length;
				for ( i = 0; i < nb_lists; i ++ ) {
					vm.common_list_detail.email_lists_detail[ i ].checked = false;
				}
				vm.common_list_detail.email_lists_detail[ key ].checked = ! vm.common_list_detail.email_lists_detail[ key ].checked;
				vm.common_list_detail.email_lists_detail_current_list_name = vm.common_list_detail.email_lists_detail[ key ].name;
				vm.common_list_detail.email_lists_detail_current_list_id = vm.common_list_detail.email_lists_detail[ key ].id;
				vm.common_list_detail.email_lists_detail_current_list_type = vm.common_list_detail.email_lists_detail[ key ].type;
				vm.common_list_detail.get_email_detail_data();
			},

			get_email_detail_data: function () {
				if ( vm.common_list_detail.email_lists_detail_current_list_type === 'list' ) {
					var data_parameters = {
						'id': vm.common_list_detail.email_lists_detail_current_list_id,
						'email': vm.common_list_detail.list_contact_detail_email
					};
					UrlService.post_data( 'jackmail_get_email_detail', data_parameters, function ( data ) {
						vm.common_list_detail.nb_sends = data.sends;
						vm.common_list_detail.nb_opens = data.opens;
						vm.common_list_detail.nb_clicks = data.clicks;
						vm.common_list_detail.synthesis_timeline = $rootScope.generate_timeline_data( data.timeline, true, true, true );
						vm.common_list_detail.nb_synthesis_timeline = vm.common_list_detail.synthesis_timeline.length;
					}, function () {

					} );
				}
				else {
					vm.common_list_detail.synthesis_timeline = [];
					vm.common_list_detail.nb_synthesis_timeline = 0;
				}
			},

			display_hide_list_contact_detail_columns: function () {
				vm.common_list_detail.show_list_contact_detail_columns = ! vm.common_list_detail.show_list_contact_detail_columns;
			},

			update_detail_contact: function ( key, subkey ) {
				var email = vm.common_list_detail.email_lists_detail[ key ].contact.email;
				var field_id = subkey.substr( 5 );
				var field = vm.common_list_detail.email_lists_detail[ key ].contact[ subkey ];
				var id = vm.common_list_detail.email_lists_detail_current_list_id;
				var type = vm.common_list_detail.email_lists_detail_current_list_type;
				var field_saved = vm.common_list_detail.email_lists_detail_saved[ key ].contact[ subkey ];
				if ( field !== field_saved ) {
					var data_parameters = {
						'email': email,
						'field_id': field_id,
						'field': field
					};
					if ( type === 'list' ) {
						data_parameters[ 'id_list' ] = id;
					}
					else {
						data_parameters[ 'id_campaign' ] = id;
					}
					var url = 'jackmail_update_contact_field';
					UrlService.post_data( url, data_parameters, function ( data ) {
						$rootScope.display_success_error( data.success, $rootScope.translations.contact_saved, '' );
						if ( data.success ) {
							if ( id === url_id && type === vm.page_type ) {
								vm.common.list.contacts[ vm.common_list_detail.list_contact_detail_email_key ][ subkey ] = field;
							}
							vm.common_list_detail.email_lists_detail_saved = angular.copy( vm.common_list_detail.email_lists_detail );
						}
					}, function () {

					} );
				}
			},

			unsubscribe_contact: function ( key ) {
				var url = 'jackmail_unsubscribe_contact';
				vm.common_list_detail.unsubscribe_or_unblacklist_contact( key, url );
			},

			unblacklist_contact: function ( key ) {
				var url = 'jackmail_unblacklist_contact';
				vm.common_list_detail.unsubscribe_or_unblacklist_contact( key, url );
			},

			unsubscribe_or_unblacklist_contact: function ( key, url ) {
				var email = vm.common_list_detail.email_lists_detail[ key ].contact.email;
				var id = vm.common_list_detail.email_lists_detail_current_list_id;
				var type = vm.common_list_detail.email_lists_detail_current_list_type;
				var data_parameters = {
					'email': email
				};
				if ( type === 'list' ) {
					data_parameters[ 'id_list' ] = id;
				}
				else {
					data_parameters[ 'id_campaign' ] = id;
				}
				UrlService.post_data( url, data_parameters, function ( data ) {
					$rootScope.display_success_error( data.success, $rootScope.translations.contact_saved, '' );
					if ( data.success ) {
						if ( id === url_id && type === vm.page_type ) {
							var blacklist = '0';
							if ( url === 'jackmail_unsubscribe_contact' ) {
								blacklist = '1';
							}
							vm.common.list.contacts[ vm.common_list_detail.list_contact_detail_email_key ].blacklist = blacklist;
						}
						vm.common_list_detail.email_lists_detail[ key ].contact.blacklist = blacklist;
						vm.common_list_detail.email_lists_detail_saved = angular.copy( vm.common_list_detail.email_lists_detail );
					}
				}, function () {

				} );
			}

		};

		if ( vm.page_type === 'list' ) {

			$timeout( function () {
				vm.common.get_list_data();
			} );

			var show_connectors = false;

			var targeting_rule_options_email = [ '=', '!=', 'LIKE', 'UNSUBSCRIBED' ];
			var targeting_rule_options = [ '=', '!=', 'LIKE', 'EMPTY', 'NUMBER>', 'NUMBER<', 'DATE>', 'DATE<' ];

			var targeting_rule_and_or = [ 'AND', 'OR' ];

			var focused_list_name = '';

			vm.only_list = {

				show_connectors: false,

				connectors_example: {
					'action_add': {
						'url': '',
						'parameters': ''
					},
					'action_update': {
						'url': '',
						'parameters': ''
					},
					'action_delete': {
						'url': '',
						'parameters': ''
					}
				},

				connectors_actions: [
					{ 'id': 'action_add', 'name': $rootScope.translations.add_a_recipient },
					{ 'id': 'action_update', 'name': $rootScope.translations.update_contact_details },
					{ 'id': 'action_delete', 'name': $rootScope.translations.delete_a_contact }
				],

				display_connectors_action: 'action_add',

				connectors_actived: - 1,

				show_targeting_settings: false,

				targeting_rules: [],

				targeting_rule_options_email: [
					$rootScope.translations.is_equals_to,
					$rootScope.translations.is_differents_from,
					$rootScope.translations.contains,
					$rootScope.translations.is_an_opt_out
				],

				targeting_rule_options: [
					$rootScope.translations.is_equals_to,
					$rootScope.translations.is_differents_from,
					$rootScope.translations.contains,
					$rootScope.translations.is_empty,
					'(' + $rootScope.translations.number + ') ' + $rootScope.translations.superiors_to,
					'(' + $rootScope.translations.number + ') ' + $rootScope.translations.inferiors_from,
					'(' + $rootScope.translations.date + ') ' + $rootScope.translations.superiors_to,
					'(' + $rootScope.translations.date + ') ' + $rootScope.translations.inferiors_from
				],

				targeting_rule_and_or: [ $rootScope.translations.and, $rootScope.translations.or ],

				targeting_rule_current: '',

				nb_searched_targeting_rules: 0,

				display_hide_connectors: function () {
					vm.only_list.show_connectors = ! vm.only_list.show_connectors;
					if ( vm.only_list.show_connectors && ! show_connectors ) {
						show_connectors = true;
						var data_parameters = {
							'id': url_id
						};
						UrlService.post_data( 'jackmail_display_connectors', data_parameters, function ( data ) {
							if ( data.active ) {
								vm.only_list.connectors_actived = 1;
							}
							else {
								vm.only_list.connectors_actived = 0;
							}
						}, function () {

						} );
					}
					var i = 0;
					var nb_fields = vm.common.list_fields.length;
					var columns_add = '';
					var columns_update = '';
					for ( i = 1; i < nb_fields; i ++ ) {
						columns_add += '&<span>' + encodeURIComponent( vm.common.list_fields[ i ].toLowerCase() ) + '</span>=' + 'value';
						columns_update += '&<span>' + encodeURIComponent( vm.common.list_fields[ i ].toLowerCase() ) + '</span>=' + 'new_value';
					}
					if ( columns_update === '' ) {
						var action_update = [
							$sce.trustAsHtml(
								'<span>email</span>=' + $rootScope.translations.email_at_example_com + '&<span>new_email</span>=' + $rootScope.translations.new_at_example_com
							)
						];
					}
					else {
						var action_update = [
							$sce.trustAsHtml(
								'<span>email</span>=' + $rootScope.translations.email_at_example_com + '&<span>new_email</span>=' + $rootScope.translations.new_at_example_com + columns_update
							),
							$sce.trustAsHtml(
								'<span>email</span>=' + $rootScope.translations.email_at_example_com + columns_update
							)
						];
					}
					vm.only_list.connectors_example = {
						'action_add': {
							'url': $sce.trustAsHtml( $rootScope.settings.website_url + '?<span>jackmail</span>&<span>action</span>=add&<span>list</span>=' + vm.common.list.list.connector_key ),
							'parameters': [ $sce.trustAsHtml( '<span>email</span>=' + $rootScope.translations.email_at_example_com + columns_add ) ]
						},
						'action_update': {
							'url': $sce.trustAsHtml( $rootScope.settings.website_url + '?<span>jackmail</span>&<span>action</span>=update&<span>list</span>=' + vm.common.list.list.connector_key ),
							'parameters': action_update
						},
						'action_delete': {
							'url': $sce.trustAsHtml( $rootScope.settings.website_url + '?<span>jackmail</span>&<span>action</span>=delete&<span>list</span>=' + vm.common.list.list.connector_key ),
							'parameters': [ $sce.trustAsHtml( '<span>email</span>=' + $rootScope.translations.email_at_example_com ) ]
						}
					};
					vm.only_list.connectors_calculate_position();
				},

				connectors_calculate_position: function () {
					$timeout( function () {
						var height = angular.element( '.jackmail_confirmation_connectors' ).height();
						angular.element( '.jackmail_confirmation_connectors' ).css( { 'margin-top': '-' + ( angular.element( '.jackmail_confirmation_connectors' ).height() / 2 ) + 'px' } );
					} );
				},

				ok_list_name_popup: function () {
					var data_parameters = {
						'name': vm.common.list.list.name
					};
					UrlService.post_data( 'jackmail_create_list', data_parameters, function ( data ) {
						if ( data.success ) {
							url_id = data.id;
							vm.common.get_list_data_reset();
							var params = data.id;
							if ( vm.param_option ) {
								params += '/' + vm.param_option;
							}
							UrlService.change_url_parameters_without_reload_and_history( 'list/' + params );
							vm.common.show_name_popup = false;
						} else {
							$rootScope.display_error( $rootScope.translations.list_name_must_be_unique );
						}
					}, function () {

					} );

				},

				cancel_list_name_popup: function () {
					$rootScope.change_page( 'lists' );
				},

				focus_list_name: function () {
					focused_list_name = vm.common.list.list.name;
					angular.element( '.jackmail_name .jackmail_content_editable' ).focus();
					vm.common.name_editing = true;
				},

				blur_list_name: function () {
					if ( vm.common.name_editing ) {
						var data_parameters = {
							'id': url_id,
							'name': vm.common.list.list.name
						};
						UrlService.post_data( 'jackmail_save_name', data_parameters, function ( data ) {
							$rootScope.display_success_error( data.success, $rootScope.translations.the_name_of_the_list_has_been_saved, $rootScope.translations.list_name_must_be_unique );
							if ( ! data.success ) {
								vm.common.list.list.name = focused_list_name;
							}
						}, function () {

						} );
						vm.common.name_editing = false;
					}
				},

				create_campaign_with_list: function () {
					var sender_email = $rootScope.settings.email;
					var sender_name = '';
					if ( $rootScope.settings.firstname !== '' || $rootScope.settings.lastname !== '' ) {
						var sender_name = $rootScope.settings.firstname + ' ' + $rootScope.settings.lastname;
					}
					var data_parameters = {
						'sender_name': sender_name,
						'sender_email': sender_email
					};
					$rootScope.display_success( $rootScope.translations.creating_campaign );
					UrlService.post_data( 'jackmail_create_campaign_with_data', data_parameters, function ( data ) {
						if ( data.success ) {
							var contacts_selection_and_contacts_selection_type = vm.common.get_contacts_selection_and_contacts_selection_type();
							var id_campaign = data.id;
							var data_parameters = {
								'id_campaign': id_campaign,
								'id_lists': $rootScope.join( [ url_id ] ),
								'search': vm.common.list_search,
								'targeting_rules': vm.common.get_targeting_rules(),
								'contacts_selection': JSON.stringify( contacts_selection_and_contacts_selection_type[ 'contacts_selection' ] ),
								'contacts_selection_type': contacts_selection_and_contacts_selection_type[ 'contacts_selection_type' ]
							};
							UrlService.post_data( 'jackmail_set_campaign_lists', data_parameters, function ( data ) {
								$rootScope.change_page_with_parameters( 'campaign', id_campaign + '/create' );
							}, function () {

							} );
						}
						else {
							$rootScope.display_error_writable();
						}
					}, function () {

					} );
				},

				display_hide_targeting_settings: function () {
					vm.only_list.show_targeting_settings = ! vm.only_list.show_targeting_settings;
					$timeout( function () {
						$rootScope.grid_service.refresh_grid_max_height( 'refresh' );
					} );
					if ( vm.only_list.targeting_rules.length === 0 ) {
						vm.only_list.add_targeting_rule();
					}
				},

				add_targeting_rule: function () {
					$timeout( function () {
						$rootScope.grid_service.refresh_grid_max_height( 'refresh' );
					} );
					vm.only_list.targeting_rules.push( {
						'rule_and_or': 'AND',
						'rule_column': 0,
						'rule_option': '=',
						'rule_content': ''
					} );
				},

				get_targeting_rule_current: function ( key ) {
					vm.only_list.targeting_rule_current = key;
				},

				remove_targeting_rule: function ( key ) {
					vm.only_list.targeting_rules.splice( key, 1 );
					if ( vm.only_list.targeting_rules.length === 0 ) {
						vm.only_list.show_targeting_settings = false;
					}
					$timeout( function () {
						$rootScope.grid_service.refresh_grid_max_height( 'refresh' );
					} );
					vm.only_list.get_list_data_targeting_rules_reset();
				},

				select_targeting_rule_and_or: function ( key ) {
					change_and_get_list_data_targeting_rules_reset_if_needed( 'rule_and_or', key );
				},

				select_targeting_rule_column: function ( key ) {
					change_and_get_list_data_targeting_rules_reset_if_needed( 'rule_column', key );
				},

				select_targeting_rule_option: function ( key ) {
					change_and_get_list_data_targeting_rules_reset_if_needed( 'rule_option', key );
				},

				get_targeting_rule_option_title: function ( rule_column, rule_option ) {
					var select = targeting_rule_options;
					if ( rule_column === 0 ) {
						select = targeting_rule_options_email;
					}
					var key = 0;
					var i;
					var length = select.length;
					for ( i = 0; i < length; i ++ ) {
						if ( select[ i ] === rule_option ) {
							key = i;
							break;
						}
					}
					if ( rule_column === 0 ) {
						return vm.only_list.targeting_rule_options_email[ key ];
					}
					else {
						return vm.only_list.targeting_rule_options[ key ];
					}
				},

				get_targeting_rule_and_or_title: function ( rule_column, rule_option ) {
					var select = targeting_rule_and_or;
					var key = 0;
					var i;
					var length = select.length;
					for ( i = 0; i < length; i ++ ) {
						if ( select[ i ] === rule_option ) {
							key = i;
							break;
						}
					}
					return vm.only_list.targeting_rule_and_or[ key ];
				},

				get_list_data_targeting_rules_reset: function () {
					$rootScope.display_success( $rootScope.translations.searching );
					var i;
					var nb_targeting_rules = vm.only_list.targeting_rules.length;
					var nb_searched_targeting_rules = 0;
					for ( i = 0; i < nb_targeting_rules; i ++ ) {
						if ( vm.only_list.targeting_rules[ i ].rule_content !== '' || vm.only_list.targeting_rules[ i ].rule_option === 'EMPTY' || vm.only_list.targeting_rules[ i ].rule_option === 'UNSUBSCRIBED' ) {
							nb_searched_targeting_rules ++;
						}
					}
					vm.only_list.nb_searched_targeting_rules = nb_searched_targeting_rules;
					vm.common.get_list_data_reset();
				}

			};

			vm.create_list = function () {
				vm.common.display_grid();
			};

			vm.display_import = function () {
				vm.common.display_copy_paste();
			};

			function change_and_get_list_data_targeting_rules_reset_if_needed( current, key ) {
				var value;
				var refresh = false;
				if ( current === 'rule_and_or' ) {
					value = targeting_rule_and_or[ key ];
					refresh = true;
				}
				else if ( current === 'rule_column' ) {
					value = key;
					refresh = true;
				}
				else if ( current === 'rule_option' ) {
					if ( vm.only_list.targeting_rules[ vm.only_list.targeting_rule_current ].rule_column === 0 ) {
						value = targeting_rule_options_email[ key ];
					}
					else {
						value = targeting_rule_options[ key ];
					}
					if ( value === 'EMPTY' || value === 'UNSUBSCRIBED' ) {
						vm.only_list.targeting_rules[ vm.only_list.targeting_rule_current ].rule_content = '';
					}
					if ( vm.only_list.targeting_rules[ vm.only_list.targeting_rule_current ].rule_option !== value ) {
						if ( vm.only_list.targeting_rules[ vm.only_list.targeting_rule_current ].rule_content !== '' || ( vm.only_list.targeting_rules[ vm.only_list.targeting_rule_current ].rule_option === 'UNSUBSCRIBED' || value === 'UNSUBSCRIBED' ) ) {
							refresh = true;
						}
					}
				}
				if ( vm.only_list.targeting_rules[ vm.only_list.targeting_rule_current ][ current ] !== value ) {
					if ( current === 'rule_column' ) {
						if ( vm.only_list.targeting_rules[ vm.only_list.targeting_rule_current ][ current ] === 0 && value !== 0 ) {
							var current_rule_option = vm.only_list.targeting_rules[ vm.only_list.targeting_rule_current ].rule_option;
							if ( current_rule_option !== '=' && current_rule_option !== '!=' && current_rule_option !== 'LIKE' ) {
								vm.only_list.targeting_rules[ vm.only_list.targeting_rule_current ].rule_option = '=';
							}
						}
					}
					vm.only_list.targeting_rules[ vm.only_list.targeting_rule_current ][ current ] = value;
					if ( refresh ) {
						vm.only_list.get_list_data_targeting_rules_reset();
					}
				}
			}

		}

		if ( vm.page_type === 'campaign' ) {

			var saved_campaign = {};

			$timeout( function () {

				vm.c_common.get_campaign_data( true );

				UrlService.get_file_data( $rootScope.settings.jackmail_url + 'libs/emoji_pretty_light3.json', function ( data ) {
					vm.c_common.emojis = JSON.parse( data );
				}, function () {

				} );

			} );

			var url = $location.path().split( '/' );

			vm.c_common = {

				campaign: {},

				lists: [],

				current_step_name: url[ url.length - 1 ],

				error: {
					'sender': '',
					'reply_to': '',
					'recipients': '',
					'object': '',
					'content_email': '',
					'send_option_date': ''
				},

				customized_columns_used: [],

				customized_columns_unknown: [],

				campaign_data_checked: false,

				content_email_changes: false,

				content_email_types: '',

				content_email_nb_links: '0',

				content_email_unsubscribe_link: false,

				current_content_email_type: '',

				sending_campaign: false,

				test_campaign_recipient: $rootScope.settings.email,

				save_campaign_success: true,

				nb_contacts_from_lists: '0',
				nb_selected_lists: '0',

				show_reply_to: false,

				import_lists_grid_height: {},

				show_templates: false,

				display_send_test_confirmation: false,

				emoji_categories: [
					$rootScope.translations.emoticons,
					$rootScope.translations.weather,
					$rootScope.translations.sport,
					$rootScope.translations.heart,
					$rootScope.translations.office,
					$rootScope.translations.mail,
					$rootScope.translations.business,
					$rootScope.translations.gestures,
					$rootScope.translations.tools,
					$rootScope.translations.characters,
					$rootScope.translations.games,
					$rootScope.translations.music,
					$rootScope.translations.monsters,
					$rootScope.translations.astronomy,
					$rootScope.translations.hobbies,
					$rootScope.translations.health,
					$rootScope.translations.clothing,
					$rootScope.translations.hours,
					$rootScope.translations.nature,
					$rootScope.translations.food,
					$rootScope.translations.transport,
					$rootScope.translations.technology,
					$rootScope.translations.animals,
					$rootScope.translations.tourism,
					$rootScope.translations.objects,
					$rootScope.translations.symbols
				],

				emojis: [],

				emoji_categorie_selected_key: 0,

				display_emojis_dropdown: false,

				create_list_and_go: function () {
					if ( vm.campaign_type === 'campaign' ) {
						var back = 'campaign-' + url_id;
					} else {
						var back = 'scenario-' + vm.only_scenario.scenario_choice + '-' + url_id;
					}
					$rootScope.change_page_with_parameters( 'list', '0/' + back );
				},

				show_hide_emojis_dropdown: function () {
					vm.c_common.display_emojis_dropdown = ! vm.c_common.display_emojis_dropdown;
				},

				hide_emojis_dropdown: function () {
					vm.c_common.display_emojis_dropdown = false;
				},

				select_emoji_categorie: function ( id ) {
					vm.c_common.emoji_categorie_selected_key = id;
				},

				insert_emoji: function ( key ) {
					var i;
					var nb_emojis = vm.c_common.emojis.length;
					var j = 0;
					for ( i = 0; i < nb_emojis; i ++ ) {
						if ( vm.c_common.emojis[ i ].category === vm.c_common.emoji_categorie_selected_key ) {
							if ( j === key ) {
								vm.c_common.campaign.object = vm.c_common.campaign.object + String.fromCodePoint( "0x" + vm.c_common.emojis[ i ].unified );
								break;
							}
							j ++;
						}
					}
				},

				object_to_save: function ( object ) {
					object = object.replace( /&nbsp;/g, ' ' );
					
					object = object.replace( /<img draggable=\"false\" class=\"emoji\" alt=\"/g, '' );
					
					object = object.replace( /\" src=\"[a-zA-Z0-9\:\/\.]*\"\>/g, '' );
					object = angular.element( '<div>' + object + '</div>' ).text();
					return object;
				},

				object_trust_html: function ( object ) {
					return $sce.trustAsHtml( object );
				},

				set_created_campaign_info: function ( data ) {
					if ( data.id !== undefined ) {
						url_id = data.id;
						vm.c_common.campaign.id = data.id;
					}
					vm.c_common.campaign.content_size = data.content_size;
					if ( vm.c_common.campaign.content_email_json !== data.content_email_json ) {
						vm.c_common.campaign.content_email_json = data.content_email_json;
						EmailContentService.set_emailbuilder_json( vm.c_common.campaign.content_email_json );
					}
					if ( vm.c_common.campaign.content_email_html !== data.content_email_html ) {
						vm.c_common.campaign.content_email_html = data.content_email_html;
						EmailContentService.set_email_content_editor( vm.c_common.current_content_email_type, vm.c_common.campaign.content_email_html );
					}
					if ( vm.c_common.campaign.content_email_txt !== data.content_email_txt ) {
						vm.c_common.campaign.content_email_txt = data.content_email_txt;
						EmailContentService.set_email_content_editor( vm.c_common.current_content_email_type, vm.c_common.campaign.content_email_txt );
					}
					change_url();
				},

				close_templates: function () {
					vm.c_common.show_templates = false;
					EmailContentService.display( vm.c_common.current_content_email_type );
				},

				import_template_in_campaign: function ( template_type, id ) {
					if ( template_type === 'template' ) {
						var data_parameters = {
							'id': id,
							'link_tracking': vm.c_common.campaign.link_tracking
						};
						var url = 'jackmail_get_template_json';
					}
					else {
						var data_parameters = {
							'gallery_id': id,
							'link_tracking': vm.c_common.campaign.link_tracking
						};
						var url = 'jackmail_get_gallery_template_json';
					}
					UrlService.post_data( url, data_parameters, function ( data ) {
						var content = EmailContentService.init_and_display_emailbuilder( data.content_email_json );
						vm.c_common.campaign.content_email_json = content.content_email_json;
						vm.c_common.campaign.content_email_html = '';
						vm.c_common.campaign.content_email_txt = '';
						vm.c_common.current_content_email_type = content.current_content_email_type;
						vm.content_email_types = content.content_email_types;
						vm.c_common.show_templates = false;
					}, function () {

					} );
				},

				insert_subject_customize: function ( key, title ) {
					vm.c_common.campaign.object = vm.c_common.campaign.object + ' ((' + title + '))';
				},

				calculate_import_lists_grid_height: function () {
					grid_height();
					angular.element( $window ).on( 'resize', function () {
						grid_height();
					} );
					function grid_height() {
						var row_height = 54;
						var height = angular.element( $window ).height();
						var div_height = height - 350;
						var import_lists_grid_height = 0;
						if ( div_height < 160 ) {
							import_lists_grid_height = row_height * 3;
						}
						else {
							var nb_lists = vm.c_common.lists.length;
							if ( nb_lists < 3 ) {
								import_lists_grid_height = row_height * 3;
							}
							else {
								var max_rows = Math.floor( div_height / row_height );
								if ( nb_lists > max_rows ) {
									import_lists_grid_height = row_height * max_rows;
								}
								else {
									import_lists_grid_height = row_height * nb_lists;
								}
							}
						}
						vm.c_common.import_lists_grid_height = {
							'height': import_lists_grid_height + 'px'
						};
					}
				},

				focus_campaign_name: function () {
					angular.element( '.jackmail_name .jackmail_content_editable' ).focus();
					vm.common.name_editing = true;
				},

				blur_campaign_name: function () {
					if ( vm.c_common.campaign.name === '' ) {
						vm.c_common.campaign.name = $rootScope.translations.campaign_with_no_name;
					}
					vm.common.name_editing = false;
				},

				select_list: function ( key ) {
					vm.c_common.lists[ key ].selected = ! vm.c_common.lists[ key ].selected;
					vm.c_common.selected_lists_total();
				},

				selected_lists_total: function () {
					var i;
					var nb = vm.c_common.lists.length;
					var nb_contacts_from_lists = 0;
					var nb_selected_lists = 0;
					for ( i = 0; i < nb; i ++ ) {
						if ( vm.c_common.lists[ i ].selected ) {
							nb_contacts_from_lists = nb_contacts_from_lists + parseInt( vm.c_common.lists[ i ].nb_display_contacts );
							nb_selected_lists ++;
						}
					}
					vm.c_common.nb_contacts_from_lists = nb_contacts_from_lists;
					vm.c_common.nb_selected_lists = nb_selected_lists;
					vm.c_common.campaign.id_lists = vm.c_common.get_selected_lists();
				},

				get_campaign_data: function ( init ) {
					var data_parameters = {
						'id': url_id
					};
					if ( vm.campaign_type === 'scenario' ) {
						data_parameters[ 'choice' ] = vm.only_scenario.scenario_choice;
					}
					UrlService.post_data( url_get_data, data_parameters, function ( data ) {
						if ( data === null ) {
							$rootScope.go_page( 'campaigns' );
						}
						if ( ! $rootScope.settings.emailbuilder_installed ) {
							if ( data.content_email_json !== '' ) {
								$rootScope.go_page( 'campaigns' );
							}
						}
						if ( url_id === '0' ) {
							data.sender_email = $rootScope.settings.email;
							if ( $rootScope.settings.firstname !== '' || $rootScope.settings.lastname !== '' ) {
								data.sender_name = $rootScope.settings.firstname + ' ' + $rootScope.settings.lastname;
							}
							if ( vm.campaign_type === 'scenario' && data.content_email_json === '' ) {
								EmailContentService.set_nb_posts_content( data.nb_posts_content );
							}
						}
						vm.c_common.campaign = data;
						if ( vm.c_common.campaign.link_tracking === '1' ) {
							EmailContentService.activate_link_tracking();
						}
						else {
							EmailContentService.deactivate_link_tracking();
						}
						if ( url_id === '0' ) {
							if ( $rootScope.settings.emailbuilder_installed ) {
								vm.c_common.current_content_email_type = 'emailbuilder';
							}
							else {
								vm.c_common.current_content_email_type = 'html';
							}
							vm.c_common.content_email_types = '';
							vm.c_common.content_email_nb_links = 0;
							vm.c_common.content_email_unsubscribe_link = false;
							if ( vm.c_common.campaign.content_email_json !== '' ) {
								var content = EmailContentService.set( vm.c_common.campaign.content_email_json, vm.c_common.campaign.content_email_html, vm.c_common.campaign.content_email_txt );
								
							}
						}
						else {
							var content = EmailContentService.set( vm.c_common.campaign.content_email_json, vm.c_common.campaign.content_email_html, vm.c_common.campaign.content_email_txt );
							vm.c_common.current_content_email_type = content.current_content_email_type;
							vm.c_common.content_email_types = content.content_email_types;
							vm.c_common.content_email_nb_links = content.content_email_nb_links;
							vm.c_common.content_email_unsubscribe_link = content.content_email_unsubscribe_link;
						}
						saved_campaign = angular.copy( vm.c_common.campaign );
						if ( init ) {
							if ( vm.campaign_type === 'scenario' ) {
								vm.c_common.get_lists_available();
								UrlService.post_data( url_categories, {}, function ( data ) {
									vm.only_scenario.post_categories_available = vm.only_scenario.get_post_categories_select( data );
								}, function () {

								} );
							}
							if ( vm.campaign_type === 'campaign' ) {
								vm.common.get_list_data();
							}
							else {
								vm.common.list_fields = [ 'EMAIL' ];
								vm.common.list_fields_plus = [ 'EMAIL' ];
							}
						}
						else {
							vm.c_common.go_step( vm.c_common.current_step_name );
						}
					}, function () {

					} );
				},

				get_lists_available: function () {
					var data_parameters = {
						'id': vm.c_common.campaign.id
					};
					UrlService.post_data( url_lists_available, data_parameters, function ( data ) {
						vm.c_common.lists = data;
						if ( vm.campaign_type === 'campaign' ) {
							if ( vm.c_common.campaign ) {
								if ( vm.c_common.campaign.id_lists ) {
									var selected_lists = vm.c_common.get_selected_lists();
									if ( selected_lists !== vm.c_common.campaign.id_lists ) {
										vm.c_common.campaign.id_lists = selected_lists;
										saved_campaign.id_lists = selected_lists;
									}
								}
							}
						}
						vm.c_common.selected_lists_total();
						if ( vm.campaign_type === 'scenario' ) {
							vm.c_common.display_import_lists();
						}
						vm.c_common.go_step( vm.c_common.current_step_name );
					}, function () {

					} );
				},

				go_step_correct_sender: function ( error ) {
					if ( error !== '' ) {
						vm.c_common.go_step( 'create' );
						$timeout( function () {
							angular.element( 'span[input-value="lc.c_common.campaign.sender_email"] span[ng-click="djce.show_editable()"] > span' ).click();
						}, 250 );
					}
				},

				go_step_correct_reply_to: function ( error ) {
					if ( error !== '' ) {
						vm.c_common.go_step( 'create' );
						if ( ! vm.c_common.show_reply_to ) {
							vm.c_common.display_hide_reply_to();
						}
						$timeout( function () {
							angular.element( '.jackmail span[input-value="lc.c_common.campaign.reply_to_email"] span[ng-click="djce.show_editable()"] > span' ).click();
						}, 250 );
					}
				},

				go_step_correct_recipients: function ( error ) {
					if ( error !== '' ) {
						vm.c_common.go_step( 'contacts' );
						$timeout( function () {
							angular.element( '.jackmail .jackmail_grid_content tr:first-child td.jackmail_column_0_email input' ).focus();
						} );
					}
				},

				go_step_correct_object: function ( error ) {
					if ( error !== '' ) {
						vm.c_common.go_step( 'create' );
						$timeout( function () {
							angular.element( '.jackmail span[input-value="lc.c_common.campaign.object"] span[ng-click="djce.show_editable()"] > span' ).click();
						}, 250 );
					}
				},

				go_step_correct_content_email: function ( error ) {
					if ( error !== '' ) {
						vm.c_common.go_step( 'create' );
					}
				},

				go_step: function ( step_name ) {
					if ( vm.campaign_type === 'campaign' && vm.c_common.current_step_name === 'contacts' && vm.common.show_grid === 0 && vm.show_import_lists ) {
						vm.only_campaign.import_selected_lists( step_name );
					} else {
						vm.c_common.campaign_data_checked = false;
						var step = steps.indexOf( step_name ) + 1;
						if ( step_name === 'create' ) {
							if ( ! vm.c_common.show_templates ) {
								EmailContentService.display( vm.c_common.current_content_email_type );
							}
						}
						else {
							EmailContentService.hide_emailbuilder();
							if ( vm.campaign_type === 'scenario' ) {
								vm.c_common.campaign.nb_posts_content = EmailContentService.get_nb_posts_content();
							}
						}
						var current_step_name_temp = vm.c_common.current_step_name;
						if ( steps.indexOf( vm.c_common.current_step_name ) >= step ) {
							vm.c_common.current_step_name = step_name;
						}
						else {
							vm.c_common.current_step_name = step_name;
							if ( vm.c_common.current_step_name === 'checklist' ) {
								var save_needed = check_save_campaign_needed();
								
								if ( ! save_needed ) {
									vm.c_common.check_campaign_data_ws();
								}
							}
						}
						if ( current_step_name_temp !== vm.c_common.current_step_name ) {
							$rootScope.scroll_top();
						}
						change_url();
					}
				},

				previous_step: function () {
					var step_name = steps[ steps.indexOf( vm.c_common.current_step_name ) - 1 ];
					vm.c_common.go_step( step_name );
				},

				next_step: function () {
					var step_name = steps[ steps.indexOf( vm.c_common.current_step_name ) + 1 ];
					vm.c_common.go_step( step_name );
				},

				save_campaign: function ( refresh_emailbuilder_changes ) {
					if ( refresh_emailbuilder_changes ) {
						vm.c_common.refresh_content_email();
					}
					if ( vm.campaign_type === 'scenario' ) {
						vm.c_common.campaign.nb_posts_content = EmailContentService.get_nb_posts_content();
					}
					if ( url_id === '0' ) {
						var data_parameters = angular.copy( vm.c_common.campaign );
						data_parameters[ 'object' ] = vm.c_common.object_to_save( vm.c_common.campaign.object );
						UrlService.post_data( url_create, data_parameters, function ( data ) {
							vm.c_common.save_campaign_success = data.success;
							if ( data.success ) {
								vm.c_common.set_created_campaign_info( data );
								saved_campaign = angular.copy( vm.c_common.campaign );
							}
							$rootScope.display_success_error_writable( data.success, $rootScope.translations.the_campaign_has_been_saved );
							vm.c_common.check_campaign_data_ws();
							
						}, function () {

						} );
					}
					else {
						saved_campaign = angular.copy( vm.c_common.campaign );
						var data_parameters = angular.copy( vm.c_common.campaign );
						data_parameters[ 'object' ] = vm.c_common.object_to_save( vm.c_common.campaign.object );
						UrlService.post_data( url_update, data_parameters, function ( data ) {
							vm.c_common.save_campaign_success = data.success;
							if ( data.success ) {
								vm.c_common.set_created_campaign_info( data );
							}
							$rootScope.display_success_error_writable( data.success, $rootScope.translations.the_campaign_has_been_saved );
							vm.c_common.check_campaign_data_ws();
							
						}, function () {

						} );
					}
				},

				display_hide_reply_to: function () {
					vm.c_common.show_reply_to = ! vm.c_common.show_reply_to;
					EmailContentService.hide_emailbuilder_menu();
					if ( vm.c_common.current_content_email_type === 'html' ) {
						var height = parseInt( angular.element( '.jackmail #jackmail_content_email > div' ).css( 'height' ) );
						if ( vm.c_common.show_reply_to ) {
							height = height - 50;
						}
						else {
							height = height + 50;
						}
						angular.element( '.jackmail #jackmail_content_email > div' ).css( { 'height': height + 'px' } );
					}
					$timeout( function () {
						EmailContentService.refresh_email_content_height();
					}, 500 );
				},

				change_current_content_email_type: function ( current_content_email_type ) {
					if ( vm.c_common.current_content_email_type !== current_content_email_type ) {
						var content = EmailContentService.change_current_content_email_type( vm.c_common.current_content_email_type, current_content_email_type, vm.c_common.campaign.content_email_json, vm.c_common.campaign.content_email_html, vm.c_common.campaign.content_email_txt );
						vm.c_common.campaign.content_email_json = content.content_email_json;
						vm.c_common.campaign.content_email_html = content.content_email_html;
						vm.c_common.campaign.content_email_txt = content.content_email_txt;
						vm.c_common.current_content_email_type = content.current_content_email_type;
						vm.c_common.content_email_types = content.content_email_types;
						vm.c_common.content_email_nb_links = content.content_email_nb_links;
						vm.c_common.content_email_unsubscribe_link = content.content_email_unsubscribe_link;
					}
				},

				insert_email_content_editor_customize: function ( key, title ) {
					
					EmailContentService.insert_email_content_editor_customize( vm.c_common.current_content_email_type, title );
				},

				get_selected_lists: function () {
					var i;
					var nb_lists = vm.c_common.lists.length;
					var selected_lists = [];
					for ( i = 0; i < nb_lists; i ++ ) {
						if ( vm.c_common.lists[ i ].selected ) {
							selected_lists.push( vm.c_common.lists[ i ].id );
						}
					}
					selected_lists = $rootScope.join( selected_lists );
					return selected_lists;
				},

				display_import_lists: function () {
					vm.c_common.calculate_import_lists_grid_height();
					vm.common.show_grid = 0;
					vm.common_list_detail.show_list_contact_detail = false;
					vm.show_import_lists = true;
				},

				check_campaign_data_ws: function () {
					if ( vm.c_common.current_step_name === 'checklist' ) {
						if ( url_id === '0' ) {
							vm.c_common.check_campaign_data();
							vm.c_common.campaign_data_checked = true;
						}
						else {
							var data_parameters = {
								'id': url_id
							};
							UrlService.post_data( url_last_step_checker, data_parameters, function ( data ) {
								vm.shared_campaign.checked_campaign_data = data;
								vm.c_common.check_campaign_data();
								vm.c_common.campaign_data_checked = true;
							}, function () {

							} );
						}
					}
				},

				check_campaign_data: function () {
					var error = {
						'sender': '',
						'sender_warning': false,
						'reply_to': '',
						'recipients': '',
						'object': '',
						'content_email': '',
						'send_option_date': ''
					};
					var customized_columns_used = [];
					var customized_columns_unknown = [];
					if ( vm.c_common.campaign.sender_name === '' || vm.c_common.campaign.sender_email === '' ) {
						error.sender = $rootScope.translations.the_sender_field_is_required;
					}
					else if ( ! VerificationService.email( vm.c_common.campaign.sender_email ) ) {
						error.sender = $rootScope.translations.a_valid_sender_email_address_is_required;
					}
					if ( error.sender === '' ) {
						if ( vm.c_common.campaign.sender_email.indexOf( '@outlook.' ) !== - 1 ) {
							error.sender_warning = true;
						} else if ( vm.c_common.campaign.sender_email.indexOf( '@hotmail.' ) !== - 1 ) {
							error.sender_warning = true;
						} else if ( vm.c_common.campaign.sender_email.indexOf( '@gmail.' ) !== - 1 ) {
							error.sender_warning = true;
						} else if ( vm.c_common.campaign.sender_email.indexOf( '@yahoo.' ) !== - 1 ) {
							error.sender_warning = true;
						}
					}
					if ( ( vm.c_common.campaign.reply_to_name === '' && vm.c_common.campaign.reply_to_email !== '' ) || ( vm.c_common.campaign.reply_to_name !== '' && vm.c_common.campaign.reply_to_email === '' ) ) {
						error.reply_to = $rootScope.translations.the_reply_to_field_is_required;
					}
					else if ( vm.c_common.campaign.reply_to_email !== '' ) {
						if ( ! VerificationService.email( vm.c_common.campaign.reply_to_email ) ) {
							error.reply_to = $rootScope.translations.a_valid_reply_to_email_address_is_required;
						}
					}
					if ( vm.c_common.campaign.object === '' ) {
						error.object = $rootScope.translations.the_subject_field_is_required;
					}
					if ( vm.c_common.campaign.content_email_json === '' && vm.c_common.campaign.content_email_html === '' && vm.c_common.campaign.content_email_txt === '' ) {
						error.content_email = $rootScope.translations.the_message_field_is_required;
					}
					if ( ! vm.c_common.campaign.content_size ) {
						error.content_email = $rootScope.translations.message_content_is_too_large;
					}
					if ( vm.campaign_type === 'campaign' ) {
						if ( vm.shared_campaign.checked_campaign_data.nb_contacts_valids < 1 ) {
							error.recipients = $rootScope.translations.no_recipient_included_in_the_campaign;
						}
					}
					else {
						if ( vm.c_common.nb_selected_lists < 1 ) {
							error.recipients = $rootScope.translations.no_list_associated_in_the_campaign;
						}
					}
					vm.c_common.error = angular.copy( error );

					var i;
					var nb_list_fields = vm.common.list_fields.length;
					for ( i = 0; i < nb_list_fields; i ++ ) {
						var column = vm.common.list_fields[ i ];
						var column_with_separator = '((' + column + '))';
						if ( ( $rootScope.html_to_text( vm.c_common.campaign.content_email_json + '' + vm.c_common.campaign.content_email_html + '' + vm.c_common.campaign.content_email_txt ) ).indexOf( column_with_separator ) !== - 1 ) {
							customized_columns_used.push( column );
						}
					}
					vm.c_common.customized_columns_used = angular.copy( customized_columns_used );

					var regex_columns = /\(\(\s*(.*?)\s*\)\)/g;
					var result;
					while ( result = regex_columns.exec( $rootScope.html_to_text( vm.c_common.campaign.content_email_json + '' + vm.c_common.campaign.content_email_html + '' + vm.c_common.campaign.content_email_txt ) ) ) {
						var column = result[ 1 ];
						if ( column !== 'WEBCOPY_LINK' && column !== 'UNSUBSCRIBE_LINK' ) {
							if ( customized_columns_unknown.indexOf( column ) === - 1 ) {
								if ( customized_columns_used.indexOf( column ) === - 1 ) {
									customized_columns_unknown.push( column );
								}
							}
						}
					}
					vm.c_common.customized_columns_unknown = angular.copy( customized_columns_unknown );
				},

				check_campaign_data_input_value: function ( error ) {
					if ( error === '' ) {
						return $rootScope.translations.edit;
					}
					return $rootScope.translations.correct;
				},

				activate_link_tracking: function () {
					EmailContentService.activate_link_tracking();
					vm.c_common.campaign.link_tracking = '1';
					vm.c_common.save_campaign( true );
				},

				deactivate_link_tracking: function () {
					EmailContentService.deactivate_link_tracking();
					vm.c_common.campaign.link_tracking = '0';
					vm.c_common.save_campaign( true );
				},

				refresh_content_email: function () {
					var content = EmailContentService.refresh_content_email( vm.c_common.current_content_email_type, vm.c_common.campaign.content_email_json, vm.c_common.campaign.content_email_html, vm.c_common.campaign.content_email_txt );
					if ( content.content_email_changes ) {
						vm.c_common.content_email_changes = content.content_email_changes;
						vm.c_common.campaign.content_email_json = content.content_email_json;
						vm.c_common.campaign.content_email_html = content.content_email_html;
						vm.c_common.campaign.content_email_txt = content.content_email_txt;
						vm.c_common.content_email_types = content.content_email_types;
						vm.c_common.content_email_nb_links = content.content_email_nb_links;
						vm.c_common.content_email_unsubscribe_link = content.content_email_unsubscribe_link;
					}
				},


				send_test_check: function () {
					if ( vm.c_common.test_campaign_recipient === '' || ! VerificationService.email( vm.c_common.test_campaign_recipient ) ) {
						if ( vm.c_common.test_campaign_recipient === '' ) {
							vm.c_common.send_test_status.error_email = $rootScope.translations.the_test_recipient_is_required;
						}
						else if ( ! VerificationService.email( vm.c_common.test_campaign_recipient ) ) {
							vm.c_common.send_test_status.error_email = $rootScope.translations.a_valid_test_recipient_email_address_is_required;
						}
					}
					else {
						vm.c_common.send_test_status.error_email = '';
					}
					if ( vm.c_common.error.sender !== '' || vm.c_common.error.reply_to !== '' || vm.c_common.error.object !== '' || vm.c_common.error.content_email !== '' || vm.c_common.send_test_status.error_email !== '' ) {
						vm.c_common.send_test_status.campaign_ok = false;
					}
					else {
						vm.c_common.send_test_status.campaign_ok = true;
					}
					if ( ! vm.shared_campaign.checked_campaign_data.nb_credits_checked || vm.shared_campaign.checked_campaign_data.nb_credits_before < 1 ) {
						vm.c_common.send_test_status.credits_ok = false;
					}
					else {
						vm.c_common.send_test_status.credits_ok = true;
					}
					if ( vm.c_common.send_test_status.error_email !== '' || ! vm.c_common.send_test_status.campaign_ok || ! vm.c_common.send_test_status.credits_ok ) {
						return false;
					}
					return true;
				},

				send_test_status: {
					'error_email': '',
					'campaign_ok': false,
					'credits_ok': false
				},

				send_test_confirmation_validation: function () {
					if ( ! vm.c_common.save_campaign_success ) {
						$rootScope.display_error( $rootScope.translations.error_while_save_campaign );
						vm.c_common.display_send_test_confirmation = false;
					}
					else if ( ! $rootScope.settings.openssl_random_pseudo_bytes_extension_function_exists ) {
						$rootScope.display_error( $rootScope.translations.please_activate_the_extension_openssl_random_pseudo_bytes_on_your_web_server );
						vm.c_common.display_send_test_confirmation = false;
					}
					else if ( ! $rootScope.settings.gzdecode_gzencode_function_exists && display_confirmation === '' ) {
						$rootScope.display_error( $rootScope.translations.gzdecode_or_gzencode_php_function_not_found );
						vm.c_common.display_send_test_confirmation = false;
					}
					else if ( ! $rootScope.settings.base64_decode_base64_encode_function_exists && display_confirmation === '' ) {
						$rootScope.display_error( $rootScope.translations.base64_encode_or_base64_decode_php_function_not_found );
						vm.c_common.display_send_test_confirmation = false;
					}
					else if ( ! $rootScope.settings.json_encode_json_decode_function_exists && display_confirmation === '' ) {
						$rootScope.display_error( $rootScope.translations.json_encode_json_decode_php_function_not_found );
						vm.c_common.display_send_test_confirmation = false;
					}
					else if ( vm.c_common.send_test_check() ) {
						vm.c_common.display_send_test_confirmation = true;
					}
					else {
						vm.c_common.display_send_test_confirmation = false;
						if ( vm.c_common.send_test_status.error_email !== '' ) {
							$rootScope.display_error( vm.c_common.send_test_status.error_email );
						}
						else if ( vm.c_common.error.sender !== '' ) {
							$rootScope.display_error( vm.c_common.error.sender );
						}
						else if ( vm.c_common.error.reply_to !== '' ) {
							$rootScope.display_error( vm.c_common.error.reply_to );
						}
						else if ( vm.c_common.error.object !== '' ) {
							$rootScope.display_error( vm.c_common.error.object );
						}
						else if ( vm.c_common.error.content_email !== '' ) {
							$rootScope.display_error( vm.c_common.error.content_email );
						}
						else if ( ! vm.shared_campaign.checked_campaign_data.nb_credits_checked ) {
							if ( ! $rootScope.settings.is_authenticated ) {
								$rootScope.display_error( $rootScope.translations.you_must_login_or_create_a_jackmail_account );
								$rootScope.display_account_connection_popup( 'create' );
							}
							else {
								$rootScope.display_error( $rootScope.translations.error_while_checking_credits_available );
							}
						}
						else if ( vm.shared_campaign.checked_campaign_data.nb_credits_before < 1 ) {
							$rootScope.display_error( $rootScope.translations.you_don_t_have_enough_credits_available );
						}
						else {
							$rootScope.display_error( $rootScope.translations.one_or_more_campaign_fields_are_missing );
						}
					}
				},

				send_test: function () {
					var data_parameters = {
						'id': url_id,
						'test_recipient': vm.c_common.test_campaign_recipient
					};
					UrlService.post_data( url_send_test, data_parameters, function ( data ) {
						vm.c_common.check_campaign_data();
						if ( data.success ) {
							$rootScope.display_success( $rootScope.translations.the_test_email_has_been_sent );
							vm.shared_campaign.checked_campaign_data.nb_credits_before = ( vm.shared_campaign.checked_campaign_data.nb_credits_before - 1 ).toString();
							vm.shared_campaign.checked_campaign_data.nb_credits_after = ( vm.shared_campaign.checked_campaign_data.nb_credits_after - 1 ).toString();
						}
						else {
							var message = get_send_campaign_message( data.message );
							$rootScope.display_error( message );
						}
						vm.c_common.display_send_test_confirmation = false;
					}, function () {

					} );
				},

				cancel_test: function () {
					vm.c_common.display_send_test_confirmation = false;
				},

			};

			$interval( function () {
				if ( vm.c_common.current_step_name !== 'checklist' ) {
					check_save_campaign_needed();
				}
			}, 300000 );

			function check_save_campaign_needed() {
				var save = false;
				if ( ! $rootScope.show_help2 ) {
					if ( ! vm.c_common.sending_campaign ) {
						if ( $rootScope.nb_loading === 0 ) {
							vm.c_common.refresh_content_email();
							if ( url_id !== '0' || vm.c_common.campaign.name !== $rootScope.translations.campaign_with_no_name || vm.c_common.content_email_changes
								|| vm.c_common.campaign.sender_email !== $rootScope.settings.email || vm.c_common.campaign.sender_name !== ( $rootScope.settings.firstname + ' ' + $rootScope.settings.lastname )
								|| vm.c_common.campaign.reply_to_email !== '' || vm.c_common.campaign.reply_to_name !== ''
								|| vm.c_common.campaign.object !== ''
								|| vm.c_common.campaign.content_email_json !== '' || vm.c_common.campaign.content_email_html !== '' || vm.c_common.campaign.content_email_txt !== '' ) {
								if ( VerificationService.differents_arrays( vm.c_common.campaign, saved_campaign ) ) {
									vm.c_common.save_campaign( false );
									save = true;
								}
							}
							else if ( vm.campaign_type === 'scenario' && vm.c_common.campaign.id_lists !== saved_campaign.id_lists ) {
								vm.c_common.save_campaign( false );
								save = true;
							}
						}
					}
				}
				return save;
			}

			function change_url() {
				if ( vm.campaign_type === 'campaign' ) {
					var url = '/campaign/' + url_id + '/' + vm.c_common.current_step_name;
				}
				else {
					var url = '/scenario/' + vm.c_common.campaign.send_option + '/' + url_id + '/' + vm.c_common.current_step_name;
				}
				if ( $location.path() !== url ) {
					if ( $location.path().indexOf( '/0/' ) !== - 1 && url_id !== '0' ) {
						UrlService.change_url_parameters_without_reload_and_history( url );
					}
					else {
						UrlService.change_url_parameters_without_reload( url );
					}
				}
			}

			window.onpopstate = function () {
				$timeout( function () {
					var url = $location.path().split( '/' );
					var step_name = steps[ steps.indexOf( url[ url.length - 1 ] ) ];
					if ( step_name !== vm.c_common.current_step_name ) {
						vm.c_common.go_step( step_name );
					}
				} );
			};

			function get_send_campaign_message( message ) {
				var error = $rootScope.translations.an_error_occurred;
				if ( message === 'ERROR_READ_EMAIL_CONTENT_FILE' ) {
					error = $rootScope.translations.an_error_occurred_the_campaign_was_not_found;
				}
				else if ( message === 'FORBIDDEN' ) {
					error = $rootScope.translations.the_user_name_or_the_password_is_incorrect;
				}
				else if ( message === 'MESSAGE_CONTENT_IS_TOO_LARGE' ) {
					error = $rootScope.translations.message_content_is_too_large;
				}
				else if ( message === 'MISSING_PHP_OPENSSL' ) {
					error = $rootScope.translations.please_activate_the_extension_openssl_random_pseudo_bytes_on_your_web_server;
				}
				else if ( message === 'MISSING_GZDECODE_OR_GZENCODE' ) {
					error = $rootScope.translations.gzdecode_or_gzencode_php_function_not_found;
				}
				else if ( message === 'MISSING_BASE64_ENCODE_OR_BASE64_DECODE' ) {
					error = $rootScope.translations.base64_encode_or_base64_decode_php_function_not_found;
				}
				else if ( message === 'MISSING_JSON_ENCODE_OR_JSON_DECODE' ) {
					error = $rootScope.translations.json_encode_or_json_decode_php_function_not_found;
				}
				else if ( message === 'ERROR_WHILE_LOADING_IMAGES' ) {
					error = $rootScope.translations.error_while_loading_images;
				}
				else if ( message === 'ERROR_WHILE_LOADING_CONTENT' ) {
					error = $rootScope.translations.error_while_loading_content;
				}
				else if ( message === 'NOT_ENOUGH_CREDITS' ) {
					error = $rootScope.translations.not_enough_credits;
				}
				else if ( message === 'ERROR_WHILE_CHECKING_CREDITS' ) {
					error = $rootScope.translations.error_while_checking_credits_available;
				}
				else if ( message === 'NO_VALIDS_RECIPIENTS' ) {
					error = $rootScope.translations.no_valids_recipients;
				}
				else if ( message === 'NB_DISPLAYED_CONTACTS' ) {
					error = $rootScope.translations.data_displayed_are_not_up_to_date_click_ok_to_reload_it_before_you_send_the_campaign;
					if ( vm.c_common.campaign.send_option !== 'NOW' && ! vm.c_common.sending_campaign ) {
						error = $rootScope.translations.data_displayed_are_not_up_to_date_click_ok_to_reload_it_before_you_schedule_the_campaign;
					}
				}
				else {
					error = $rootScope.translations.an_error_occurred + ' (' + message + ')';
				}
				return error;
			}

		}

		if ( vm.page_type === 'campaign' && vm.campaign_type === 'campaign' ) {

			EmailContentService.init_emailbuilder();

			$rootScope.$watch( 'settings.is_authenticated', function ( new_value ) {
				if ( new_value === true ) {
					vm.c_common.check_campaign_data_ws();
				}
			} );

			vm.only_campaign = {

				save_choice_select_title: [
					$rootScope.translations.save_campaign,
					$rootScope.translations.save_as_template
				],

				import_selected_lists: function ( step_name ) {
					vm.show_import_lists = false;
					if ( url_id === '0' ) {
						if ( vm.c_common.nb_selected_lists > 0 ) {
							$rootScope.display_success( $rootScope.translations.import_contacts );
							UrlService.post_data( url_create, vm.c_common.campaign, function ( data ) {
								if ( data.success ) {
									vm.c_common.set_created_campaign_info( data );
									import_selected_lists();
								}
								else {
									$rootScope.display_error_writable();
								}
							}, function () {

							} );
						}
						else {
							vm.c_common.go_step( step_name );
						}
					}
					else {
						import_selected_lists();
					}

					function import_selected_lists() {
						var data_parameters = {
							'id_campaign': url_id,
							'id_lists': vm.c_common.campaign.id_lists
						};
						if ( vm.c_common.campaign.id_lists !== saved_campaign.id_lists ) {
							$rootScope.display_success( $rootScope.translations.import_contacts );
							UrlService.post_data( 'jackmail_set_campaign_lists', data_parameters, function ( data ) {
								vm.c_common.get_campaign_data( false );
								after_import();
							}, function () {

							} );
						}
						else {
							after_import();
						}
					}

					function after_import() {
						vm.common.get_list_data_reset();
						vm.c_common.go_step( step_name );
					}

				},

				hide_grid: function () {
					vm.common.show_grid = 0;
					vm.common_list_detail.show_list_contact_detail = false;
					vm.show_import_lists = false;
				},

				change_campaign_option: function ( option ) {
					if ( vm.c_common.campaign.send_option !== option ) {
						vm.c_common.campaign.send_option = option;
						vm.c_common.campaign.send_option_date_begin_gmt = '0000-00-00 00:00:00';
						vm.c_common.campaign.send_option_date_end_gmt = '0000-00-00 00:00:00';
						$rootScope.scroll_bottom();
					}
				},

				change_send_option_date: function ( date1, date2 ) {
					vm.c_common.campaign.send_option_date_begin_gmt = date1;
					if ( date2 ) {
						vm.c_common.campaign.send_option_date_end_gmt = date2;
					}
					else {
						vm.c_common.campaign.send_option_date_end_gmt = date1;
					}
				},

				send_campaign_confirmation_validation: function ( type_send ) {
					var display_confirmation = '';
					if ( ! vm.c_common.save_campaign_success ) {
						display_confirmation = $rootScope.translations.error_while_save_campaign;
					}
					else if ( ! $rootScope.settings.openssl_random_pseudo_bytes_extension_function_exists ) {
						display_confirmation = $rootScope.translations.please_activate_the_extension_openssl_random_pseudo_bytes_on_your_web_server;
					}
					else if ( ! $rootScope.settings.gzdecode_gzencode_function_exists ) {
						display_confirmation = $rootScope.translations.gzdecode_or_gzencode_php_function_not_found;
					}
					else if ( ! $rootScope.settings.base64_decode_base64_encode_function_exists ) {
						display_confirmation = $rootScope.translations.base64_encode_or_base64_decode_php_function_not_found;
					}
					else if ( ! $rootScope.settings.json_encode_json_decode_function_exists ) {
						display_confirmation = $rootScope.translations.json_encode_json_decode_php_function_not_found;
					}
					else if ( vm.c_common.error.sender !== '' ) {
						display_confirmation = vm.c_common.error.sender;
					}
					else if ( vm.c_common.error.reply_to !== '' ) {
						display_confirmation = vm.c_common.error.reply_to;
					}
					else if ( vm.c_common.error.recipients !== '' ) {
						display_confirmation = vm.c_common.error.recipients;
					}
					else if ( vm.c_common.error.object !== '' ) {
						display_confirmation = vm.c_common.error.object;
					}
					else if ( vm.c_common.error.content_email !== '' ) {
						display_confirmation = vm.c_common.error.content_email;
					}
					else if ( ( vm.c_common.campaign.send_option !== 'NOW' && ( vm.c_common.campaign.send_option_date_begin_gmt === '0000-00-00 00:00:00' || vm.c_common.campaign.send_option_date_end_gmt === '0000-00-00 00:00:00' ) ) ) {
						vm.c_common.error.send_option_date = $rootScope.translations.selected_date_is_not_valid;
						display_confirmation = vm.c_common.error.send_option_date;
					}
					if ( type_send === 'send' ) {
						if ( ! vm.shared_campaign.checked_campaign_data.nb_credits_checked && display_confirmation === '' ) {
							if ( ! $rootScope.settings.is_authenticated ) {
								display_confirmation = $rootScope.translations.you_must_login_or_create_a_jackmail_account;
								$rootScope.display_account_connection_popup( 'create' );
							}
							else {
								display_confirmation = $rootScope.translations.error_while_checking_credits_available;
							}
						}
						else if ( vm.shared_campaign.checked_campaign_data.nb_credits_after < 0 && display_confirmation === '' ) {
							display_confirmation = $rootScope.translations.you_don_t_have_enough_credits_available;
						}
					}
					if ( display_confirmation !== '' ) {
						$rootScope.display_error( display_confirmation );
					}
					else {
						if ( url_id !== '0' ) {
							var recipients = $filter( 'numberSeparator' )( vm.shared_campaign.checked_campaign_data.nb_contacts_valids ) + ' ';
							if ( vm.shared_campaign.checked_campaign_data.nb_contacts_valids <= 1 ) {
								recipients += $rootScope.translations.recipient;
							}
							else {
								recipients += $rootScope.translations.recipients;
							}
							var message_title = '';
							var message_confirmation = '';
							if ( type_send === 'program' ) {
								message_title = $rootScope.translations.scheduled_sending_confirmation;
								message_confirmation = $rootScope.translations.the_campaign_named + ' "' + vm.c_common.campaign.name + '" ' + $rootScope.translations.will_be_scheduled_to + ' ' + recipients + '. ' + $rootScope.translations.do_you_confirm_this_scheduling;
							}
							else if ( type_send === 'send' ) {
								message_title = $rootScope.translations.sending_confirmation;
								message_confirmation = $rootScope.translations.the_campaign_named + ' "' + vm.c_common.campaign.name + '" ' + $rootScope.translations.will_be_sent_to + ' ' + recipients + '. ' + $rootScope.translations.do_you_confirm_this_sending;
							}
							if ( message_title !== '' && message_confirmation !== '' ) {
								$rootScope.display_validation(
									'<span class="jackmail_title">' +
									'	' + message_title +
									'</span>' +
									'<br/><br/>' +
									'<span class="jackmail_campaign_confirmation_message jackmail_grey">' +
									'	' + message_confirmation +
									'</span>',
									function () {
										vm.c_common.sending_campaign = true;
										var data_parameters = {
											'id': vm.c_common.campaign.id,
											'send_option': vm.c_common.campaign.send_option,
											'send_option_date_begin_gmt': vm.c_common.campaign.send_option_date_begin_gmt,
											'send_option_date_end_gmt': vm.c_common.campaign.send_option_date_end_gmt,
											'nb_contacts_valids_displayed': vm.shared_campaign.checked_campaign_data.nb_contacts_valids
										};
										UrlService.post_data( 'jackmail_send_campaign', data_parameters, function ( data ) {
											if ( data.success ) {
												if ( $rootScope.settings.analytics ) {
													$window.ga( 'send', {
														'hitType': 'event',
														'appName': 'Jackmail',
														'appVersion': $rootScope.settings.version,
														'screenName': $location.url(),
														'eventCategory': 'Send campaign',
														'eventLabel': 'Send campaign',
														'eventAction': 'click',
														'eventValue': parseInt( vm.shared_campaign.checked_campaign_data.nb_contacts_valids )
													} );
												}
												$rootScope.go_page( 'campaigns' );
											}
											else {
												vm.c_common.sending_campaign = false;
												var error_message = get_send_campaign_message( data.message );
												if ( data.message === 'NB_DISPLAYED_CONTACTS' ) {
													$rootScope.display_validation(
														error_message,
														function () {
															vm.check_campaign_data();
															$rootScope.cancel_validation();
														}
													);
												}
												else {
													$rootScope.display_error( error_message );
												}
											}
										}, function () {
											vm.c_common.sending_campaign = false;
										} );
									}
								);
							}
						}
						else {
							$rootScope.display_error( $rootScope.translations.an_error_occurred );
						}
					}
				},

				save_campaign_or_create_template: function ( key ) {
					if ( key === 0 ) {
						vm.c_common.save_campaign( true );
					}
					else {
						save_as_template();
					}
				},

				reset_emailbuilder_content: function () {
					$rootScope.display_validation( $rootScope.translations.you_will_loose_your_emailbuilder_content, function () {
						EmailContentService.set_emailbuilder_json( '' );
						vm.c_common.refresh_content_email();
					} );
				}

			};

			vm.shared_campaign = {

				checked_campaign_data: {
					'nb_contacts_valids': '0',
					'nb_credits_before': '0',
					'nb_credits_after': '0',
					'nb_credits_checked': false,
					'subscription_type': '',
					'domain_is_valid': '',
					'analysis_checked': false,
					'analysis': []
				},

				type_choice_select_titles: [
					$rootScope.translations.emailbuilder,
					$rootScope.translations.html_code,
					$rootScope.translations.plain_text,
					$rootScope.translations.templates_gallery,
					$rootScope.translations.my_templates
				],

				display_current_content_email_type_or_template: function ( key ) {
					if ( ! $rootScope.settings.emailbuilder_installed && ( key === 0 || key === 3 || key === 4 ) ) {
						$rootScope.display_emailbuilder_popup();
					}
					else {
						vm.c_common.refresh_content_email();
						if ( key === 0 ) {
							if ( vm.c_common.campaign.content_email_html !== '' ) {
								$rootScope.display_validation( $rootScope.translations.you_will_loose_your_html_content, function () {
									vm.c_common.change_current_content_email_type( 'emailbuilder' );
								} );
							} else {
								vm.c_common.change_current_content_email_type( 'emailbuilder' );
							}
						}
						else if ( key === 1 ) {
							if ( vm.c_common.campaign.content_email_json !== '' ) {
								$rootScope.display_validation( $rootScope.translations.you_will_loose_your_emailbuilder_content, function () {
									vm.c_common.change_current_content_email_type( 'html' );
								} );
							} else {
								vm.c_common.change_current_content_email_type( 'html' );
							}
						}
						else if ( key === 2 ) {
							vm.c_common.change_current_content_email_type( 'txt' );
						}
						else if ( key === 3 || key === 4 ) {
							vm.c_common.show_templates = true;
							$rootScope.get_templates_from_campaign_page( key === 3 ? 'templates_gallery' : 'templates' );
							EmailContentService.hide_emailbuilder();
						}
					}
				}

			};

			vm.shared_campaign.type_choice_select_titles = [
				$rootScope.translations.emailbuilder,
				$rootScope.translations.html_code,
				$rootScope.translations.plain_text,
				$rootScope.translations.templates_gallery,
				$rootScope.translations.my_templates
			];

			function save_as_template() {
				if ( vm.c_common.current_content_email_type === 'emailbuilder' ) {
					vm.c_common.refresh_content_email();
					var data_parameters = {
						'name': $rootScope.translations.template_from + ' "' + vm.c_common.campaign.name + '"',
						'content_email_json': vm.c_common.campaign.content_email_json,
						'content_email_html': vm.c_common.campaign.content_email_html,
						'content_email_txt': ''
					};
					UrlService.post_data( 'jackmail_create_template', data_parameters, function ( data ) {
						$rootScope.display_success_error_writable( data.success, $rootScope.translations.the_template_was_saved );
					}, function () {

					} );
				}
			}

			vm.create_list = function () {
				if ( url_id === '0' ) {
					UrlService.post_data( url_create, vm.c_common.campaign, function ( data ) {
						if ( data.success ) {
							vm.c_common.set_created_campaign_info( data );
							vm.common.display_grid();
						}
						else {
							$rootScope.display_error_writable();
						}
					}, function () {

					} );
				}
				else {
					vm.common.display_grid();
				}
			};

			vm.display_import = function ( key ) {
				if ( key === 0 ) {
					vm.c_common.display_import_lists();
				}
				else {
					vm.common.display_copy_paste();
				}
			};

		}

		if ( vm.page_type === 'campaign' && vm.campaign_type === 'scenario' ) {

			EmailContentService.init_emailbuilder( true );

			vm.only_scenario = {

				scenario_choice: $routeParams.choice,

				post_categories_available: [],
				

				nb_selected_post_categories: - 1,

				selected_post_categories_title: '',

				activating_or_desactivating_scenario: false,

				post_categories_check_unckeck: function ( key, checked ) {
					var post_categories = [];
					var nb_selected_post_categories = 0;
					var i;
					var nb_post_categories_available = vm.only_scenario.post_categories_available.length;
					if ( key === 0 ) {
						vm.only_scenario.post_categories_available[ 0 ].checked = ! vm.only_scenario.post_categories_available[ 0 ].checked;
						var all_checked = false;
						if ( vm.only_scenario.post_categories_available[ 0 ].checked ) {
							all_checked = true;
						}
						for ( i = 0; i < nb_post_categories_available; i ++ ) {
							if ( i !== 0 ) {
								vm.only_scenario.post_categories_available[ i ].checked = true;
								if ( ! all_checked ) {
									post_categories.push( vm.only_scenario.post_categories_available[ i ].id );
									nb_selected_post_categories ++;
								}
							}
						}
					}
					else {
						vm.only_scenario.post_categories_available[ key ].checked = ! vm.only_scenario.post_categories_available[ key ].checked;
						if ( ! vm.only_scenario.post_categories_available[ key ].checked ) {
							vm.only_scenario.post_categories_available[ 0 ].checked = false;
						}
						for ( i = 0; i < nb_post_categories_available; i ++ ) {
							if ( i !== 0 ) {
								if ( vm.only_scenario.post_categories_available[ i ].checked ) {
									post_categories.push( vm.only_scenario.post_categories_available[ i ].id );
									nb_selected_post_categories ++;
								}
							}
						}
						if ( nb_selected_post_categories === 0 ) {
							vm.only_scenario.post_categories_available[ 0 ].checked = true;
							for ( i = 0; i < nb_post_categories_available; i ++ ) {
								if ( i !== 0 ) {
									vm.only_scenario.post_categories_available[ i ].checked = true;
								}
							}
						}
					}
					vm.c_common.campaign.post_categories = $rootScope.join( post_categories );
					vm.only_scenario.nb_selected_post_categories = nb_selected_post_categories;
					vm.only_scenario.selected_post_categories_title = vm.only_scenario.get_selected_post_categories_title();
				},

				get_post_categories_select: function ( data ) {
					var nb_selected_post_categories = 0;
					var post_categories = $rootScope.split( vm.c_common.campaign.post_categories );
					var nb_post_categories = post_categories.length;
					var nb_post_categories_available = data.length;
					var all_checked = false;
					if ( nb_post_categories === 0 ) {
						all_checked = true;
						for ( i = 0; i < nb_post_categories_available; i ++ ) {
							data[ i ].checked = true;
							data[ i ].name = data[ i ].label;
						}
					}
					else {
						var i;
						for ( i = 0; i < nb_post_categories_available; i ++ ) {
							data[ i ].name = data[ i ].label;
							if ( post_categories.indexOf( data[ i ].id ) !== - 1 ) {
								data[ i ].checked = true;
								nb_selected_post_categories ++;
							}
							else {
								data[ i ].checked = false;
							}
						}
					}
					data.unshift( { 'id': 0, 'name': $rootScope.translations.all_categories, 'checked': all_checked } );
					vm.only_scenario.nb_selected_post_categories = nb_selected_post_categories;
					vm.only_scenario.selected_post_categories_title = vm.only_scenario.get_selected_post_categories_title();
					return data;
				},

				get_selected_post_categories_title: function () {
					if ( vm.c_common.campaign.post_categories === '[]' ) {
						return $rootScope.translations.all_categories;
					}
					else if ( vm.only_scenario.nb_selected_post_categories === 1 ) {
						return vm.only_scenario.nb_selected_post_categories + ' ' + $rootScope.translations.selected_category;
					}
					else if ( vm.only_scenario.nb_selected_post_categories > 1 ) {
						return vm.only_scenario.nb_selected_post_categories + ' ' + $rootScope.translations.selected_categories;
					}
					return '';
				},

				activate_scenario_confirmation_validation: function () {
					var display_confirmation = '';
					if ( ! vm.c_common.save_campaign_success ) {
						display_confirmation = $rootScope.translations.error_while_save_campaign;
					}
					else if ( ! $rootScope.settings.openssl_random_pseudo_bytes_extension_function_exists ) {
						display_confirmation = $rootScope.translations.please_activate_the_extension_openssl_random_pseudo_bytes_on_your_web_server;
					}
					else if ( ! $rootScope.settings.gzdecode_gzencode_function_exists ) {
						display_confirmation = $rootScope.translations.gzdecode_or_gzencode_php_function_not_found;
					}
					else if ( ! $rootScope.settings.base64_decode_base64_encode_function_exists ) {
						display_confirmation = $rootScope.translations.base64_encode_or_base64_decode_php_function_not_found;
					}
					else if ( ! $rootScope.settings.json_encode_json_decode_function_exists ) {
						display_confirmation = $rootScope.translations.json_encode_json_decode_php_function_not_found;
					}
					else if ( vm.c_common.error.sender !== '' ) {
						display_confirmation = vm.c_common.error.sender;
					}
					else if ( vm.c_common.error.reply_to !== '' ) {
						display_confirmation = vm.c_common.error.reply_to;
					}
					else if ( vm.c_common.error.recipients !== '' ) {
						display_confirmation = vm.c_common.error.recipients;
					}
					else if ( vm.c_common.error.object !== '' ) {
						display_confirmation = vm.c_common.error.object;
					}
					else if ( vm.c_common.error.content_email !== '' ) {
						display_confirmation = vm.c_common.error.content_email;
					}
					else if ( ! $rootScope.settings.is_authenticated ) {
						display_confirmation = $rootScope.translations.you_must_login_or_create_a_jackmail_account;
						$rootScope.display_account_connection_popup( 'create' );
					}
					if ( display_confirmation !== '' ) {
						$rootScope.display_error( display_confirmation );
					}
					else {
						if ( url_id !== '0' ) {
							$rootScope.display_validation(
								$rootScope.translations.activate_scenario,
								function () {
									vm.only_scenario.activating_or_desactivating_scenario = true;
									var data_parameters = {
										'id': vm.c_common.campaign.id
									};
									UrlService.post_data( 'jackmail_activate_scenario', data_parameters, function ( data ) {
										scenario_after_activate_or_deactivate( data, $rootScope.translations.the_scenario_was_activated, data.message );
									}, function () {

									} );
								}
							);
						}
						else {
							$rootScope.display_error( $rootScope.translations.an_error_occurred );
						}
					}
				},

				deactivate_scenario_confirmation_validation: function () {
					if ( url_id !== '0' ) {
						$rootScope.display_validation(
							$rootScope.translations.deactivate_scenario,
							function () {
								vm.only_scenario.activating_or_desactivating_scenario = true;
								var data_parameters = {
									'id': vm.c_common.campaign.id
								};
								UrlService.post_data( 'jackmail_deactivate_scenario', data_parameters, function ( data ) {
									scenario_after_activate_or_deactivate( data, $rootScope.translations.the_scenario_was_disabled, '' );
								}, function () {

								} );
							}
						);
					}
					else {
						$rootScope.display_error( $rootScope.translations.an_error_occurred );
					}
				}

			};

			vm.shared_campaign = {

				checked_campaign_data: {
					'nb_contacts_valids': '0',
					'nb_credits_checked': false
				},

				type_choice_select_titles: [
					$rootScope.translations.emailbuilder,
					$rootScope.translations.plain_text,
					$rootScope.translations.templates_gallery,
					$rootScope.translations.my_templates
				],

				display_current_content_email_type_or_template: function ( key ) {
					if ( key === 0 ) {
						vm.c_common.change_current_content_email_type( 'emailbuilder' );
					}
					else if ( key === 1 ) {
						vm.c_common.change_current_content_email_type( 'txt' );
					}
					else if ( key === 2 || key === 3 ) {
						vm.c_common.show_templates = true;
						$rootScope.get_templates_from_campaign_page( key === 2 ? 'templates_gallery' : 'templates' );
						EmailContentService.hide_emailbuilder();
					}
				}

			};

			if ( vm.only_scenario.scenario_choice === 'publish_a_post' ) {

				vm.shared_scenario = {

					settings_hours_choice: [ '1', '2', '3', '4', '5', '6', '7', '8', '9', '10', '11', '12', '13', '14', '15', '16', '17', '18', '19', '20', '21', '22', '23' ],

					settings_days_choice: [ '1', '2', '3', '4', '5' ],

					select_periodicity_value: function ( key ) {
						if ( vm.c_common.campaign.periodicity_type === 'HOURS' ) {
							vm.c_common.campaign.periodicity_value = vm.shared_scenario.settings_hours_choice[ key ];
						}
						else if ( vm.c_common.campaign.periodicity_type === 'DAYS' ) {
							vm.c_common.campaign.periodicity_value = vm.shared_scenario.settings_days_choice[ key ];
						}
					},

					select_periodicity_type: function ( key ) {
						if ( key === 0 ) {
							vm.c_common.campaign.periodicity_type = 'HOURS';
						}
						else {
							if ( vm.c_common.campaign.periodicity_value > 5 ) {
								vm.c_common.campaign.periodicity_value = '5';
							}
							vm.c_common.campaign.periodicity_type = 'DAYS';
						}
					},

					change_periodicity_type: function ( choice ) {
						vm.c_common.campaign.periodicity_value = '1';
						vm.c_common.campaign.periodicity_type = choice;
					}

				};

			}

			else if ( vm.only_scenario.scenario_choice === 'automated_newsletter' || vm.only_scenario.scenario_choice === 'woocommerce_automated_newsletter' ) {

				vm.shared_scenario = {

					settings_days_choice: [ '1', '2', '3', '4', '5', '6', '7', '8', '9', '10', '11', '12', '13', '14', '15', '16', '17', '18', '19', '20', '21', '22', '23', '24', '25', '26', '27', '28', '29', '30', '31' ],

					settings_months_choice: [ '1', '2', '3', '4', '5', '6', '7', '8', '9', '10', '11', '12' ],

					settings_nb_posts_periodicity_value_choice: [ '1', '2', '3', '4', '5', '6', '7', '8', '9', '10' ],

					settings_nb_posts_content_choice: [ '1', '2', '3', '4', '5', '6', '7', '8', '9', '10' ],

					select_periodicity_value: function ( key ) {
						if ( vm.c_common.campaign.periodicity_type === 'DAYS' ) {
							vm.c_common.campaign.periodicity_value = vm.shared_scenario.settings_days_choice[ key ];
						}
						else if ( vm.c_common.campaign.periodicity_type === 'MONTHS' ) {
							vm.c_common.campaign.periodicity_value = vm.shared_scenario.settings_months_choice[ key ];
						}
					},

					select_periodicity_type: function ( key ) {
						if ( key === 0 ) {
							vm.c_common.campaign.periodicity_type = 'DAYS';
						}
						else {
							if ( vm.c_common.campaign.periodicity_value > 12 ) {
								vm.c_common.campaign.periodicity_value = '12';
							}
							vm.c_common.campaign.periodicity_type = 'MONTHS';
						}
					},

					change_periodicity_type: function ( choice ) {
						if ( choice === 'POSTS' ) {
							vm.c_common.campaign.periodicity_value = '5';
							vm.c_common.campaign.event_date_gmt = $rootScope.settings.current_time;
						}
						else {
							vm.c_common.campaign.periodicity_value = '1';
						}
						vm.c_common.campaign.periodicity_type = choice;
					},

					change_nb_posts_periodicity_value: function ( key ) {
						vm.c_common.campaign.periodicity_value = vm.shared_scenario.settings_nb_posts_periodicity_value_choice[ key ];
					},

					change_nb_posts_content: function ( key ) {
						vm.c_common.campaign.nb_posts_content = vm.shared_scenario.settings_nb_posts_content_choice[ key ];
						EmailContentService.set_nb_posts_content( vm.c_common.campaign.nb_posts_content );
					},

					change_event_date: function ( date ) {
						vm.c_common.campaign.event_date_gmt = date;
					}

				};

			}

			function scenario_after_activate_or_deactivate( data, message_ok, message_error ) {
				vm.only_scenario.activating_or_desactivating_scenario = false;
				$rootScope.display_success_error( data.success, message_ok, message_error );
				if ( data.success ) {
					if ( vm.c_common.campaign.status === 'ACTIVED' ) {
						vm.c_common.campaign.status = 'DRAFT';
					}
					else {
						vm.c_common.campaign.status = 'ACTIVED';
					}
				}
				else {
					vm.c_common.get_campaign_data( false );
				}
			}

		}

	} ] );

angular.module( 'jackmail.controllers' ).controller( 'ListsController', [
	'$rootScope', 'UrlService', '$timeout', 'ExportService', 'GridService', 'PluginService',
	function ( $rootScope, UrlService, $timeout, ExportService, GridService, PluginService ) {

		var vm = this;

		$rootScope.grid_service = new GridService();

		vm.columns = [
			{ 'name': $rootScope.translations.list_name, 'field': 'name' },
			{ 'name': $rootScope.translations.Recipients, 'field': 'nb_contacts' },
			{ 'name': $rootScope.translations.statistics, 'field': '' },
			{ 'name': $rootScope.translations.actions, 'field': '' }
		];

		vm.lists = [];
		vm.nb_lists = - 1;
		vm.nb_deletable_lists = - 1;

		vm.show_new_plugins_confirmation = false;
		var nb_new_plugins = 0;
		vm.new_plugins = PluginService.get_compatible_plugins();

		UrlService.post_data( 'jackmail_get_new_plugins', {}, function ( data ) {
			vm.new_plugins = PluginService.get_plugins_to_display( vm.new_plugins, data );
			nb_new_plugins = data.length;
			if ( nb_new_plugins > 0 ) {
				vm.show_new_plugins_confirmation = true;
			}

		}, function () {

		} );

		function get_lists( search ) {
			var data_parameters = {
				'search': search
			};
			UrlService.post_data( 'jackmail_get_lists', data_parameters, function ( data ) {
				vm.lists = data;
				vm.nb_lists = data.length;
				var i;
				var nb_lists = data.length;
				var nb_deletable_lists = 0;
				for ( i = 0; i < nb_lists; i ++ ) {
					if ( data[ i ].type === '' ) {
						nb_deletable_lists ++;
					}
				}
				vm.nb_deletable_lists = nb_deletable_lists;
				$timeout( function () {
					$rootScope.grid_service.refresh_grid_max_height( 'refresh' );
				} );
			}, function () {

			} );
		}

		get_lists( '' );

		vm.get_lists_search = function ( search ) {
			get_lists( search );
		};

		vm.delete_list_confirmation_validation = function ( key, id, in_widget, in_scenario ) {
			var message = $rootScope.translations.delete_list;
			if ( in_widget ) {
				message += '<br/>' + $rootScope.translations.widget_which_use_this_list_will_be_deactived;
			}
			if ( in_scenario ) {
				message += '<br/>' + $rootScope.translations.workflows_which_use_only_this_list_will_be_deactived;
			}
			var data_parameters = {
				'id_lists': $rootScope.join( [ id ] )
			};
			UrlService.post_data( 'jackmail_delete_lists', data_parameters, function ( data ) {
				if ( data.success ) {
					if ( vm.lists[ key ].selected ) {
						$rootScope.grid_service.set_nb_selected( $rootScope.grid_service.nb_selected - 1 );
					}
					vm.lists.splice( key, 1 );
					vm.nb_lists = vm.lists.length;
					vm.nb_deletable_lists --;
				}
				$rootScope.display_success_error( data.success, $rootScope.translations.the_list_was_deleted, '' );
			}, function () {

			} );
		};

		vm.delete_selection_confirmation = function () {
			var i;
			var nb_lists = vm.lists.length;
			var delete_lists = [];
			for ( i = 0; i < nb_lists; i ++ ) {
				if ( vm.lists[ i ].selected ) {
					delete_lists.push( vm.lists[ i ].id );
				}
			}
			if ( delete_lists.length > 0 ) {
				var data_parameters = {
					'id_lists': $rootScope.join( delete_lists )
				};
				UrlService.post_data( 'jackmail_delete_lists', data_parameters, function ( data ) {
					if ( data.success ) {
						var nb_deletable_lists = vm.nb_deletable_lists;
						for ( i = nb_lists - 1; i >= 0; i -- ) {
							if ( vm.lists[ i ].selected ) {
								vm.lists.splice( i, 1 );
								nb_deletable_lists --;
							}
						}
						vm.nb_lists = vm.lists.length;
						vm.nb_deletable_lists = nb_deletable_lists;
						$rootScope.grid_service.set_nb_selected( 0 );
						$rootScope.display_success_error( data.success, $rootScope.translations.selected_lists_have_been_deleted, '' );
					}
				}, function () {

				} );
			}
		};

		vm.export_all = function ( id, nb_contacts ) {
			if ( nb_contacts > 0 ) {
				var part = 3000;
				var nb_parts = Math.ceil( nb_contacts / part );
				var begin = 0;
				var i;
				var data_parameters = [];
				for ( i = 0; i < nb_parts; i ++ ) {
					data_parameters.push( {
						'id': id,
						'begin': begin.toString(),
						'sort_by': '',
						'sort_order': '',
						'search': '',
						'targeting_rules': ''
					} );
					begin = begin + part;
				}
				UrlService.post_multiple_data( 'jackmail_export_list', data_parameters, $rootScope.translations.downloading, function ( data ) {
					ExportService.export_contact_file_multiple_data( data );
				}, function () {

				} );
			} else {
				$rootScope.display_error( $rootScope.translations.the_list_is_empty );
			}
		};

		vm.grid_select_or_unselect_row = function ( key ) {
			vm.lists = $rootScope.grid_service.grid_select_or_unselect_row( vm.lists, key );
		};

		vm.grid_select_or_unselect_all = function () {
			vm.lists = $rootScope.grid_service.grid_select_or_unselect_all_with_field_restriction( vm.lists, 'type' );
		};

		vm.range_by = function ( i ) {
			vm.lists = $rootScope.grid_service.grid_range( vm.lists, i );
		};

		vm.create_list = function () {
			$rootScope.change_page_with_parameters( 'list', '0' );
		};

		vm.edit_list_name = function ( name, id ) {
			var data_parameters = {
				'name': name,
				'id': id
			};
			UrlService.post_data( 'jackmail_save_name', data_parameters, function ( data ) {
				if ( ! data.success ) {
					get_lists( '' );
				}
				$rootScope.display_success_error( data.success, $rootScope.translations.the_list_was_saved, $rootScope.translations.list_name_must_be_unique );
			}, function () {

			} );
		};

		vm.select_or_unselect_new_plugin = function ( plugin ) {
			vm.new_plugins[ plugin ].selected = ! vm.new_plugins[ plugin ].selected;
		};

		vm.cancel_new_plugins_confirmation = function () {
			vm.show_new_plugins_confirmation = false;
		};

		vm.select_new_plugin = function ( plugin ) {
			vm.new_plugins[ plugin ].selected = true;
			vm.new_plugins[ plugin ].hide = true;
			vm.save_new_plugin_confirmation();
		};

		vm.unselect_new_plugin = function ( plugin ) {
			vm.new_plugins[ plugin ].selected = false;
			vm.new_plugins[ plugin ].hide = true;
			vm.save_new_plugin_confirmation();
		};

		vm.save_new_plugin_confirmation = function () {
			nb_new_plugins --;
			if ( nb_new_plugins === 0 ) {
				angular.element( '.jackmail_notice.jackmail_notice_plugins' ).parent().parent().hide();
				vm.show_new_plugins_confirmation = false;
				var plugins = PluginService.get_active_plugins( vm.new_plugins );
				var data_parameters = {
					'plugins': $rootScope.join( plugins )
				};
				UrlService.post_data( 'jackmail_save_new_plugins', data_parameters, function ( data ) {
					if ( plugins.length > 0 ) {
						get_lists( '' );
					}
				}, function () {

				} );
			}
		};

	} ] );

angular.module( 'jackmail.controllers' ).controller( 'MainController', [
	'$rootScope', '$location', '$timeout', '$sce', '$window', '$document', '$interval', 'EmailContentService', 'UrlService',
	function ( $rootScope, $location, $timeout, $sce, $window, $document, $interval, EmailContentService, UrlService ) {

		$rootScope.translations = jackmail_translations_object;

		jackmail_ajax_object.jackmail_file_path_name = str( jackmail_ajax_object.jackmail_file_path_name );
		jackmail_ajax_object.key = str( jackmail_ajax_object.key );
		jackmail_ajax_object.email = str( jackmail_ajax_object.email );
		jackmail_ajax_object.lastname = str( jackmail_ajax_object.lastname );
		jackmail_ajax_object.firstname = str( jackmail_ajax_object.firstname );
		$rootScope.settings = jackmail_ajax_object;

		var url = $location.absUrl();

		if ( $rootScope.settings.analytics ) {
			$window.ga( 'create', $rootScope.settings.analytics_account, 'auto' );
		}

		$rootScope.jackmail_success = false;
		$rootScope.jackmail_error = false;
		$rootScope.jackmail_message = '';
		$rootScope.loading = false;
		$rootScope.nb_loading = 0;
		

		$rootScope.validation_popup = false;
		$rootScope.validation_popup_message = '';
		$rootScope.validation_popup_callback = function () {
		};

		$rootScope.show_help1 = false;
		$rootScope.show_help2 = false;

		$rootScope.show_emailbuilder = false;

		$rootScope.show_html_preview_popup = false;
		$rootScope.html_preview_content_popup = '';

		$rootScope.show_account_connection_popup = false;

		if ( url.indexOf( 'jackmail_installation' ) !== - 1 ) {
			$rootScope.show_account_connection_popup_form = 'create';
		} else {
			$rootScope.show_account_connection_popup_form = 'connection';
		}

		$rootScope.show_emailbuilder_popup = false;
		$rootScope.emailbuilder_popup_licence = '';

		$timeout( function () {
			$rootScope.content_loaded = true;
		} );

		var begin = url.indexOf( 'page=jackmail_' ) + 14;
		if ( url.indexOf( '#/' ) === - 1 ) {
			$location.path( url.substr( begin, url.length - begin ) );
		}

		$rootScope.$on( '$routeChangeStart', function () {
			if ( $rootScope.settings.analytics ) {
				$window.ga( 'send', { 'hitType': 'pageview', 'appName': 'Jackmail', 'appVersion': $rootScope.settings.version, 'screenName': $location.url() } );
			}
			
		} );
		
		$rootScope.split = function ( data ) {
			if ( data !== undefined && data !== '' ) {
				return JSON.parse( data );
			}
			else {
				return [];
			}
		};

		$rootScope.join = function ( data ) {
			return JSON.stringify( data );
		};

		$rootScope.split_fields = function ( data ) {
			if ( data !== undefined && data !== '' ) {
				data = data.toUpperCase();
				return JSON.parse( data );
			}
			else {
				return [];
			}
		};

		$rootScope.join_fields = function ( data ) {
			return JSON.stringify( data );
		};

		$rootScope.refresh_header_footer_position = function () {
			var document_width = parseInt( angular.element( $document ).width() );
			var window_width = parseInt( angular.element( $window ).width() );
			var scroll_left = parseInt( angular.element( $window ).scrollLeft() );
			
			var menu_width = 0;
			if ( angular.element( '#adminmenuback' ).css( 'display' ) === 'block' ) {
				menu_width = parseInt( angular.element( '#adminmenuback' ).width() );
			}
			var width = 'auto';
			var width_notices = 'auto';
			if ( window_width < document_width ) {
				width = ( document_width - menu_width ) + 'px';
				width_notices = ( document_width - menu_width - 80 ) + 'px';
			}
			var left = menu_width - scroll_left;
			angular.element(
				'.jackmail .jackmail_campaigns_action_container > div,' +
				'.jackmail .jackmail_message.jackmail_success > div,' +
				'.jackmail .jackmail_message.jackmail_error > div,' +
				'.jackmail .jackmail_header > div,' +
				'.jackmail .jackmail_footer > div' ).css( {
				'margin-left': left + 'px',
				'width': width
			} );
			angular.element( '.jackmail_notice_subcontainer > div' ).css( {
				'margin-left': ( left + 15 ) + 'px',
				'width': width_notices
			} );
			angular.element( '#adminmenuback' ).css( { 'left': ( - scroll_left ) + 'px' } );
		};

		function refresh_grid_max_height( type ) {
			if ( $rootScope.grid_service ) {
				if ( $rootScope.grid_service.length ) {
					var i;
					var nb_grids = $rootScope.grid_service.length;
					for ( i = 0; i < nb_grids; i ++ ) {
						$rootScope.grid_service[ i ].refresh_grid_max_height( type );
					}
				}
				else {
					$rootScope.grid_service.refresh_grid_max_height( type );
				}
			}
		}

		$window.onbeforeunload = function ( event ) {
			$timeout( function () {
				$rootScope.loading = true;
			} );
		};

		angular.element( $window ).on( 'scroll', function () {
			$rootScope.refresh_header_footer_position();
			
			refresh_grid_max_height( 'scroll' );
			EmailContentService.refresh_email_content_height();
		} );

		angular.element( $window ).on( 'resize', function () {
			$timeout( function () {
				
				$rootScope.refresh_header_footer_position();
			} );
			refresh_grid_max_height( 'resize' );
			EmailContentService.refresh_email_content_height();
		} );

		angular.element( '#collapse-menu' ).click( function () {
			$timeout( function () {
				$rootScope.refresh_header_footer_position();
				refresh_grid_max_height( 'resize' );
			} );
		} );

		angular.element( '.notice' ).click( function () {
			$timeout( function () {
				$rootScope.refresh_header_footer_position();
			}, 200 );
		} );

		$rootScope.$on( '$viewContentLoaded', function () {
			$timeout( function () {
				$rootScope.refresh_header_footer_position();
			} );
		} );

		$rootScope.go_page = function ( page ) {
			$timeout( function () {
				$rootScope.loading = true;
			} );
			window.location.href = 'admin.php?page=jackmail_' + page;
		};

		$rootScope.change_page = function ( page ) {
			$timeout( function () {
				$rootScope.loading = true;
			} );
			if ( $location.path() != '/' + page ) {
				window.location.href = 'admin.php?page=jackmail_' + page + '#/' + page;
			}
			else {
				window.location.reload();
			}
		};

		$rootScope.change_page_with_parameters = function ( page, parameters ) {
			$timeout( function () {
				$rootScope.loading = true;
			} );
			var page_parameters = '/' + page + '/' + parameters;
			if ( $location.path() != page_parameters ) {
				window.location.href = 'admin.php?page=jackmail_' + page + '#' + page_parameters;
			}
			else {
				window.location.reload();
			}
		};

		var progress_success_error;
		
		$rootScope.display_success_error = function ( status, message_ok, messsage_error ) {
			$timeout.cancel( progress_success_error );
			if ( status ) {
				$rootScope.jackmail_success = true;
				$rootScope.jackmail_error = false;
				$rootScope.jackmail_message = message_ok;
				progress_success_error = $timeout( function () {
					$rootScope.jackmail_success = false;
					$rootScope.jackmail_message = '';
				}, 2500 );
			}
			else {
				$rootScope.jackmail_error = true;
				$rootScope.jackmail_success = false;
				if ( messsage_error ) {
					$rootScope.jackmail_message = messsage_error;
				}
				else {
					$rootScope.jackmail_message = $rootScope.translations.an_error_occurred;
				}
				progress_success_error = $timeout( function () {
					$rootScope.jackmail_error = false;
					$rootScope.jackmail_message = '';
				}, 5000 );
			}
		};

		$rootScope.display_success = function ( message_ok ) {
			$rootScope.display_success_error( true, message_ok, '' );
		};

		$rootScope.display_error = function ( message_error ) {
			$rootScope.display_success_error( false, '', message_error );
		};

		$rootScope.display_success_error_writable = function ( status, message_ok ) {
			var message_error = $rootScope.translations.an_error_occurred;
			if ( ! $rootScope.settings.jackmail_file_path_writable ) {
				message_error += ' - ' + $rootScope.translations.the_file + ' "' + $rootScope.settings.jackmail_file_path_name + '" ' + $rootScope.translations.should_be_editable;
			}
			$rootScope.display_success_error( status, message_ok, message_error );
		};

		$rootScope.display_error_writable = function () {
			$rootScope.display_success_error_writable( false, '' );
		};

		$rootScope.hide_success_error = function () {
			$rootScope.jackmail_success = false;
			$rootScope.jackmail_error = false;
		};

		$rootScope.display_validation = function ( message, callback ) {
			$rootScope.validation_popup = true;
			$rootScope.validation_popup_message = $sce.trustAsHtml( message );
			$rootScope.validation_popup_callback = callback;
		};

		$rootScope.hide_validation = function () {
			$rootScope.validation_popup = false;
		};

		$rootScope.cancel_validation = function () {
			$rootScope.validation_popup = false;
		};

		$rootScope.ok_validation = function () {
			$rootScope.validation_popup_callback();
			$rootScope.validation_popup = false;
		};

		$rootScope.display_emailbuilder_popup = function () {
			UrlService.post_data( 'jackmail_get_emailbuilder_licence', {}, function ( data ) {
				if ( data.licence ) {
					if ( data.licence.length > 2000 && data.licence.indexOf( 'EmailBuilder' ) !== - 1 ) {
						$rootScope.emailbuilder_popup_licence = $sce.trustAsHtml( data.licence );
						$rootScope.show_emailbuilder_popup = true;
					} else {
						$rootScope.display_error( $rootScope.translations.can_t_create_new_template_error_while_loading_emailbuilder_licence );
					}
				} else {
					$rootScope.display_error( $rootScope.translations.can_t_create_new_template_error_while_loading_emailbuilder_licence );
				}
			}, function () {

			} );
		};

		$rootScope.cancel_emailbuilder_popup = function () {
			$rootScope.show_emailbuilder_popup = false;
		};

		$rootScope.ok_emailbuilder_popup = function () {
			UrlService.post_data( 'jackmail_install_emailbuilder', {}, function ( data ) {
				if ( data.success ) {
					
					$rootScope.display_success( $rootScope.translations.emailbuilder_is_now_installed );
					window.location.reload();
				}
			}, function () {

			} );
			$rootScope.show_emailbuilder_popup = false;
		};

		$rootScope.hide_update_available_popup = function () {
			$rootScope.settings.update_available = false;
			UrlService.post_data( 'jackmail_hide_update_popup', {}, function ( data ) {

			}, function () {

			} );
		};

		$rootScope.scroll_top = function () {
			angular.element( 'html' ).scrollTop( 0 );
		};

		$rootScope.scroll_bottom = function () {
			$timeout( function () {
				angular.element( 'html' ).scrollTop( angular.element( 'html' ).height() );
			} );
		};

		$rootScope.active_item_menu = function ( i ) {
			angular.element( '#toplevel_page_jackmail_campaigns > ul > li:nth-child( ' + i + ' ), #toplevel_page_jackmail_campaigns > ul > li:nth-child( ' + i + ' ) > a' ).addClass( 'current' );
		};

		$rootScope.months = [
			$rootScope.translations.jan,
			$rootScope.translations.feb,
			$rootScope.translations.mar,
			$rootScope.translations.apr,
			$rootScope.translations.may,
			$rootScope.translations.june,
			$rootScope.translations.july,
			$rootScope.translations.aug,
			$rootScope.translations.sept,
			$rootScope.translations.oct,
			$rootScope.translations.nov,
			$rootScope.translations.dec
		];

		$rootScope.date_from_str = function ( date ) {
			var a = date.split( ' ' );
			var d = a [ 0 ].split( '-' );
			var t = a [ 1 ].split( ':' );
			return new Date( d [ 0 ], ( d [ 1 ] - 1 ), d [ 2 ], t [ 0 ], t [ 1 ], t [ 2 ] );
		};

		$rootScope.add_date_interval = function ( date, interval ) {
			var date = $rootScope.date_from_str( date );
			date.setSeconds( date.getSeconds() + interval );
			var selected_day = date.getDate();
			if ( selected_day < 10 ) {
				selected_day = '0' + selected_day;
			}
			var selected_month = date.getMonth() + 1;
			if ( selected_month < 10 ) {
				selected_month = '0' + selected_month;
			}
			var selected_year = date.getFullYear();
			var selected_hour = date.getHours();
			if ( selected_hour < 10 ) {
				selected_hour = '0' + selected_hour;
			}
			var selected_minute = date.getMinutes();
			if ( selected_minute < 10 ) {
				selected_minute = '0' + selected_minute;
			}
			var selected_second = date.getSeconds();
			if ( selected_second < 10 ) {
				selected_second = '0' + selected_second;
			}
			return selected_year + '-' + selected_month + '-' + selected_day + ' ' + selected_hour + ':' + selected_minute + ':' + selected_second;
		};

		$rootScope.generate_grid = function ( datas ) {
			var container_width = angular.element( '.jackmail' ).width();
			var column_width = 240;
			var column_width_padding = column_width + 20;
			var nb_campaigns = datas.length;
			var nb_columns = parseInt( container_width / column_width_padding );
			var nb_datas_columns = ( nb_campaigns / nb_columns );
			if ( nb_columns > nb_campaigns ) {
				nb_columns = nb_campaigns;
			}
			var i;
			var j;
			var campaigns_grid = [];
			for ( i = 0; i < nb_columns; i ++ ) {
				var column_datas = [];
				for ( j = 0; j < nb_datas_columns; j ++ ) {
					if ( datas[ i + j * nb_columns ] ) {
						column_datas.push( datas[ i + j * nb_columns ] );
					}
				}
				campaigns_grid.push( column_datas );
			}
			var grid_width = column_width_padding * campaigns_grid.length;
			if ( grid_width !== 0 ) {
				angular.element( '.jackmail_previews_grid_container > div' ).css( { 'width': grid_width + 'px' } );
			}
			return campaigns_grid;
		};

		$rootScope.display_html_preview_popup = function () {
			var html_preview = EmailContentService.get_email_content_editor( 'html' );
			html_preview = html_preview.replace( /..\/\?jackmail_image\=/g, $rootScope.settings.website_url + '?jackmail_image=' );
			
			$rootScope.html_preview_content_popup = $sce.trustAsResourceUrl( 'data:text/html;charset=utf-8,' + encodeURIComponent( html_preview ) );
			$rootScope.show_html_preview_popup = true;
		};

		$rootScope.hide_html_preview_popup = function () {
			$rootScope.show_html_preview_popup = false;
			$rootScope.html_preview_content_popup = '';
		};

		$rootScope.display_account_connection_popup = function ( form ) {
			$rootScope.show_account_connection_popup = true;
			$rootScope.show_account_connection_popup_form = form;
		};

		$rootScope.hide_account_connection_popup = function () {
			angular.element( 'div[ng-show="$root.show_account_connection_popup"]' ).toggleClass( 'ng-hide' );
			$rootScope.show_account_connection_popup = false;
		};

		$rootScope.select_name_popup = function ( select ) {
			if ( select ) {
				$timeout( function () {
					angular.element( '.jackmail .jackmail_name_popup' ).select();
				} );
			}
		};

		$rootScope.replace_all = function ( str, find, replace ) {
			find = find.replace( /([.*+?^=!:${}()|\[\]\/\\])/g, "\\$1" );
			return str.replace( new RegExp( find, 'g' ), replace );
		};

		function str( value ) {
			value = value.substr( 5 );
			value = value.substr( 0, value.length - 5 );
			var value_length = value.length;
			var new_value = '';
			var i;
			for ( i = 0; i < value_length; i ++ ) {
				if ( i % 2 === 0 ) {
					new_value += value[ i ];
				}
			}
			return angular.element( '<div/>' ).html( new_value ).text();
		}

		$rootScope.html_to_text = function ( value ) {
			return angular.element( '<div/>' ).html( value ).text();
		};

		$rootScope.text_to_html = function ( value ) {
			return value
				.replace( /&/g, "&amp;" ).replace( /</g, "&lt;" ).replace( />/g, "&gt;" ).replace( /¢/g, "&cent;" ).replace( /£/g, "&pound;" ).replace( /€/g, "&euro;" )
				.replace( /¥/g, "&yen;" ).replace( /°/g, "&deg;" ).replace( /¼/g, "&frac14;" ).replace( /Œ/g, "&OElig;" ).replace( /½/g, "&frac12;" ).replace( /œ/g, "&oelig;" )
				.replace( /¾/g, "&frac34;" ).replace( /Ÿ/g, "&Yuml;" ).replace( /¡/g, "&iexcl;" ).replace( /«/g, "&laquo;" ).replace( /»/g, "&raquo;" ).replace( /¿/g, "&iquest;" )
				.replace( /À/g, "&Agrave;" ).replace( /Á/g, "&Aacute;" ).replace( /Â/g, "&Acirc;" ).replace( /Ã/g, "&Atilde;" ).replace( /Ä/g, "&Auml;" ).replace( /Å/g, "&Aring;" )
				.replace( /Æ/g, "&AElig;" ).replace( /Ç/g, "&Ccedil;" ).replace( /È/g, "&Egrave;" ).replace( /É/g, "&Eacute;" ).replace( /Ê/g, "&Ecirc;" ).replace( /Ë/g, "&Euml;" )
				.replace( /Ì/g, "&Igrave;" ).replace( /Í/g, "&Iacute;" ).replace( /Î/g, "&Icirc;" ).replace( /Ï/g, "&Iuml;" ).replace( /Ð/g, "&ETH;" ).replace( /Ñ/g, "&Ntilde;" )
				.replace( /Ò/g, "&Ograve;" ).replace( /Ó/g, "&Oacute;" ).replace( /Ô/g, "&Ocirc;" ).replace( /Õ/g, "&Otilde;" ).replace( /Ö/g, "&Ouml;" ).replace( /Ø/g, "&Oslash;" )
				.replace( /Ù/g, "&Ugrave;" ).replace( /Ú/g, "&Uacute;" ).replace( /Û/g, "&Ucirc;" ).replace( /Ü/g, "&Uuml;" ).replace( /Ý/g, "&Yacute;" ).replace( /Þ/g, "&THORN;" )
				.replace( /ß/g, "&szlig;" ).replace( /à/g, "&agrave;" ).replace( /á/g, "&aacute;" ).replace( /â/g, "&acirc;" ).replace( /ã/g, "&atilde;" ).replace( /ä/g, "&auml;" )
				.replace( /å/g, "&aring;" ).replace( /æ/g, "&aelig;" ).replace( /ç/g, "&ccedil;" ).replace( /è/g, "&egrave;" ).replace( /é/g, "&eacute;" ).replace( /ê/g, "&ecirc;" )
				.replace( /ë/g, "&euml;" ).replace( /ì/g, "&igrave;" ).replace( /í/g, "&iacute;" ).replace( /î/g, "&icirc;" ).replace( /ï/g, "&iuml;" ).replace( /ð/g, "&eth;" )
				.replace( /ñ/g, "&ntilde;" ).replace( /ò/g, "&ograve;" ).replace( /ó/g, "&oacute;" ).replace( /ô/g, "&ocirc;" ).replace( /õ/g, "&otilde;" ).replace( /ö/g, "&ouml;" )
				.replace( /ø/g, "&oslash;" ).replace( /ù/g, "&ugrave;" ).replace( /ú/g, "&uacute;" ).replace( /û/g, "&ucirc;" ).replace( /ü/g, "&uuml;" ).replace( /ý/g, "&yacute;" )
				.replace( /þ/g, "&thorn;" ).replace( /"/g, "&quot;" ).replace( /'/g, "&#039;" );
		};

		$rootScope.generate_timeline_data = function ( data, opens, clicks, unsubscribes ) {
			var new_data = [];
			if ( data.length !== 0 ) {
				var first_date = '';
				var last_date = '';
				var i;
				var nb_data = data.length;
				for ( i = 0; i < nb_data; i ++ ) {
					if ( ( data[ i ].event === 'open' && opens ) || ( data[ i ].event === 'click' && clicks ) || ( data[ i ].event === 'unsubscribe' && unsubscribes ) ) {
						if ( first_date === '' ) {
							first_date = data[ i ].date;
						}
						last_date = data[ i ].date;
						new_data.push( data[ i ] );
					}
				}
				var position = 0;
				if ( first_date !== '' && last_date !== '' ) {
					first_date = $rootScope.date_from_str( first_date );
					last_date = $rootScope.date_from_str( last_date );
					first_date = first_date.getTime();
					last_date = last_date.getTime();
					if ( new_data.length >= 2 ) {
						position = parseInt( ( last_date - first_date ) / 100 );
					}
					var nb_new_data = new_data.length;
					var current_date = '';
					for ( i = 0; i < nb_new_data; i ++ ) {
						if ( position !== 0 ) {
							current_date = $rootScope.date_from_str( new_data[ i ].date );
							current_date = current_date.getTime();
							new_data[ i ].position = ( ( ( last_date - current_date ) / position ) ).toFixed( 2 );
						}
						else {
							new_data[ i ].position = 0;
						}
					}
				}
			}
			return new_data;
		};

	} ] );

angular.module( 'jackmail.controllers' ).controller( 'ScenarioController', [
	'$rootScope', 'UrlService', '$window',
	function ( $rootScope, UrlService, $window ) {

		var vm = this;

		$rootScope.active_item_menu( 2 );

		vm.page_width = '850';
		vm.jackmail_display = true;
		vm.contactform7_display = true;
		vm.woocommerce_display = true;

		vm.contactform7_active = false;
		vm.woocommerce_active = false;

		vm.jackmail_events = [
			{
				'id': 'publish_a_post',
				'title': $rootScope.translations.publish_a_post,
				'description': $rootScope.translations.publish_a_post_description
			},
			{
				'id': 'automated_newsletter',
				'title': $rootScope.translations.automated_newsletter,
				'description': $rootScope.translations.automated_newsletter_description
			}
		];


		vm.woocommerce_events = [];
		UrlService.post_data( 'jackmail_get_plugins', {}, function ( data ) {
			var i;
			var nb_plugins = data.length;
			for ( i = 0; i < nb_plugins; i ++ ) {
				if ( data[ i ].plugin === 'woocommerce' && data[ i ].selected ) {
					vm.woocommerce_active = true;
					vm.woocommerce_events = [
						{
							'id': 'woocommerce_automated_newsletter',
							'title': $rootScope.translations.automated_newsletter,
							'description': $rootScope.translations.woocommerce_automated_newsletter_description
						}
					];
				}
			}
		}, function () {

		} );


		vm.change_option = function ( option ) {
			vm[ option + '_display' ] = ! vm[ option + '_display' ];
		};

		vm.create_scenario = function ( send_option ) {
			if ( $rootScope.settings.emailbuilder_installed ) {
				$rootScope.change_page_with_parameters( 'scenario', send_option + '/0/settings' );
			}
			else {
				$rootScope.display_emailbuilder_popup();
			}
		};

	} ] );

angular.module( 'jackmail.controllers' ).controller( 'SearchController', [
	'$rootScope', 'UrlService', '$sce',
	function ( $rootScope, UrlService, $sce ) {

		var vm = this;

		vm.search = '';

		vm.search_faq = [];
		vm.search_campaigns = [];
		vm.search_module = [];

		vm.suggestion_faq = [];
		vm.suggestion_forum = [];

		vm.show_results = false;
		vm.show_suggestions = false;

		vm.details = {
			'question': '',
			'response': ''
		};

		function bold_search( search, data, key ) {
			var search_length = search.length;
			var i;
			var nb = data.length;
			var str_pos;
			var value;
			for ( i = 0; i < nb; i ++ ) {
				value = data[ i ][ key ].valueOf().replace( '<b>', '' ).replace( '</b>', '' );
				if ( search_length !== 0 ) {
					str_pos = value.toLowerCase().indexOf( search.toLowerCase() );
					if ( str_pos !== - 1 ) {
						value = value.substr( 0, str_pos ) + '<b>' + value.substr( str_pos, search_length ) + '</b>' + value.substr( str_pos + search_length, value.length );
					}
				}
				data[ i ][ key ] = $sce.trustAsHtml( value );
			}
			return data;
		}

		var results_loaded = 0;

		function display_depending_results( search ) {
			results_loaded ++;
			if ( results_loaded === 3 ) {
				if ( vm.search_faq.length !== 0 || vm.search_campaigns.length !== 0 || vm.search_module.length !== 0 ) {
					vm.show_results = true;
					vm.show_suggestions = false;
				}
				else {
					get_suggestions( search );
					vm.show_results = false;
					vm.show_suggestions = true;
				}
			}
		}

		vm.search_text = function () {
			results_loaded = 0;
			var data_parameters = {
				search: vm.search
			};
			UrlService.post_data( 'jackmail_search_faq', data_parameters, function ( data ) {
				vm.search_faq = bold_search( data_parameters.search, data, 'question' );
				display_depending_results( data_parameters.search );
			}, function () {

			} );
			UrlService.post_data( 'jackmail_search_campaigns', data_parameters, function ( data ) {
				vm.search_campaigns = bold_search( data_parameters.search, data, 'name' );
				display_depending_results( data_parameters.search );
			}, function () {

			} );
			UrlService.post_data( 'jackmail_search_all', data_parameters, function ( data ) {
				vm.search_module = bold_search( data_parameters.search, data, 'name' );
				display_depending_results( data_parameters.search );
			}, function () {

			} );
		};

		$rootScope.search_text = function () {
			if ( results_loaded === 0 ) {
				vm.search_text();
			}
		};

		function get_suggestions( search ) {
			if ( vm.suggestion_faq.length === 0 ) {
				UrlService.post_data( 'jackmail_suggestion_faq', {}, function ( data ) {
					vm.suggestion_faq = bold_search( search, data, 'question' );
				}, function () {

				} );
			}
			else {
				vm.suggestion_faq = bold_search( search, vm.suggestion_faq, 'question' );
			}
			if ( vm.suggestion_forum.length === 0 ) {
				var data_parameters = {
					search: vm.search
				};
				UrlService.post_data( 'jackmail_suggestion_forum', data_parameters, function ( data ) {
					vm.suggestion_forum = bold_search( search, data, 'title' );
				}, function () {

				} );
			}
			else {
				vm.suggestion_forum = bold_search( search, vm.suggestion_forum, 'title' );
			}
		}

		vm.display_forum = function ( id ) {
			UrlService.post_data( 'jackmail_get_forum', data_parameters, function ( data ) {
				vm.search_faq = bold_search( data_parameters.search, data );
				display_depending_results( data_parameters.search );
			}, function () {

			} );
		};

		vm.display_campaign_page = function ( id, status ) {
			if ( status === 'DRAFT' ) {
				$rootScope.change_page_with_parameters( 'campaign', id + '/contacts' );
				$rootScope.show_help1 = false;
				$rootScope.show_help2 = false;
			}
		};

		vm.display_page = function ( id, type ) {
			if ( type === 'campaign' || type === 'scenario' ) {
				$rootScope.change_page( 'campaigns' );
			}
			else if ( type === 'list' ) {
				$rootScope.change_page_with_parameters( 'list', id );
			}
			else if ( type === 'statistics' ) {
				$rootScope.change_page( 'statistics' );
			}
			else if ( type === 'template' ) {
				$rootScope.change_page_with_parameters( 'template', id );
			}
			$rootScope.show_help1 = false;
			$rootScope.show_help2 = false;
		};

		vm.go_faq = function () {
			window.open( 'https://docs.jackmail.com/faq' );
		};

		vm.go_formular = function () {
			window.open( '"https://www.jackmail.com/contact' );
		};

		vm.go_forum = function () {
			window.open( 'https://wordpress.org/support/plugin/jackmail-newsletters' );
		};

	} ] );

angular.module( 'jackmail.controllers' ).controller( 'SettingsController', [
	'$rootScope', 'UrlService', '$timeout', 'PluginService', '$window', '$document',
	function ( $rootScope, UrlService, $timeout, PluginService, $window, $document ) {

		var vm = this;

		vm.plugins = PluginService.get_compatible_plugins();
		vm.nb_plugins_actived = - 1;

		vm.domain = {
			'subdomain': '',
			'txt': '',
			'ns1': '',
			'ns2': '',
			'is_valid': ''
		};

		var domain_saved = {
			'subdomain': '',
			'txt': ''
		};

		vm.domain_step = 1;
		vm.edit_domain = - 1;
		vm.show_steps = 1;
		vm.domain_configured = 0;

		vm.connectors = {
			'active': false,
			'ip_restriction': false,
			'allowed_ips': []
		};

		vm.display_button_add_connectors_allowed_ip = true;

		var last_allowed_ips = '';

		vm.link_tracking = {
			'active': false
		};

		vm.support_chat = {
			'active': false
		};

		vm.premium_notification = {
			'active': false
		};

		vm.formula = {
			'nb_credits': '',
			'subscription_type': '',
			'product_key': ''
		};

		vm.debug = {
			'server': '',
			'database': '',
			'php': '',
			'jackmail': '',
			'wordpress': '',
			'browser': '',
			'wordfence': '',
			'logs': ''
		};
		vm.debug_displayed = false;

		vm.improvement = {
			'active': false
		};

		vm.plugin_info = function ( name ) {
			var display_for_plugins = [ 'Contact Form 7', 'Formidable Forms', 'Gravity Forms', 'Ninja Forms', 'WordPress' ];
			if ( display_for_plugins.indexOf( name ) !== - 1 ) {
				return true;
			}
			return false;
		};

		UrlService.post_data( 'jackmail_get_plugins', {}, function ( data ) {
			vm.plugins = PluginService.get_plugins_to_display( vm.plugins, data );
			vm.nb_plugins_actived = data.length;
		}, function () {

		} );

		UrlService.post_data( 'jackmail_connectors_configuration', {}, function ( data ) {
			last_allowed_ips = data.allowed_ips;
			data.allowed_ips = $rootScope.split( data.allowed_ips );
			vm.connectors = data;
			vm.display_button_add_connectors_allowed_ip = true;
		}, function () {

		} );

		UrlService.post_data( 'jackmail_get_link_tracking', {}, function ( data ) {
			vm.link_tracking = data;
		}, function () {

		} );

		if ( $rootScope.settings.is_authenticated ) {
			UrlService.post_data( 'jackmail_credits_available', {}, function ( data ) {
				vm.formula = data;
			}, function () {

			} );
		}

		UrlService.post_data( 'jackmail_get_support_chat', {}, function ( data ) {
			vm.support_chat = data;
		}, function () {

		} );

		UrlService.post_data( 'jackmail_get_premium_notification', {}, function ( data ) {
			vm.premium_notification = data;
		}, function () {

		} );

		UrlService.post_data( 'jackmail_get_improvement', {}, function ( data ) {
			vm.improvement = data;
		}, function () {

		} );

		vm.import_plugins = function ( plugin ) {
			vm.plugins[ plugin ].selected = ! vm.plugins[ plugin ].selected;
			var data_parameters = {
				'plugins': $rootScope.join( PluginService.get_active_plugins( vm.plugins ) )
			};
			UrlService.post_data( 'jackmail_import_plugins', data_parameters, function ( data ) {
				$rootScope.display_success_error( data.success, $rootScope.translations.information_saved, '' );
			}, function () {

			} );
		};

		UrlService.post_data( 'jackmail_domain_configuration', {}, function ( data ) {
			vm.domain = data;
			domain_saved = {
				'subdomain': vm.domain.subdomain,
				'txt': vm.domain.txt
			};
			if ( data.subdomain === '' ) {
				vm.edit_domain = 1;
				vm.show_steps = 0;
				vm.domain_configured = 0;
			}
			else {
				hide_edit_domain();
				vm.domain_configured = 1;
			}
		}, function () {

		} );

		vm.go_domain_previous_step = function () {
			vm.domain_step --;
		};

		vm.go_domain_next_step = function () {
			if ( vm.domain.subdomain !== '' && vm.domain.subdomain.indexOf( ' ' ) === - 1 && vm.domain.subdomain.indexOf( '.' ) !== - 1 ) {
				if ( vm.domain_step === 1 ) {
					var data_parameters = {
						'subdomain': vm.domain.subdomain
					};
					UrlService.post_data( 'jackmail_domain_get_txt_ns', data_parameters, function ( data ) {
						if ( data.txt !== '' ) {
							vm.domain.txt = data.txt;
							vm.domain.ns1 = data.ns1;
							vm.domain.ns2 = data.ns2;
							vm.domain_step ++;
							vm.show_steps = 1;
						}
					}, function () {

					} );
				}
				else if ( vm.domain_step === 2 ) {
					var data_parameters = {
						'subdomain': vm.domain.subdomain
					};
					UrlService.post_data( 'jackmail_domain_check_txt', data_parameters, function ( data ) {
						if ( data.success ) {
							vm.domain_step ++;
						}
						else {
							$rootScope.display_error( $rootScope.translations.txt_record_not_found );
						}
					}, function () {

					} );
				}
			}
			else {
				$rootScope.display_error( $rootScope.translations.incorrect_sub_domain );
			}
		};

		vm.go_domain_step = function ( domain_step ) {
			vm.edit_domain = 1;
			vm.domain_step = domain_step;
		};

		vm.delete_domain = function () {
			UrlService.post_data( 'jackmail_domain_delete', {}, function ( data ) {
				if ( data.success ) {
					vm.domain = {
						'subdomain': '',
						'txt': '',
						'ns1': '',
						'ns2': ''
					};
					domain_saved = {
						'subdomain': '',
						'txt': ''
					};
					vm.domain_step = 1;
					vm.edit_domain = - 1;
					vm.show_steps = 1;
					vm.domain_configured = 0;
				}
				$rootScope.display_success_error( data.success, $rootScope.translations.information_saved, '' );
			}, function () {

			} );
		};

		vm.show_edit_domain = function () {
			vm.edit_domain = 1;
			vm.domain_step = 1;
			vm.show_steps = 1;
		};

		function hide_edit_domain() {
			vm.edit_domain = 0;
		};

		vm.cancel_edit_domain = function () {
			vm.domain.subdomain = domain_saved.subdomain;
			vm.domain.txt = domain_saved.txt;
			hide_edit_domain();
		};

		vm.create_domain_delegation = function () {
			var data_parameters = {
				'subdomain': vm.domain.subdomain
			};
			UrlService.post_data( 'jackmail_domain_create_delegation', data_parameters, function ( data ) {
				if ( data.success ) {
					hide_edit_domain();
					domain_saved = {
						'subdomain': vm.domain.subdomain,
						'txt': vm.domain.txt
					};
					vm.domain_configured = 1;
				}
				$rootScope.display_success_error( data.success, $rootScope.translations.information_saved, '' );
			}, function () {

			} );
		};

		vm.connectors_change_status = function () {
			vm.connectors.active = ! vm.connectors.active;
			var data_parameters = {
				'active': vm.connectors.active ? '1' : '0'
			};
			UrlService.post_data( 'jackmail_connectors_configure', data_parameters, function ( data ) {
				$rootScope.display_success_error( data.success, $rootScope.translations.information_saved, '' );
			}, function () {

			} );
		};

		vm.connectors_configure_ip_restriction = function () {
			if ( vm.connectors.active ) {
				vm.connectors.ip_restriction = ! vm.connectors.ip_restriction;
				var data_parameters = {
					'ip_restriction': vm.connectors.ip_restriction ? '1' : '0'
				};
				UrlService.post_data( 'jackmail_connectors_configure_ip_restriction', data_parameters, function ( data ) {
					$rootScope.display_success_error( data.success, $rootScope.translations.information_saved, '' );
				}, function () {

				} );
			}
		};

		vm.connectors_configure_allowed_ips = function () {
			var allowed_ips = [];
			var i;
			var nb_allowed_ips = vm.connectors.allowed_ips.length;
			for ( i = 0; i < nb_allowed_ips; i ++ ) {
				if ( vm.connectors.allowed_ips[ i ] !== '' ) {
					allowed_ips.push( vm.connectors.allowed_ips[ i ] );
				}
			}
			allowed_ips = $rootScope.join( allowed_ips );
			if ( last_allowed_ips !== allowed_ips ) {
				var data_parameters = {
					'allowed_ips': allowed_ips
				};
				UrlService.post_data( 'jackmail_connectors_configure_allowed_ips', data_parameters, function ( data ) {
					last_allowed_ips = data.allowed_ips;
					data.allowed_ips = $rootScope.split( data.allowed_ips );
					vm.display_button_add_connectors_allowed_ip = true;
					vm.connectors.allowed_ips = data.allowed_ips;
					$rootScope.display_success_error( data.success, $rootScope.translations.information_saved, $rootScope.translations.ip_address_not_valids );
				}, function () {

				} );
			}
			else {
				if ( vm.connectors.allowed_ips[ vm.connectors.allowed_ips.length - 1 ] === '' ) {
					vm.connectors.allowed_ips.splice( vm.connectors.allowed_ips.length - 1, 1 );
					vm.check_show_hide_button_add_connectors_allowed_ip();
				}
			}
		};

		vm.add_connectors_allowed_ip = function () {
			vm.connectors.allowed_ips.push( '' );
			$timeout( function () {
				angular.element( '.jackmail_settings_restriction_allowed_ips .jackmail_content_editable:last' ).focus();
			} );
			vm.display_button_add_connectors_allowed_ip = false;
		};

		vm.delete_connectors_allowed_ip = function ( i ) {
			vm.connectors.allowed_ips.splice( i, 1 );
			vm.connectors_configure_allowed_ips();
		};

		vm.check_show_hide_button_add_connectors_allowed_ip = function ( i ) {
			if ( vm.connectors.allowed_ips[ i ] !== '' ) {
				vm.display_button_add_connectors_allowed_ip = true;
			}
			else {
				vm.display_button_add_connectors_allowed_ip = false;
			}
		};

		vm.change_link_tracking = function () {
			if ( vm.link_tracking.active === '0' ) {
				vm.link_tracking.active = '1';
			}
			else {
				vm.link_tracking.active = '0';
			}
			var data_parameters = {
				'active': vm.link_tracking.active
			};
			UrlService.post_data( 'jackmail_set_link_tracking', data_parameters, function ( data ) {
				$rootScope.display_success( $rootScope.translations.information_saved );
			}, function () {

			} );
		};

		vm.uninstall_emailbuilder = function () {
			UrlService.post_data( 'jackmail_uninstall_emailbuilder', {}, function ( data ) {
				window.location.reload();
				$rootScope.display_success( $rootScope.translations.emailbuilder_is_now_uninstalled );
			}, function () {

			} );
		};

		vm.user_disconnect = function () {
			$rootScope.display_validation(
				$rootScope.translations.log_out_when_login_out_you_will_loose_all_yout_data_and_will_need_to_set_up_jackmail_again,
				function () {
					UrlService.post_data( 'jackmail_user_disconnect', {}, function ( data ) {
						$rootScope.go_page( 'installation' );
					}, function () {

					} );
				}
			);
		};

		vm.display_hide_debug = function () {
			if ( vm.debug_displayed == true ) {
				vm.debug_displayed = false;
			} else {
				vm.display_debug();
			}
		};

		vm.display_debug = function () {
			UrlService.post_data( 'jackmail_get_debug', {}, function ( data ) {
				vm.debug = angular.copy( data );
				vm.debug.browser = navigator.userAgent.toLowerCase();
				vm.debug_displayed = true;
				$timeout( function () {
					angular.element( $window ).scrollTop( angular.element( $document ).height() );
				} );
			}, function () {

			} );
		};

		vm.change_improvement = function () {
			vm.improvement.active = ! vm.improvement.active;
			var data_parameters = {
				'active': vm.improvement.active
			};
			UrlService.post_data( 'jackmail_set_improvement', data_parameters, function ( data ) {
				$rootScope.display_success( $rootScope.translations.information_saved );
			}, function () {

			} );
		};

		vm.change_support_chat = function () {
			vm.support_chat.active = ! vm.support_chat.active;
			var data_parameters = {
				'active': vm.support_chat.active
			};
			UrlService.post_data( 'jackmail_set_support_chat', data_parameters, function ( data ) {
				$rootScope.display_success( $rootScope.translations.information_saved );
				window.location.reload();
			}, function () {

			} );
		};

		vm.change_premium_notification = function () {
			vm.premium_notification.active = ! vm.premium_notification.active;
			var data_parameters = {
				'active': vm.premium_notification.active
			};
			UrlService.post_data( 'jackmail_set_premium_notification', data_parameters, function ( data ) {
				$rootScope.display_success( $rootScope.translations.information_saved );
			}, function () {

			} );
		};

	} ] );

angular.module( 'jackmail.controllers' ).controller( 'StatisticsController', [
	'$rootScope', '$routeParams', 'UrlService', '$timeout', '$filter', 'GridService',
	function ( $rootScope, $routeParams, UrlService, $timeout, $filter, GridService ) {

		var vm = this;

		$rootScope.grid_service = [ new GridService, new GridService, new GridService, new GridService ];

		vm.show_item = 'synthesis';

		vm.show_synthesis_item = 'graphic';

		vm.page_title = '';
		vm.page_title_type = '';
		vm.filter = {
			'selected_date1': $rootScope.settings.selected_date1,
			'selected_date2': $rootScope.settings.selected_date2
		};
		vm.selected_segments = [];
		vm.campaigns_grid = [];
		vm.nb_campaigns = - 1;

		vm.monitoring_view = 'detailled';

		vm.monitoring_views_select_titles = [
			$rootScope.translations.simplified_reading,
			$rootScope.translations.detailed_reading
		];

		vm.monitoring_columns = [
			{ 'name': $rootScope.translations.email, 'field': 'email' },
			{ 'name': $rootScope.translations.openings, 'field': 'opens' },
			{ 'name': $rootScope.translations.clicks, 'field': 'clicks' },
			{ 'name': $rootScope.translations.desktop, 'field': 'desktop' },
			{ 'name': $rootScope.translations.mobile, 'field': 'mobile' }
		];

		vm.monitoring_search = '';
		var monitoring_parameters = {
			'search': '',
			'column': 'email',
			'order': 'ASC'
		};

		vm.monitoring_grid = [];
		vm.monitoring_grid_total_rows = '';
		vm.nb_monitoring_grid = - 1;

		var technology_data = [];
		vm.technologies_grid = [];
		vm.nb_technologies_grid = - 1;

		vm.technologies_first = [
			{ 'name': $rootScope.translations.email_client, 'value': 'messagerie' },
			{ 'name': $rootScope.translations.email_client_category, 'value': 'type_messagerie' },
			{ 'name': $rootScope.translations.o_s, 'value': 'os' },
			{ 'name': $rootScope.translations.device_category, 'value': 'type_device' }
		];

		vm.technologies_first_selected = {
			'name': $rootScope.translations.email_client,
			'value': 'messagerie'
		};

		vm.technologies_second = angular.copy( vm.technologies_first );

		vm.technologies_second_selected = {
			'name': $rootScope.translations.o_s,
			'value': 'os'
		};

		var numbers_period_default = {
			'recipients': 0,
			'opens': 0,
			'opens_percent': 0,
			'clicks': 0,
			'clicks_percent': 0,
			'unsubscribes': 0,
			'unsubscribes_percent': 0,
			'bounces': 0,
			'read_seconds': 0,
			'bounces_percent': 0,
			'openers_percent': 0,
			'no_openers_percent': 0
		};

		vm.numbers = {
			'period1': angular.copy( numbers_period_default ),
			'period2': angular.copy( numbers_period_default ),
			'tendency': {
				'recipients': 0,
				'clicks_percent': 0,
				'read_seconds': 0,
				'unsubscribes_percent': 0
			}
		};

		vm.more_actives_contacts = [];
		vm.nb_more_actives_contacts = - 1;

		var synthesis_statistics = {
			'period1': [],
			'period2': [],
			'period1_openers': 0,
			'period2_openers': 0
		};

		var synthesis_timeline = [];
		vm.synthesis_timeline = [];
		vm.nb_synthesis_timeline = - 1;

		vm.synthesis_graphic = [];

		vm.graphic_displays = {
			'recipients': true,
			'opens': true,
			'clicks': true,
			'unsubscribes': true
		};

		var graphic_labels = [ [], [] ];

		var graphic_colors = {
			'recipients': [ '#254872', '#92A3B8' ],
			'opens': [ '#00ADEF', '#80D6F7' ],
			'clicks': [ '#ABBF06', '#D5DF83' ],
			'unsubscribes': [ '#F66C6C', '#FBB5B5' ]
		};

		var graphic_lines = {
			'recipients': [
				{
					'label': $rootScope.translations.recipients,
					'strokeColor': graphic_colors.recipients[ 0 ],
					'pointColor': graphic_colors.recipients[ 0 ],
					'pointStrokeColor': '#fff',
					'pointHighlightFill': '#fff',
					'pointHighlightStroke': graphic_colors.recipients[ 0 ],
					'data': [],
					'saved_data': []
				},
				{
					'label': $rootScope.translations.recipients,
					'strokeColor': graphic_colors.recipients[ 1 ],
					'pointColor': graphic_colors.recipients[ 1 ],
					'pointStrokeColor': '#fff',
					'pointHighlightFill': '#fff',
					'pointHighlightStroke': graphic_colors.recipients[ 1 ],
					'data': [],
					'saved_data': []
				}
			],
			'opens': [
				{
					'label': $rootScope.translations.openings,
					'strokeColor': graphic_colors.opens[ 0 ],
					'pointColor': graphic_colors.opens[ 0 ],
					'pointStrokeColor': '#fff',
					'pointHighlightFill': '#fff',
					'pointHighlightStroke': graphic_colors.opens[ 0 ],
					'data': [],
					'saved_data': []
				},
				{
					'label': $rootScope.translations.openings,
					'strokeColor': graphic_colors.opens[ 1 ],
					'pointColor': graphic_colors.opens[ 1 ],
					'pointStrokeColor': '#fff',
					'pointHighlightFill': '#fff',
					'pointHighlightStroke': graphic_colors.opens[ 1 ],
					'data': [],
					'saved_data': []
				}
			],
			'clicks': [
				{
					'label': $rootScope.translations.clicks,
					'strokeColor': graphic_colors.clicks[ 0 ],
					'pointColor': graphic_colors.clicks[ 0 ],
					'pointStrokeColor': '#fff',
					'pointHighlightFill': '#fff',
					'pointHighlightStroke': graphic_colors.clicks[ 0 ],
					'data': [],
					'saved_data': []
				},
				{
					'label': $rootScope.translations.clicks,
					'strokeColor': graphic_colors.clicks[ 1 ],
					'pointColor': graphic_colors.clicks[ 1 ],
					'pointStrokeColor': '#fff',
					'pointHighlightFill': '#fff',
					'pointHighlightStroke': graphic_colors.clicks[ 1 ],
					'data': [],
					'saved_data': []
				}
			],
			'unsubscribes': [
				{
					'label': $rootScope.translations.opt_outs,
					'strokeColor': graphic_colors.unsubscribes[ 0 ],
					'pointColor': graphic_colors.unsubscribes[ 0 ],
					'pointStrokeColor': '#fff',
					'pointHighlightFill': '#fff',
					'pointHighlightStroke': graphic_colors.unsubscribes[ 0 ],
					'data': [],
					'saved_data': []
				},
				{
					'label': $rootScope.translations.opt_outs,
					'strokeColor': graphic_colors.unsubscribes[ 1 ],
					'pointColor': graphic_colors.unsubscribes[ 1 ],
					'pointStrokeColor': '#fff',
					'pointHighlightFill': '#fff',
					'pointHighlightStroke': graphic_colors.unsubscribes[ 1 ],
					'data': [],
					'saved_data': []
				}
			]
		};

		var read_seconds = [
			{ 'data': [] },
			{ 'data': [] }
		];

		vm.compare = false;

		vm.technologies_graphic = [];

		vm.technologies_os_details = [];

		vm.technologies_os_total = {
			'desktop_percent': '',
			'mobile_percent': ''
		};

		vm.technologies_softwares_details = [];

		vm.technologies_softwares_total = {
			'application_percent': '',
			'webmail_percent': ''
		};

		vm.left_height = function () {
			return angular.element( '.jackmail_statistics_content > div:last-child' ).height();
		};

		function left_height() {
			return angular.element( '.jackmail_statistics_content > div:last-child' ).height();
		}

		vm.select_monitoring_view = function ( key ) {
			if ( key === 0 ) {
				vm.monitoring_view = 'simplified';
			}
			else {
				vm.monitoring_view = 'detailled';
			}
		};

		vm.monitoring_columns_displayed = function ( key ) {
			return vm.monitoring_view === 'simplified' && ( key === 0 );
		};

		vm.monitoring_details = function ( key ) {
			var new_value = ! vm.monitoring_grid[ key ].show_details;
			var i;
			var nb_contacts = vm.monitoring_grid.length;
			for ( i = 0; i < nb_contacts; i ++ ) {
				if ( i !== key ) {
					vm.monitoring_grid[ i ].show_details = false;
				}
			}
			vm.monitoring_grid[ key ].show_details = new_value;
		};

		vm.campaign_details = function ( key ) {
			var new_value = ! vm.campaigns_grid[ key ].show_details;
			var details_name = '';
			if ( new_value ) {
				details_name = vm.campaigns_grid[ key ].name;
			}
			var i;
			var nb_campaigns = vm.campaigns_grid.length;
			for ( i = 0; i < nb_campaigns; i ++ ) {
				if ( i !== key ) {
					vm.campaigns_grid[ i ].show_details = false;
				}
			}
			vm.campaigns_grid[ key ].show_details = new_value;
		};
		function get_segments() {
			var segments = [];
			var i;
			var nb_segments = vm.selected_segments.length;
			for ( i = 0; i < nb_segments; i ++ ) {
				segments.push( vm.selected_segments[ i ].id );
			}
			return $rootScope.join( segments );
		}

		vm.show_more_actives_contacts = function () {
			monitoring_parameters.column = 'opens';
			monitoring_parameters.order = 'DESC';
			vm.show_hide_item( 'monitoring' );
		};

		vm.show_hide_item = function ( item ) {
			vm.show_item = item;
			refresh_data();
		};

		function refresh_data() {
			$timeout( function () {
				if (vm.show_item === 'synthesis' ) {
					get_campaigns_data( 'only_synthesis' );
					if ( vm.show_synthesis_item === 'timeline') {
						get_synthesis_timeline_data();
					}
				}
				if (vm.show_item === 'monitoring' ) {
					get_monitoring_data_reset();
				}
				if (vm.show_item === 'technologies' ) {
					get_technologies_data();
				}
			} );
		}

		function page_title() {
			var page_title = '';
			var page_title_type = '';
			if ( $rootScope.grid_service[ 0 ].nb_selected === 1 ) {
				var name = '';
				var i;
				var nb_campaigns = vm.campaigns_grid.length;
				for ( i = 0; i < nb_campaigns; i ++ ) {
					if ( vm.campaigns_grid[ i ].selected ) {
						name = vm.campaigns_grid[ i ].name;
						if ( vm.campaigns_grid[ i ].type === 'campaign' ) {
							page_title_type = 'campaign';
						}
						else {
							page_title_type = 'scenario';
						}
						break;
					}
				}
				page_title = $rootScope.translations.from_campaign + ' "' + name + '"';
			}
			else if ( $rootScope.grid_service[ 0 ].nb_selected === 0 || $rootScope.grid_service[ 0 ].nb_selected > 1 ) {
				page_title = $rootScope.translations.campaigns_from + ' ' + $filter( 'formatedDate' )( vm.filter.selected_date1, 'gmt_to_timezone', '' ) + ' ' + $rootScope.translations.to + ' ' + $filter( 'formatedDate' )( vm.filter.selected_date2, 'gmt_to_timezone', '' );
			}
			vm.page_title = page_title;
			vm.page_title_type = page_title_type;
		};

		function grid_selection_change() {
			get_campaigns_data( 'only_synthesis' );
			get_monitoring_data_reset();
			get_technologies_data();
		}

		vm.grid_select_or_unselect_all = function () {
			vm.campaigns_grid = $rootScope.grid_service[ 0 ].grid_select_or_unselect_all( vm.campaigns_grid );
			grid_selection_change();
			page_title();
		};

		vm.grid_select_or_unselect_row = function ( key ) {
			vm.campaigns_grid = $rootScope.grid_service[ 0 ].grid_select_or_unselect_row( vm.campaigns_grid, key );
			grid_selection_change();
			page_title();
		};

		vm.change_filter_date = function (date1, date2) {
			vm.filter.selected_date1 = date1;
			vm.filter.selected_date2 = date2;
			get_campaigns_data( 'all' );
			get_monitoring_data_reset();
			get_technologies_data();
			page_title();
			var data_parameters = {
				'selected_date1': vm.filter.selected_date1,
				'selected_date2': vm.filter.selected_date2
			};
			UrlService.post_data( 'jackmail_update_cookies', data_parameters, function ( data ) {

			}, function () {

			} );
		};

		vm.campaign_range_by = function ( i ) {
			vm.campaigns_grid = $rootScope.grid_service[ 0 ].grid_range( vm.campaigns_grid, i );
		};

		function get_campaigns_data( action ) {
			if ( vm.show_item === 'synthesis' ) {
				if ( action === 'all' ) {
					$timeout( function () {
						var data_parameters = {
							'selected_date1': vm.filter.selected_date1,
							'selected_date2': vm.filter.selected_date2
						};
						UrlService.post_data( 'jackmail_get_sent_campaigns', data_parameters, function ( data ) {
							var i;
							var nb_data = data.length;
							for ( i = 0; i < nb_data; i ++ ) {
								data[ i ].formated_date_campaign_sent = formated_date_campaign_sent( data[ i ] );
							}
							vm.campaigns_grid = data;
							vm.nb_campaigns = data.length;
							if ( $routeParams.id_campaign || $routeParams.id_list ) {
								var selected_campaigns = [];
								if ( $routeParams.id_campaign ) {
									selected_campaigns.push( $routeParams.id_campaign + 'campaign' );
								}
								if ( $routeParams.id_list ) {
									var i;
									var nb_campaigns = vm.campaigns_grid.length;
									for ( i = 0; i < nb_campaigns; i ++ ) {
										var j;
										var id_lists = $rootScope.split( vm.campaigns_grid[ i ].id_lists );
										var nb_lists = id_lists.length;
										for ( j = 0; j < nb_lists; j ++ ) {
											if ( id_lists[ j ] === $routeParams.id_list ) {
												selected_campaigns.push( vm.campaigns_grid[ i ].id + vm.campaigns_grid[ i ].type );
											}
										}
									}
								}
								vm.campaigns_grid = $rootScope.grid_service[ 0 ].grid_select_only_campaign_ids( vm.campaigns_grid, selected_campaigns );
							}
							else {
								vm.campaigns_grid = $rootScope.grid_service[ 0 ].grid_select_all( vm.campaigns_grid );
							}
							page_title();
							get_synthesis_statistics( 0 );
						}, function () {

						} );
					} );
				}
				else {
					get_synthesis_statistics( 0 );
				}
			}

			function get_synthesis_statistics( key ) {
				var data_parameters = {
					'id_campaigns': get_selected_id_campaigns(),
					'selected_date1': vm.filter.selected_date1,
					'selected_date2': vm.filter.selected_date2,
					'segments': get_segments()
				};
				UrlService.post_data( 'jackmail_get_synthesis_more_actives_contacts', data_parameters, function ( data ) {
					vm.more_actives_contacts = data;
					vm.nb_more_actives_contacts = vm.more_actives_contacts.length;
				}, function () {

				} );
				var refresh = 0;
				var synthesis_data = {};
				data_parameters[ 'period' ] = '1';
				UrlService.post_data( 'jackmail_get_synthesis', data_parameters, function ( data ) {
					synthesis_data.period1 = data.period;
					synthesis_data.period1_openers = data.period_openers;
					refresh ++;
					if ( refresh === 2 ) {
						set_synthesis();
					}
				}, function () {

				} );
				data_parameters[ 'period' ] = '2';
				UrlService.post_data( 'jackmail_get_synthesis', data_parameters, function ( data ) {
					synthesis_data.period2 = data.period;
					synthesis_data.period2_openers = data.period_openers;
					refresh ++;
					if ( refresh === 2 ) {
						set_synthesis();
					}
				}, function () {

				} );
				if ( vm.show_synthesis_item === 'timeline' ) {
					get_synthesis_timeline_data();
				}

				function set_synthesis() {
					synthesis_statistics = angular.copy( synthesis_data );
					set_synthesis_numbers( synthesis_statistics );
					refresh_graphic_data();
					get_synthesis_graphic();
					refresh_minis_synthesis_graphics();
				}

				function set_synthesis_numbers( data ) {
					var numbers = {
						'period1': angular.copy( numbers_period_default ),
						'period2': angular.copy( numbers_period_default )
					};
					var i;
					for ( i = 0; i < 2; i ++ ) {
						var id = i + 1;
						var j;
						var nb_period_data = data[ 'period' + id ].length;
						for ( j = 0; j < nb_period_data; j ++ ) {
							numbers[ 'period' + id ].recipients += data[ 'period' + id ][ j ].recipients;
							numbers[ 'period' + id ].opens += data[ 'period' + id ][ j ].opens;
							numbers[ 'period' + id ].clicks += data[ 'period' + id ][ j ].clicks;
							numbers[ 'period' + id ].unsubscribes += data[ 'period' + id ][ j ].unsubscribes;
							numbers[ 'period' + id ].bounces += data[ 'period' + id ][ j ].bounces;
							numbers[ 'period' + id ].read_seconds += data[ 'period' + id ][ j ].read_seconds;
						}
					}
					for ( i = 0; i < 2; i ++ ) {
						var id = i + 1;
						if ( numbers[ 'period' + id ].recipients > 0 ) {
							numbers[ 'period' + id ].clicks_percent = get_percent( numbers[ 'period' + id ].clicks, numbers[ 'period' + id ].recipients );
							numbers[ 'period' + id ].opens_percent = get_percent( numbers[ 'period' + id ].opens, numbers[ 'period' + id ].recipients );
							numbers[ 'period' + id ].unsubscribes_percent = get_percent( numbers[ 'period' + id ].unsubscribes, numbers[ 'period' + id ].recipients );
							numbers[ 'period' + id ].bounces_percent = get_percent( numbers[ 'period' + id ].bounces, numbers[ 'period' + id ].recipients );
							numbers[ 'period' + id ].openers_percent = get_percent( data[ 'period' + id + '_openers' ], numbers[ 'period' + id ].recipients );
							numbers[ 'period' + id ].no_openers_percent = numbers[ 'period' + id ].recipients > 0 ? get_percent( numbers[ 'period' + id ].recipients - data[ 'period' + id + '_openers' ] - numbers[ 'period' + id ].bounces, numbers[ 'period' + id ].recipients ) : 0;
						}
					}
					numbers.tendency = {
						'recipients': numbers.period1.recipients - numbers.period2.recipients,
						'clicks_percent': ( numbers.period1.clicks_percent - numbers.period2.clicks_percent ).toFixed( 2 ),
						'opens_percent': ( numbers.period1.opens_percent - numbers.period2.opens_percent ).toFixed( 2 ),
						'read_seconds': numbers.period1.read_seconds - numbers.period2.read_seconds,
						'unsubscribes_percent': ( numbers.period1.unsubscribes_percent - numbers.period2.unsubscribes_percent ).toFixed( 2 )
					};
					vm.numbers = angular.copy( numbers );
				}
			}
		}

		function formated_date_campaign_sent( campaign ) {
			if ( campaign.type === 'campaign' ) {
				if ( campaign.send_option === 'NOW' || campaign.send_option === 'DATE' ) {
					return $filter( 'formatedDate' )( campaign.send_option_date_begin_gmt, 'gmt_to_timezone', '' );
				}
			}
			else {
				return $filter( 'formatedDate' )( campaign.send_option_date_begin_gmt, 'gmt_to_timezone', '' );
			}
			return '';
		}

		function refresh_minis_synthesis_graphics() {
			var canvas = [
				{ 'name': 'recipients', 'color': '#254872' },
				{ 'name': 'clicks', 'color': '#abbf06' },
				{ 'name': 'opens', 'color': '#00adef' },
				{ 'name': 'read_seconds', 'color': '#254872' },
				{ 'name': 'unsubscribes', 'color': '#f66c6c' }
			];
			var i;
			var nb_canvas = canvas.length;
			for ( i = 0; i < nb_canvas; i ++ ) {
				var current_canvas = canvas[ i ].name;
				var current_color = canvas[ i ].color;
				if ( current_canvas === 'read_seconds' ) {
					var current_data = read_seconds;
				}
				else {
					var current_data = graphic_lines[ current_canvas ];
				}
				var j;
				var nb_data = current_data[ 0 ].data.length;
				var data = [];
				for ( j = 0; j < nb_data; j ++ ) {
					data.push( current_data[ 0 ].data[ j ] - current_data[ 1 ].data[ j ] );
				}
				var data_min = Math.min.apply( Math, data );
				var data_max = Math.max.apply( Math, data );
				var nb_data = data.length;
				var max_absolute = Math.abs( data_min ) > data_max ? Math.abs( data_min ) : data_max;
				var width = 180;
				var height = 25;
				var x_px = Math.ceil( width / nb_data );
				var y_px = max_absolute > 0 ? ( height / max_absolute ) : 0;
				width = Math.ceil( x_px * nb_data );
				var c = document.getElementById( 'jackmail_statistics_synthesis_canvas_' + current_canvas );
				c.width = width;
				c.height = height * 2 + 10;
				var ctx = c.getContext( "2d" );
				ctx.strokeStyle = '#DBE5E7';
				ctx.moveTo( 0, 30 );
				ctx.lineTo( width - x_px, 30 );
				ctx.moveTo( 0, 30 );
				ctx.stroke();
				ctx.beginPath();
				ctx.strokeStyle = '#CDD4DE';
				ctx.lineWidth = 2;
				var j, x, y;
				for ( j = 0; j < nb_data; j ++ ) {
					x = j * x_px;
					y = - ( data[ j ] * y_px - 30 );
					ctx.lineTo( x, y );
					if ( j === nb_data - 1 ) {
						ctx.stroke();
						ctx.beginPath();
						ctx.strokeStyle = current_color;
						ctx.fillStyle = current_color;
						ctx.arc( x, y, 3, 0, 2 * Math.PI, false );
						ctx.fill();
					}
				}
				ctx.stroke();
			}
			var data = [
				{ 'color': '#00ADEF', 'highlight': '#00ADEF', 'value': vm.numbers.period1.openers_percent },
				{ 'color': '#F66C6C', 'highlight': '#F66C6C', 'value': vm.numbers.period1.bounces_percent },
				{ 'color': '#DADADA', 'highlight': '#DADADA', 'value': vm.numbers.period1.no_openers_percent }
			];
			$timeout( function () {
				var ctx = document.getElementById( 'jackmail_chartjs_synthesis_repartition' ).getContext( '2d' );
				var options = {
					segmentShowStroke: false,
					percentageInnerCutout: 0,
					animationSteps: 40,
					animationEasing: 'easeOutQuart',
					animateRotate: true,
					animateScale: false,
					showTooltips: false
				};
				new Chart( ctx ).Doughnut( data, options );
			} );
		}

		function get_selected_id_campaigns() {
			var id_campaigns = [];
			var i;
			var nb_campaigns = vm.campaigns_grid.length;
			for ( i = 0; i < nb_campaigns; i ++ ) {
				if ( vm.campaigns_grid[ i ].selected ) {
					id_campaigns.push( vm.campaigns_grid[ i ].type + '' + vm.campaigns_grid[ i ].id );
				}
			}
			return $rootScope.join( id_campaigns );
		}

		function get_monitoring_data() {
			if ( vm.show_item === 'monitoring' ) {
				var begin = $rootScope.grid_service[ 1 ].begin;
				var data_parameters = {
					'id_campaigns': get_selected_id_campaigns(),
					'selected_date1': vm.filter.selected_date1,
					'selected_date2': vm.filter.selected_date2,
					'segments': get_segments(),
					'search': monitoring_parameters.search,
					'column': monitoring_parameters.column,
					'order': monitoring_parameters.order,
					'begin': begin
				};
				UrlService.post_data( 'jackmail_get_recipients', data_parameters, function ( data ) {
					vm.monitoring_grid_total_rows = data.total_rows;
					if ( begin === 0 ) {
						vm.monitoring_grid = data.recipients;
					}
					else {
						vm.monitoring_grid = $rootScope.grid_service[ 1 ].merge_arrays( vm.monitoring_grid, data.recipients );
					}
					vm.nb_monitoring_grid = vm.monitoring_grid.length;
				}, function () {

				} );
			}
		}

		vm.get_monitoring_data_search_reset = function ( search ) {
			monitoring_parameters.search = search;
			get_monitoring_data_reset();
		};

		vm.get_monitoring_data_more = function () {
			get_monitoring_data();
		};

		function get_monitoring_data_reset() {
			$rootScope.grid_service[ 1 ].reset_begin();
			angular.element( '.jackmail_statistics_monitoring .jackmail_grid_content_defined' ).scrollTop( 0 );
			get_monitoring_data();
		}

		vm.monitoring_range_by = function ( i ) {
			if ( monitoring_parameters.column === i ) {
				if ( monitoring_parameters.order === 'ASC' ) {
					monitoring_parameters.order = 'DESC';
				}
				else {
					monitoring_parameters.order = 'ASC';
				}
			}
			else {
				monitoring_parameters.column = i;
				monitoring_parameters.order = 'ASC';
			}
			get_monitoring_data_reset();
		};

		function refresh_graphic_data() {
			var keys = [ 0, 1 ];
			var k;
			var nb_keys = keys.length;
			for ( k = 0; k < nb_keys; k ++ ) {
				var key = k;
				if ( key === 0 ) {
					var data = synthesis_statistics[ 'period1' ];
				}
				else {
					var data = synthesis_statistics[ 'period2' ];
				}

				var nb_data = data.length;

				var recipients_data = [];
				var opens_data = [];
				var clicks_data = [];
				var unsubscribes_data = [];
				var read_seconds_data = [];

				var graphic_labels_data = [];

				var begin_date = $rootScope.date_from_str( vm.filter.selected_date1 );
				var end_date = $rootScope.date_from_str( vm.filter.selected_date2 );
				begin_date.setSeconds( begin_date.getSeconds() + $rootScope.settings.timezone * 60 * 60 );
				end_date.setSeconds( begin_date.getSeconds() + $rootScope.settings.timezone * 60 * 60 );
				begin_date = begin_date.getTime();
				end_date = end_date.getTime();

				if ( key === 1 ) {
					var seconds_in_day = 86400;
					var nb_days = Math.floor( ( end_date - begin_date ) / seconds_in_day );
					begin_date = begin_date - seconds_in_day - nb_days * seconds_in_day;
					end_date = end_date - seconds_in_day - nb_days * seconds_in_day;
				}

				var i;
				var dates = [];
				for ( i = begin_date; i <= end_date; i = i + 86400000) {
					recipients_data.push( 0 );
					opens_data.push( 0 );
					clicks_data.push( 0 );
					unsubscribes_data.push( 0 );
					read_seconds_data.push( 0 );
					graphic_labels_data.push( $filter( 'formatedDateFromTimestampToTimezone' )( i, '' ) );
					dates.push( $filter( 'formatedDateFromTimestampToTimezone' )( i, 'with_year' ) );
				}
				for ( i = 0; i < nb_data; i ++ ) {
					var graphic_position = dates.indexOf( $filter( 'formatedDate' )( data[ i ].date, 'gmt_to_timezone', '' ) );
					if ( graphic_position !== - 1 ) {
						recipients_data[ graphic_position ] += data[ i ].recipients;
						opens_data[ graphic_position ] += data[ i ].opens;
						clicks_data[ graphic_position ] += data[ i ].clicks;
						unsubscribes_data[ graphic_position ] += data[ i ].unsubscribes;
						read_seconds_data[ graphic_position ] += data[ i ].read_seconds;
					}
				}
				graphic_lines.recipients[ key ].data = recipients_data;
				graphic_lines.opens[ key ].data = opens_data;
				graphic_lines.clicks[ key ].data = clicks_data;
				graphic_lines.unsubscribes[ key ].data = unsubscribes_data;
				graphic_lines.recipients[ key ].saved_data = recipients_data;
				graphic_lines.opens[ key ].saved_data = opens_data;
				graphic_lines.clicks[ key ].saved_data = clicks_data;
				graphic_lines.unsubscribes[ key ].saved_data = unsubscribes_data;
				read_seconds[ key ].data = read_seconds_data;

				graphic_labels[ key ] = graphic_labels_data;
			}
		}

		vm.show_hide_graphic_legend = function ( legend ) {
			if ( vm.graphic_displays[ legend ] ) {
				hide_graphic_legend( legend );
			}
			else {
				show_graphic_legend( legend );
			}
			if ( vm.show_synthesis_item === 'graphic' ) {
				get_synthesis_graphic();
			}
			else {
				get_synthesis_timeline();
			}
		};

		function show_graphic_legend( legend ) {
			vm.graphic_displays[ legend ] = true;
			graphic_lines[ legend ][ 0 ].data = angular.copy( graphic_lines[ legend ][ 0 ].saved_data );
			graphic_lines[ legend ][ 1 ].data = angular.copy( graphic_lines[ legend ][ 1 ].saved_data );
		}

		function hide_graphic_legend( legend ) {
			vm.graphic_displays[ legend ] = false;
			graphic_lines[ legend ][ 0 ].data = [];
			graphic_lines[ legend ][ 1 ].data = [];
		}

		vm.display_synthesis_graphic = function () {
			vm.show_synthesis_item = 'graphic';
			get_synthesis_graphic();
		};

		vm.display_synthesis_timeline = function () {
			vm.show_synthesis_item = 'timeline';
			get_synthesis_timeline_data();
		};

		function get_synthesis_timeline_data() {
			var data_parameters = {
				'id_campaigns': get_selected_id_campaigns(),
				'selected_date1': vm.filter.selected_date1,
				'selected_date2': vm.filter.selected_date2,
				'segments': get_segments()
			};
			UrlService.post_data( 'jackmail_get_synthesis_timeline', data_parameters, function ( data ) {
				synthesis_timeline = angular.copy( data );
				get_synthesis_timeline();
			}, function () {

			} );
		}

		function get_synthesis_timeline() {
			vm.synthesis_timeline = $rootScope.generate_timeline_data( synthesis_timeline, vm.graphic_displays.opens, vm.graphic_displays.clicks, vm.graphic_displays.unsubscribes );
			vm.nb_synthesis_timeline = vm.synthesis_timeline.length;
		}

		function get_synthesis_graphic() {
			if ( vm.show_item === 'synthesis' ) {
				$timeout( function () {
					var data = [];
					data.push( graphic_lines.recipients[ 0 ] );
					data.push( graphic_lines.opens[ 0 ] );
					data.push( graphic_lines.clicks[ 0 ] );
					data.push( graphic_lines.unsubscribes[ 0 ] );
					var labels = [];
					if ( ! vm.compare ) {
						labels = graphic_labels[ 0 ];
					}
					vm.synthesis_graphic = [ Math.floor( ( Math.random() * 100000 ) + 1 ) ];
					$timeout( function () {
						var graphic_data = {
							labels: labels,
							datasets: data
						};
						Chart.Scale = Chart.Scale.extend( {
							draw: function () {								var helpers = Chart.helpers;
								var each = helpers.each;
								var aliasPixel = helpers.aliasPixel;
								var toRadians = helpers.radians;
								var ctx = this.ctx,
									yLabelGap = ( this.endPoint - this.startPoint ) / this.steps,
									xStart = Math.round( this.xScalePaddingLeft );
								if ( this.display ) {
									ctx.fillStyle = this.textColor;
									ctx.font = this.font;
									each( this.yLabels, function ( labelString, index ) {
										var yLabelCenter = this.endPoint - ( yLabelGap * index ),
											linePositionY = Math.round( yLabelCenter );

										ctx.textAlign = "right";
										ctx.textBaseline = "middle";
										if ( this.showLabels ) {
											ctx.fillText( labelString, xStart - 10, yLabelCenter );
										}
										ctx.beginPath();
										if ( index > 0 ) {
											ctx.lineWidth = this.gridLineWidth;
											ctx.strokeStyle = this.gridLineColor;
										}
										else {
											ctx.lineWidth = this.lineWidth;
											ctx.strokeStyle = this.lineColor;
										}

										linePositionY += helpers.aliasPixel( ctx.lineWidth );

										ctx.moveTo( xStart, linePositionY );
										ctx.lineTo( this.width, linePositionY );
										ctx.stroke();
										ctx.closePath();

										ctx.lineWidth = this.lineWidth;
										ctx.strokeStyle = this.lineColor;
										ctx.beginPath();
										ctx.moveTo( xStart - 5, linePositionY );
										ctx.lineTo( xStart, linePositionY );
										ctx.stroke();
										ctx.closePath();

									}, this );

									each( this.xLabels, function ( label, index ) {
										if ( typeof label === "number" && label % 5 !== 0 ) {
											return;
										}
										var xPos = this.calculateX( index ) + aliasPixel( this.lineWidth ),
											linePos = this.calculateX( index - ( this.offsetGridLines ? 0.5 : 0 ) ) + aliasPixel( this.lineWidth ),
											isRotated = ( this.xLabelRotation > 0 );

										ctx.beginPath();

										if ( index > 0 ) {
											ctx.lineWidth = this.gridLineWidth;
											ctx.strokeStyle = this.gridLineColor;
										}
										else {
											ctx.lineWidth = this.lineWidth;
											ctx.strokeStyle = this.lineColor;
										}
										ctx.moveTo( linePos, this.endPoint );
										ctx.lineTo( linePos, this.startPoint - 3 );
										ctx.stroke();
										ctx.closePath();

										ctx.lineWidth = this.lineWidth;
										ctx.strokeStyle = this.lineColor;

										ctx.beginPath();
										ctx.moveTo( linePos, this.endPoint );
										ctx.lineTo( linePos, this.endPoint + 5 );
										ctx.stroke();
										ctx.closePath();

										ctx.save();
										ctx.translate( xPos, ( isRotated ) ? this.endPoint + 12 : this.endPoint + 8 );
										ctx.rotate( toRadians( this.xLabelRotation ) * - 1 );

										ctx.textAlign = ( isRotated ) ? "right" : "center";
										ctx.textBaseline = ( isRotated ) ? "middle" : "top";

										if ( ! vm.compare ) {
											var i;
											var nb_graphic_labels = labels.length;
											var modulo;
											if ( nb_graphic_labels < 45 ) {
												modulo = 1;
											}
											else if ( nb_graphic_labels < 90 ) {
												modulo = 2;
											}
											else if ( nb_graphic_labels < 180 ) {
												modulo = 3;
											}
											else if ( nb_graphic_labels < 360 ) {
												modulo = 4;
											}
											else if ( nb_graphic_labels < 720 ) {
												modulo = 8;
											}
											else if ( nb_graphic_labels < 1440 ) {
												modulo = 16;
											}
											else {
												modulo = 32;
											}
											if ( index % modulo !== 0 ) {
												label = '';
											}
										}

										ctx.fillText( label, 0, 0 );

										ctx.restore();

									}, this );

								}
							}
						} );
						Chart.types.Line.extend( {
							name: 'LineJackmail',
							calculateXLabelRotation: function () {
								this.xLabelRotation = 0;
							},
							initialize: function ( data ) {
								Chart.types.Line.prototype.initialize.apply( this, arguments );
							}
						} );
						var ctx = document.getElementById( 'jackmail_chartjs_synthesis' ).getContext( '2d' );
						var graphic_options = {
							datasetFill: false,
							responsive: true,
							maintainAspectRatio: false,
							animation: false,
							showTooltips: true,
							tooltipTitleFontSize: 0,
							pointDot: true,
							pointHitDetectionRadius: 0,
							scaleFontSize: 10,
							tooltipFontSize: 12,
							customTooltips: function ( tooltip ) {
								var tooltipEl = angular.element( '#jackmail_chartjs_synthesis_tooltip' );
								if ( ! tooltip ) {
									tooltipEl.css( {
										opacity: 0,
										display: 'none'
									} );
									return;
								}
								tooltipEl.removeClass( 'above below' );
								tooltipEl.addClass( tooltip.yAlign );
								var titles = tooltip.title.split( ' / ' );
								var innerHtml = '';
								var i;
								var label;
								var nb_labels = tooltip.labels.length;
								var total_labels = Object.keys( graphic_lines ).length * 2;
								for ( i = 0; i < nb_labels; i ++ ) {
									if ( i === 0 ) {
										innerHtml += '<b>' + titles[ 0 ];
										innerHtml += ' :</b>';
									}
									if ( tooltip.legendColors[ i ].fill !== 'transparent' ) {
										label = tooltip.labels[ i ].split( ':' );
										innerHtml += [
											'<div class="jackmail_chartjs_synthesis_tooltip-section">',
											'	<span class="jackmail_chartjs_synthesis_tooltip_key" style="background-color:' + tooltip.legendColors[ i ].fill + '"></span>',
											'	<span class="jackmail_chartjs_synthesis_tooltip-value">' +
											'		<span>' + label[ 0 ] + ': </span>' +											'		<span style="color:' + tooltip.legendColors[ i ].fill + '">' + label[ 1 ] + '</span>' +
											'	</span>',
											'</div>'
										].join( '' );
									}
								}
								tooltipEl.html( innerHtml );
								tooltipEl.css( {
									opacity: 1,
									display: 'block',
									left: ( tooltip.chart.canvas.offsetLeft + tooltip.x ) + 'px',
									top: ( tooltip.chart.canvas.offsetTop + tooltip.y - 45 ) + 'px',
									fontFamily: tooltip.fontFamily,
									fontSize: tooltip.fontSize,
									fontStyle: tooltip.fontStyle
								} );
							},
						};
						new Chart( ctx ).LineJackmail( graphic_data, graphic_options );
					} );
				} );
			}
		}

		function refresh_technologies_graphics() {
			var graphics_ids = [ 'jackmail_chartjs_technologies_os', 'jackmail_chartjs_technologies_softwares' ];
			var i;
			var nb_graphics = 2;
			for ( i = 0; i < nb_graphics; i ++ ) {
				if ( i === 0 ) {
					var technologies_details = angular.copy( vm.technologies_os_details );
				}
				else {
					var technologies_details = angular.copy( vm.technologies_softwares_details );
				}
				var ctx = document.getElementById( graphics_ids[ i ] ).getContext( '2d' );
				var data = [];
				var j;
				var nb_details = technologies_details.length;
				for ( j = 0; j < nb_details; j ++ ) {
					data.push( {
						'value': technologies_details[ j ].percent,
						'color': technologies_details[ j ].color,
						'highlight': technologies_details[ j ].color,
						'label': technologies_details[ j ].name
					} );
				}
				var options = {
					segmentShowStroke: false,
					percentageInnerCutout: 75,
					animationSteps: 40,
					animationEasing: 'easeOutQuart',
					animateRotate: true,
					animateScale: false,
					tooltipFontSize: 12,
					customTooltips: function ( tooltip ) {
						var tooltipEl = angular.element( '#jackmail_chartjs_technologies_tooltip' );
						if ( ! tooltip ) {
							tooltipEl.css( {
								opacity: 0
							} );
							return;
						}
						tooltipEl.removeClass( 'above below' );
						tooltipEl.addClass( tooltip.yAlign );
						var left = 21;
						var top = 42;
						var width = 82;
						var text = '';
						var texts = tooltip.text.split( ': ' );
						text = $filter( 'firstUppercaseOthersLowercase' )( texts[ 0 ] + ' :<br/>' + texts[ texts.length - 1 ] ) + ' %';
						tooltipEl.html( text );
						tooltipEl.css( {
							opacity: 1,
							left: tooltip.chart.canvas.offsetLeft + left + 'px',
							top: tooltip.chart.canvas.offsetTop + top + 'px',
							width: width + 'px',
							fontFamily: tooltip.fontFamily,
							fontSize: tooltip.fontSize,
							fontStyle: tooltip.fontStyle
						} );
					}
				};
				new Chart( ctx ).Doughnut( data, options );
			}
		}

		vm.technology_range_by = function ( i ) {
			vm.technologies_grid = $rootScope.grid_service[ 2 ].grid_range( vm.technologies_grid, i );
			vm.nb_technologies_grid = vm.technologies_grid.length
		};

		function get_technologies_data() {
			if ( vm.show_item === 'technologies' ) {
				$timeout( function () {
					vm.technologies_graphic = [ Math.floor( ( Math.random() * 100000 ) + 1 ) ];
					$timeout( function () {
						var data_parameters = {
							'id_campaigns': get_selected_id_campaigns(),
							'selected_date1': vm.filter.selected_date1,
							'selected_date2': vm.filter.selected_date2,
							'segments': get_segments()
						};
						UrlService.post_data( 'jackmail_get_technologies', data_parameters, function ( data ) {
							var i;
							var nb_data = data.operatingSystem_operatingSystemCategory.length;
							var total = 0;
							for ( i = 0; i < nb_data; i ++ ) {
								total += data.operatingSystem_operatingSystemCategory[ i ].count;
							}
							data.total = total;
							technology_data = data;
							generate_technology_grid();
							var technologies_os_details = [];
							var technologies_os_total = {
								'desktop_percent': 0,
								'mobile_percent': 0
							};
							var technologies_softwares_details = [];
							var technologies_softwares_total = {
								'application_percent': 0,
								'webmail_percent': 0
							};
							var type = '';
							var count = 0;
							var color = '';
							var i_type1 = 0;
							var i_type2 = 0;
							var technologies_colors = {
								'type1': [
									'#5A992E', '#7EAE27', '#B5CE59', '#D6E5A2', '#E7F6D9'
								],
								'type2': [
									'#1C3850', '#244674', '#31639E', '#5284C1', '#8FB2D8'
								]
							};
							var i;
							var nb_data = data.operatingSystem_operatingSystemCategory.length;
							for ( i = 0; i < nb_data; i ++ ) {
								type = ( data.operatingSystem_operatingSystemCategory[ i ].type ).toLowerCase();
								count = data.operatingSystem_operatingSystemCategory[ i ].count;
								if ( type === 'desktop' ) {
									technologies_os_total.desktop_percent += count;
									color = technologies_colors[ 'type1' ][ i_type1 ];
									i_type1 ++;
								}
								else {
									technologies_os_total.mobile_percent += count;
									color = technologies_colors[ 'type2' ][ i_type2 ];
									i_type2 ++;
								}
								technologies_os_details.push( {
									'name': os_name( data.operatingSystem_operatingSystemCategory[ i ].name ),
									'type': type,
									'percent': get_percent( count, data.total ),
									'color': color
								} );
							}
							i_type1 = 0;
							i_type2 = 0;
							var nb_data = data.browserGroup_browserCategory.length;
							for ( i = 0; i < nb_data; i ++ ) {
								type = ( data.browserGroup_browserCategory[ i ].type === 'EMAILCLIENT' ? 'application' : 'webmail' );
								count = data.browserGroup_browserCategory[ i ].count;
								if ( type === 'application' ) {
									technologies_softwares_total.application_percent += count;
									color = technologies_colors[ 'type1' ][ i_type1 ];
									i_type1 ++;
								}
								else {
									technologies_softwares_total.webmail_percent += count;
									color = technologies_colors[ 'type2' ][ i_type2 ];
									i_type2 ++;
								}
								technologies_softwares_details.push( {
									'name': data.browserGroup_browserCategory[ i ].name,
									'type': type,
									'percent': get_percent( count, data.total ),
									'color': color
								} );
							}
							vm.technologies_os_total = {
								'desktop_percent': get_percent( technologies_os_total.desktop_percent, data.total ),
								'mobile_percent': get_percent( technologies_os_total.mobile_percent, data.total )
							};
							vm.technologies_softwares_total = {
								'application_percent': get_percent( technologies_softwares_total.application_percent, data.total ),
								'webmail_percent': get_percent( technologies_softwares_total.webmail_percent, data.total )
							};
							vm.technologies_os_details = angular.copy( technologies_os_details );
							vm.technologies_softwares_details = angular.copy( technologies_softwares_details );
							refresh_technologies_graphics();
						}, function () {

						} );
					} );
				} );
			}
		}

		function generate_technology_grid() {
			var json = [];
			var i;
			if ( ( vm.technologies_first_selected.value === 'type_messagerie' && vm.technologies_second_selected.value === 'os' )
				|| ( vm.technologies_first_selected.value === 'os' && vm.technologies_second_selected.value === 'type_messagerie' ) ) {
				var nb_data = technology_data.operatingSystem_browserCategory.length;
				for ( i = 0; i < nb_data; i ++ ) {
					json.push( {
						'os': os_name( technology_data.operatingSystem_browserCategory[ i ].name ),
						'type_messagerie': technology_data.operatingSystem_browserCategory[ i ].type === 'EMAILCLIENT' ? 'APPLICATION' : 'WEBMAIL',
						'openings': technology_data.operatingSystem_browserCategory[ i ].count,
						'percent': get_percent( technology_data.operatingSystem_browserCategory[ i ].count, technology_data.total )
					} );
				}
			}
			else if ( ( vm.technologies_first_selected.value === 'type_messagerie' && vm.technologies_second_selected.value === 'messagerie' )
				|| ( vm.technologies_first_selected.value === 'messagerie' && vm.technologies_second_selected.value === 'type_messagerie' ) ) {
				var nb_data = technology_data.browserGroup_browserCategory.length;
				for ( i = 0; i < nb_data; i ++ ) {
					json.push( {
						'type_messagerie': technology_data.browserGroup_browserCategory[ i ].type === 'EMAILCLIENT' ? 'APPLICATION' : 'WEBMAIL',
						'messagerie': technology_data.browserGroup_browserCategory[ i ].name,
						'openings': technology_data.browserGroup_browserCategory[ i ].count,
						'percent': get_percent( technology_data.browserGroup_browserCategory[ i ].count, technology_data.total )
					} );
				}
			}
			else if ( ( vm.technologies_first_selected.value === 'messagerie' && vm.technologies_second_selected.value === 'os' )
				|| ( vm.technologies_first_selected.value === 'os' && vm.technologies_second_selected.value === 'messagerie' ) ) {
				var nb_data = technology_data.browserGroup_operatingSystem.length;
				for ( i = 0; i < nb_data; i ++ ) {
					json.push( {
						'messagerie': technology_data.browserGroup_operatingSystem[ i ].name,
						'os': os_name( technology_data.browserGroup_operatingSystem[ i ].type ),
						'openings': technology_data.browserGroup_operatingSystem[ i ].count,
						'percent': get_percent( technology_data.browserGroup_operatingSystem[ i ].count, technology_data.total )
					} );
				}
			}
			else if ( ( vm.technologies_first_selected.value === 'messagerie' && vm.technologies_second_selected.value === 'type_device' )
				|| ( vm.technologies_first_selected.value === 'type_device' && vm.technologies_second_selected.value === 'messagerie' ) ) {
				var nb_data = technology_data.browserGroup_operatingSystemCategory.length;
				for ( i = 0; i < nb_data; i ++ ) {
					json.push( {
						'messagerie': technology_data.browserGroup_operatingSystemCategory[ i ].name,
						'type_device': technology_data.browserGroup_operatingSystemCategory[ i ].type,
						'openings': technology_data.browserGroup_operatingSystemCategory[ i ].count,
						'percent': get_percent( technology_data.browserGroup_operatingSystemCategory[ i ].count, technology_data.total )
					} );
				}
			}
			else if ( ( vm.technologies_first_selected.value === 'type_device' && vm.technologies_second_selected.value === 'type_messagerie' )
				|| ( vm.technologies_first_selected.value === 'type_messagerie' && vm.technologies_second_selected.value === 'type_device' ) ) {
				var nb_data = technology_data.browserCategory_operatingSystemCategory.length;
				for ( i = 0; i < nb_data; i ++ ) {
					json.push( {
						'type_device': technology_data.browserCategory_operatingSystemCategory[ i ].type,
						'type_messagerie': technology_data.browserCategory_operatingSystemCategory[ i ].name === 'EMAILCLIENT' ? 'APPLICATION' : 'WEBMAIL',
						'openings': technology_data.browserCategory_operatingSystemCategory[ i ].count,
						'percent': get_percent( technology_data.browserCategory_operatingSystemCategory[ i ].count, technology_data.total )
					} );
				}
			}
			else if ( ( vm.technologies_first_selected.value === 'type_device' && vm.technologies_second_selected.value === 'os' )
				|| ( vm.technologies_first_selected.value === 'os' && vm.technologies_second_selected.value === 'type_device' ) ) {
				var nb_data = technology_data.operatingSystem_operatingSystemCategory.length;
				for ( i = 0; i < nb_data; i ++ ) {
					json.push( {
						'type_device': technology_data.operatingSystem_operatingSystemCategory[ i ].type,
						'os': os_name( technology_data.operatingSystem_operatingSystemCategory[ i ].name ),
						'openings': technology_data.operatingSystem_operatingSystemCategory[ i ].count,
						'percent': get_percent( technology_data.operatingSystem_operatingSystemCategory[ i ].count, technology_data.total )
					} );
				}
			}
			vm.technologies_grid = json;
			vm.nb_technologies_grid = json.length;
		}

		function os_name( os ) {
			os = os.replace( '_', ' ' );
			os = os.replace( 'MACOS', 'MAC OS' );
			return os;
		}

		function get_percent( value, total ) {
			if ( total !== 0 ) {
				return parseFloat( ( value / total * 100 ).toFixed( 2 ) );
			}
			return 0;
		}

		vm.check_or_uncheck_technology_first = function ( value ) {
			vm.technologies_first_selected.value = value;
			var i;
			var nb = vm.technologies_first.length;
			for ( i = 0; i < nb; i ++ ) {
				if ( vm.technologies_first_selected.value === vm.technologies_first[ i ].value ) {
					vm.technologies_first_selected.name = vm.technologies_first[ i ].name;
					break;
				}
			}
			generate_technology_grid();
		};

		vm.check_or_uncheck_technology_second = function ( value ) {
			vm.technologies_second_selected.value = value;
			var i;
			var nb = vm.technologies_second.length;
			for ( i = 0; i < nb; i ++ ) {
				if ( vm.technologies_second_selected.value === vm.technologies_second[ i ].value ) {
					vm.technologies_second_selected.name = vm.technologies_second[ i ].name;
					break;
				}
			}
			generate_technology_grid();
		};

	} ] );

angular.module( 'jackmail.controllers' ).controller( 'TemplateController', [
	'$routeParams', '$rootScope', 'UrlService', 'EmailContentService', '$interval', '$location', 'VerificationService', '$timeout',
	function ( $routeParams, $rootScope, UrlService, EmailContentService, $interval, $location, VerificationService, $timeout ) {

		var vm = this;

		EmailContentService.init_emailbuilder();

		$rootScope.active_item_menu( 8 );

		vm.url_id = $routeParams.id;
		vm.gallery_id = $routeParams.gallery_id;

		vm.template = {};
		var saved_template = {};

		vm.name_editing = false;

		vm.current_content_email_type = 'emailbuilder';

		vm.show_name_popup = vm.url_id === '0' ? true : false;

		vm.hide_name_popup = function () {
			vm.show_name_popup = false;
		};

		vm.focus_template_name = function () {
			if ( vm.template.name === 'Template sans nom' || vm.template.name === 'Template with no name' ) {
				vm.template.name = '';
			}
			angular.element( '.jackmail_name .jackmail_content_editable' ).focus();
			vm.name_editing = true;
		};

		vm.blur_template_name = function () {
			if ( vm.template.name === '' ) {
				vm.template.name = $rootScope.translations.template_with_no_name;
			}
			vm.name_editing = false;
		};

		function get_template_data() {
			if ( ! $rootScope.settings.emailbuilder_installed ) {
				$rootScope.go_page( 'templates' );
			}
			if ( vm.url_id === '0' && vm.gallery_id !== '0' ) {
				var data_parameters = {
					'gallery_id': vm.gallery_id,
					'link_tracking': '1'
				};
				UrlService.post_data( 'jackmail_get_gallery_template_json', data_parameters, function ( data ) {
					var content = EmailContentService.init_and_display_emailbuilder( data.content_email_json );
					vm.template = {
						'id': '0',
						'name': $rootScope.translations.template_with_no_name,
						'content_email_json': content.content_email_json,
						'content_email_html': '',
						'content_email_txt': ''
					};
					$rootScope.select_name_popup( vm.show_name_popup );
					saved_template = angular.copy( vm.template );
				}, function () {

				} );
			}
			else {
				var data_parameters = {
					'id': vm.url_id
				};
				UrlService.post_data( 'jackmail_get_template', data_parameters, function ( data ) {
					if ( data === null ) {
						$rootScope.go_page( 'templates' );
					}
					EmailContentService.init_and_display_emailbuilder( data.content_email_json );
					vm.template = data;
					$rootScope.select_name_popup( vm.show_name_popup );
					saved_template = angular.copy( vm.template );
				}, function () {

				} );
			}
		}

		get_template_data();

		$interval( function () {
			check_save_template_needed();
		}, 300000 );

		function check_save_template_needed() {
			if ( ! $rootScope.show_help2 && $rootScope.nb_loading === 0 ) {
				refresh_content_email();
				if ( vm.url_id !== '0' || vm.template.name !== $rootScope.translations.template_with_no_name || vm.template.content_email_changes ) {
					if ( VerificationService.differents_arrays( vm.template, saved_template ) ) {
						vm.save_template( false );
					}
				}
			}
		}

		vm.save_template = function ( refresh_emailbuilder_changes ) {
			if ( refresh_emailbuilder_changes ) {
				refresh_content_email();
			}
			var url_create = 'jackmail_create_template';
			var url_update = 'jackmail_update_template';
			if ( vm.url_id === '0' ) {
				UrlService.post_data( url_create, vm.template, function ( data ) {
					if ( data.success ) {
						vm.url_id = data.id;
						vm.template.id = data.id;
						if ( vm.template.content_email_json !== data.content_email_json ) {
							vm.template.content_email_json = data.content_email_json;
							EmailContentService.set_emailbuilder_json( vm.c_common.campaign.content_email_json );
						}
						change_url();
					}
					$rootScope.display_success_error_writable( data.success, $rootScope.translations.the_template_was_saved );
				}, function () {

				} );
			}
			else {
				UrlService.post_data( url_update, vm.template, function ( data ) {
					if ( data.success ) {
						if ( vm.template.content_email_json !== data.content_email_json ) {
							vm.template.content_email_json = data.content_email_json;
							EmailContentService.set_emailbuilder_json( vm.template.content_email_json );
						}
					}
					$rootScope.display_success_error_writable( data.success, $rootScope.translations.the_template_was_saved );
				}, function () {

				} );
			}
			saved_template = angular.copy( vm.template );
		};

		function refresh_content_email() {
			var content = EmailContentService.refresh_content_email( vm.current_content_email_type, vm.template.content_email_json, vm.template.content_email_html, vm.template.content_email_txt );
			if ( content.content_email_changes ) {
				vm.template.content_email_changes = content.content_email_changes;
				vm.template.content_email_json = content.content_email_json;
				vm.template.content_email_html = content.content_email_html;
				vm.template.content_email_txt = content.content_email_txt;
			}
		}

		function change_url() {
			var url = '/template/' + vm.url_id + '/0';
			if ( $location.path() !== url ) {
				UrlService.change_url_parameters_without_reload_and_history( url );
			}
		}

		vm.reset_emailbuilder_content = function () {
			$rootScope.display_validation( $rootScope.translations.you_will_loose_your_emailbuilder_content, function () {
				EmailContentService.set_emailbuilder_json( '' );
				refresh_content_email();
			} );
		};

	} ] );

angular.module( 'jackmail.controllers' ).controller( 'TemplatesController', [
	'UrlService', '$rootScope',
	function ( UrlService, $rootScope ) {

		var vm = this;

		vm.current_page = 'templates_page';
		if ( window.location.href.indexOf( 'jackmail_campaign' ) !== - 1 || window.location.href.indexOf( 'jackmail_scenario' ) !== - 1 ) {
			vm.current_page = 'campaign_page';
		}

		vm.templates_type = [
			$rootScope.translations.my_templates,
			$rootScope.translations.templates_gallery
		];

		var templates_all = [];
		var templates = [];
		var nb_templates = - 1;

		var templates_gallery_all = [];
		var templates_gallery = [];
		var nb_templates_gallery = - 1;

		vm.templates_grid = [];
		vm.nb_templates_grid = - 1;

		vm.displayed_templates = {
			'templates': false,
			'templates_gallery': false
		};

		vm.templates_gallery_categories = [];
		vm.selected_templates_gallery_category = '';

		vm.templates_search = '';

		function get_templates() {
			UrlService.post_data( 'jackmail_get_templates', {}, function ( data ) {
				templates_all = data;
				if ( vm.current_page === 'templates_page' ) {
					templates_all.unshift( { 'name': '' } );
				}
				nb_templates = templates_all.length;
				templates = angular.copy( templates_all );
				if ( nb_templates > 1 || vm.current_page === 'campaign_page' ) {
					vm.display_templates();
				}
				else {
					vm.display_templates_gallery();
				}
			}, function () {

			} );
		}

		if ( vm.current_page === 'templates_page' ) {
			get_templates();
		}

		$rootScope.get_templates_from_campaign_page = function ( templates_type ) {
			if ( templates_type === 'templates' ) {
				vm.displayed_templates = {
					'templates': true,
					'templates_gallery': false
				};
				if ( nb_templates === - 1 ) {
					get_templates();
				}
				else {
					generate_templates_grid();
				}
			}
			else {
				vm.displayed_templates = {
					'templates': false,
					'templates_gallery': true
				};
				if ( nb_templates_gallery === - 1 ) {
					get_templates_gallery();
				}
				else {
					generate_templates_gallery_grid();
				}
			}
		};

		function get_templates_gallery() {
			UrlService.post_data( 'jackmail_get_templates_gallery', {}, function ( data ) {
				vm.templates_gallery_categories = get_templates_gallery_categories( data );
				templates_gallery_all = data;
				if ( vm.current_page === 'templates_page' ) {
					templates_gallery_all.unshift( { 'name': '' } );
				}
				nb_templates_gallery = templates_gallery_all.length;
				templates_gallery = angular.copy( templates_gallery_all );
				generate_templates_gallery_grid();
			}, function () {

			} );
		}

		function get_templates_gallery_categories( data ) {
			var i;
			var nb_templates = data.length;
			var templates_gallery_categories = [];
			for ( i = 0; i < nb_templates; i ++ ) {
				var category = data[ i ].category;
				if ( templates_gallery_categories.indexOf( category ) === - 1 ) {
					templates_gallery_categories.push( category );
				}
			}
			templates_gallery_categories.sort();
			templates_gallery_categories.unshift( $rootScope.translations.all_categories );
			return templates_gallery_categories;
		}

		function generate_templates_grid() {
			search_on_templates();
			vm.templates_grid = $rootScope.generate_grid( templates );
			vm.nb_templates_grid = vm.templates_grid.length;
		}

		function generate_templates_gallery_grid() {
			search_on_templates();
			select_gallery_template_category();
			vm.templates_grid = $rootScope.generate_grid( templates_gallery );
			vm.nb_templates_grid = vm.templates_grid.length;
		}

		vm.display_templates_or_templates_gallery = function ( key ) {
			if ( key === 0 ) {
				vm.display_templates();
			}
			else {
				vm.display_templates_gallery();
			}
		};

		vm.display_templates = function () {
			vm.displayed_templates = {
				'templates': true,
				'templates_gallery': false
			};
			if ( nb_templates === - 1 ) {
				get_templates();
			}
			else {
				generate_templates_grid();
			}
		};

		vm.display_templates_gallery = function () {
			vm.displayed_templates = {
				'templates': false,
				'templates_gallery': true
			};
			if ( nb_templates_gallery === - 1 ) {
				get_templates_gallery();
			}
			else {
				generate_templates_gallery_grid();
			}
		};

		vm.select_gallery_template_category = function ( key, title ) {
			if ( title === $rootScope.translations.all_categories ) {
				vm.selected_templates_gallery_category = '';
			}
			else {
				vm.selected_templates_gallery_category = title;
			}
			generate_templates_gallery_grid();
		};

		function select_gallery_template_category() {
			if ( vm.selected_templates_gallery_category !== '' ) {
				var i;
				var nb_templates = templates_gallery.length;
				var data = [];
				for ( i = 0; i < nb_templates; i ++ ) {
					if ( i === 0 && vm.current_page === 'templates_page' ) {
						data.push( templates_gallery[ i ] );
					}
					else {
						var category = templates_gallery[ i ].category;
						if ( category === vm.selected_templates_gallery_category ) {
							data.push( templates_gallery[ i ] );
						}
					}
				}
				templates_gallery = angular.copy( data );
			}
		}

		function search_on_templates() {
			var i;
			var new_data = [];
			if ( vm.displayed_templates.templates ) {
				var nb_templates = templates_all.length;
				var data = templates_all;
			}
			else {
				var nb_templates = templates_gallery_all.length;
				var data = templates_gallery_all;
			}
			for ( i = 0; i < nb_templates; i ++ ) {
				if ( i === 0 && vm.current_page === 'templates_page' ) {
					new_data.push( data[ i ] );
				}
				else if ( ( data[ i ].name ).indexOf( vm.templates_search ) !== - 1 ) {
					new_data.push( data[ i ] );
				}
			}
			if ( vm.displayed_templates.templates ) {
				templates = angular.copy( new_data );
			}
			else {
				templates_gallery = angular.copy( new_data );
			}
		}

		vm.search_on_templates = function () {
			if ( vm.displayed_templates.templates ) {
				generate_templates_grid();
			}
			else {
				generate_templates_gallery_grid();
			}
		};

		vm.create_new_template = function () {
			if ( $rootScope.settings.emailbuilder_installed ) {
				$rootScope.change_page_with_parameters( 'template', '0/0' );
			}
			else {
				$rootScope.display_emailbuilder_popup();
			}
		};

		vm.edit_template = function ( id ) {
			if ( $rootScope.settings.emailbuilder_installed ) {
				$rootScope.change_page_with_parameters( 'template', id + '/0' );
			}
			else {
				$rootScope.display_emailbuilder_popup();
			}
		};

		vm.import_gallery_template = function ( id ) {
			if ( $rootScope.settings.emailbuilder_installed ) {
				$rootScope.change_page_with_parameters( 'template', '0/' + id );
			}
			else {
				$rootScope.display_emailbuilder_popup();
			}
		};

		vm.duplicate_template = function ( id ) {
			var data_parameters = {
				'id': id
			};
			UrlService.post_data( 'jackmail_duplicate_template', data_parameters, function ( data ) {
				get_templates();
				$rootScope.display_success_error_writable( data.success, $rootScope.translations.the_template_as_been_duplicated );
			}, function () {

			} );
		};

		vm.delete_confirm = function ( key, subkey ) {
			vm.templates_grid[ key ][ subkey ].show_delete_confirmation = true;
		};

		vm.delete_cancel = function ( key, subkey ) {
			vm.templates_grid[ key ][ subkey ].show_delete_confirmation = false;
		};

		vm.delete_template = function ( id ) {
			var data_parameters = {
				'id': id
			};
			UrlService.post_data( 'jackmail_delete_template', data_parameters, function ( data ) {
				$rootScope.display_success_error_writable( data.success, $rootScope.translations.the_template_was_deleted );
				get_templates();
			}, function () {

			} );
		};

		vm.display_actions = function ( key, subkey ) {
			vm.templates_grid[ key ][ subkey ].show_actions = true;
		};

		vm.hide_actions = function ( key, subkey ) {
			vm.templates_grid[ key ][ subkey ].show_actions = false;
		};

	} ] );